.intel_syntax noprefix

.data
	k:	.float	0.60725293500888144482 # Konstanta k.
		
.text
.global atans, cordic

	## Funkcija atans() racuna tabelu inverznih tangensa od 1/2^i
	## (za i=0,...,n-1). Argumenti funkcije su:
	##   [ebp+8] - duzina tabele
	##   [ebp+12] - pokazivac na pocetak tabele
	## Pretpostavlja se da je duzina tabele veca od 0. Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
atans:
        ## Prolog funkcije.
        enter	0, 0

	## Adresa tekuceg elementa tabele drzati u registru esi, dok ce
	## se registar ecx koristiti kao brojac.
	mov	esi, [ebp+12]
	mov	ecx, [ebp+8]

	## Inicijalizuje se vrijednost 2^i.
	fld1			# Stek:	1
	fld1			# Stek:	1, 1=2^0
	
next_element:
	## Izracunava se arctg(1/2^i).
	fld	st		# Stek:	1, 2^i, 2^i
	fld1			# Stek:	1, 2^i, 2^i, 1
	fxch			# Stek:	1, 2^i, 1, 2^i
	fpatan			# Stek:	1, 2^i, arctg(1/2^i)

	## Smjesta se izracunata vrijednost u tabelu i nastavlja se sa
	## izracunavanjem.
	fstp	dword ptr [esi]	# Stek:	1, 2^i
	add	esi, 4
	fscale			# Stek:	1, 2^(i+1)
	loop next_element

	## Prazni se stek.
	ffree st
	fincstp
	ffree st
	fincstp
	
        ## Epilog funkcije.
        leave
        ret

	## Funkcija cordic() izracunava sinus i kosinus datog ugla
	## CORDIC algoritmom. Argumenti funkcije su:
	##   [ebp+8] - dati ugao
	##   [ebp+12] - broj iteracija
	##   [ebp+16] - tabela vrijednosti inverznih tangensa
	
	##   [ebp+20], [ebp+24] - pokazivaci na lokacije na koje treba
        ##                        smjestiti sinus odn. kosinus
	
	## Pretpostavlja se da je ugao u intervalu (-pi/2,pi/2), kao i
	## da je broj iteracija veci od 0. Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
cordic:
        ## Prolog funkcije.
        enter	0, 0
	push	ebx

	## Registar ecx ce biti koriscen kao brojac, dok ce se adresa
	## tekuceg elementa tabele inverznih tangensa drzati u registru
	## esi.
	mov	ecx, [ebp+12]
	mov	esi, [ebp+16]
	
	## Incijalizuju se velicine koje se koriste u algorimu. U
	## komentarima, tekuca iteracije ce biti oznacena brojem u
	## zagradama. Za vrijednost sinusa ce se koristiti oznaka s, za
	## vrijednost kosinusa c, a za pomocnu promjenljivu d.
	fldz				# Stek:	0=s(0)
	fld	k			# Stek:	s(0), k=c(0)
	fld	dword ptr [ebp+8]	# Stek: s(0), c(0), theta=d(0)
	fldz				# Stek:	s(0), c(0), d(0), 0=-i
	
next_iteration:
	## Poredi se pomocna promjenljiva d(i) sa nulom.
	fldz			# Stek:	s(0), c(0), d(0), -i, 0
	fcomip	st, st(2)	# Stek:	s(0), c(0), d(0), -i
	jb greater_than_zero

	## U slucaju da je d(i)<=0, primjenjuju se odgovarajuce formule
	## za izracunavanje s(i+1), c(i+1) i d(i+1).
	fld	st(2)		# Stek:	s(i), c(i), d(i), -i, c(i)
	fscale			# Stek:	s(i), c(i), d(i), -i, c(i)/2^i
	fld	st(4)		# Stek:	s(i), c(i), d(i), -i, c(i)/2^i, s(i)
	fxch			# Stek:	s(i), c(i), d(i), -i, s(i), c(i)/2^i
	fsubp	st(5), st	# Stek:	s(i)-c(i)/2^i=s(i+1), c(i), d(i), -i, s(i)
	fscale			# Stek:	s(i+1), c(i), d(i), -i, s(i)/2^i
	faddp	st(3), st	# Stek:	s(i+1), c(i)+s(i)/2^i=c(i+1), d(i), -i
	fld	dword ptr [esi]	# Stek:	s(i+1), c(i+1), d(i), -i, arctg(1/2^i)
	add	esi, 4
	faddp	st(2)		# Stek:	s(i+1), c(i+1), d(i)+arctg(1/2^i)=d(i+1) -i
	jmp continue
	
greater_than_zero:
	## Opet - u slucaju da je d(i)>0, primjenjuju se odgovarajuce
	## formule za izracunavanje s(i+1), c(i+1) i d(i+1). Desavanja
	## na steku su ekvivalnetna onim iz druge grane.
	fld	st(2)
	fscale
	fld	st(4)
	fxch
	faddp	st(5), st
	fscale
	fsubp	st(3), st
	fld	dword ptr [esi]
	add	esi, 4
	fsubp	st(2)

continue:
	## Dekrementira se faktor skaliranja i nastavlja se sa
	## iteracijama.
	fld1			# Stek:	s(i+1), c(i+1), d(i+1), -i, 1
	fsubp			# Stek:	s(i+1), c(i+1), d(i+1), -(i+1)
	loop next_iteration

	## Skidaju se nepotrebne vrijednosti sa vrha steka.
	ffree	st		# Stek:	s(n), c(n), d(n), invalidna vrijednost
	fincstp			# Stek:	s(n), c(n), d(n)
	ffree	st		# Stek:	s(n), c(n), invalidna vrijednost
	fincstp			# Stek:	s(n), c(n)

	## Smestaju se izracunate vrijednosti na odgovarajuce lokacije.
	mov	ebx, [ebp+24]
	fstp	dword ptr [ebx]	# Stek:	s(n)
	mov	ebx, [ebp+20]
	fstp	dword ptr [ebx]

	## Epilog funkcije.
	pop	ebx
        leave
        ret
