.intel_syntax noprefix

.bss
.lcomm  pad, 8                  # Bajtovi koji se nadodaju na kraj stringa.
        
.text
.global crc

        ## Funkcija crc() racuna CRC vrijednost datog stringa. Argumenti
	## funkcije su:
        ##   [ebp+8] - adresa stringa
        ## Funkcija vraca izracunatu CRC vrijednost.  Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
crc:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Postavlja se direction fleg na odgovarajucu vrijednost.
        cld

        ## Izracunava se duzina datog stringa tako sto se pretrazuje
	## string dok se ne naidje na bajt jednak 0.  Duzina stringa se
	## smjesta u registar ecx.
        mov     edi, [ebp+8]
        xor     eax, eax
        mov     ecx, -1
        repnz scasb
        add     ecx, 2
        neg     ecx

        ## Stavlja se duzina stringa i 4 bajta jednaka 0 u niz bajtova
	## kojima se dopunjava string pri racunanju CRC-a.  Treba uociti da
        ## ce, obzirom da je IA-32 arhitektura little-endian tipa,
	## bajtovi u duzini stringa koji su razliciti od 0 doci na pocetak
	## ovog niza bajtova, dok ce ostatak niza biti jednak 0.
        mov     [pad], ecx
        mov     dword ptr [pad+4], 0

        ## CRC vrijednost ce biti drzana u registru edx.
        xor     edx, edx

        ## Provjerava se da li je string prazan i, ako jeste, odmah se
	## prelazi na procesiranje dodatnih bajtova.
        jecxz process_pad

        ## Procesira se string.  Registar esi sadrzi adresu tekuceg
	## bajta, registar ecx broj preostalih bajtova.
        mov     esi, [ebp+8]
next_message_byte:
        ## Ucitava se tekuci bajt.
        lodsb

        ## CRC vrijednost se siftuje za jedan bit ulijevo, a na bit 
        ## najmanje tezine se uvodi tekuci bit tekuceg bajta.  Ukoliko
	## je bit koji je istisnut iz CRC vrijednosti jednak 1,
	## novodobijena CRC vrijednost se xor-uje sa datim brojem.
	## Registar ecx sluzi kao brojac bitova, dok se njegova tekuca
        ## vrijednost (u smislu broja preostalih bajtova poruke)
	## privremeno cuva u registru ebx.
        mov     ebx, ecx
        mov     ecx, 8
next_message_bit:
        shl     al, 1
        rcl     edx
        jnc message_bit_applied
        xor     edx, 0x04c11db7
message_bit_applied:
        loop next_message_bit

        ## Prelazi se na naredni bajt poruke.
        mov     ecx, ebx
        loop    next_message_byte

process_pad:
        ## Broji se koliko bajtova treba procesirati u dodatnom nizu
        ## bajtova.  Polazi se od bajta najvece tezine duzine niza i
	## broji koliko ima bajtova razlicitih od 0, da bi se potom taj broj
	## oduzeo od 8, kolika je duzina dodatnog niza bajtova.
        std
        lea     edi, pad+3
        xor     eax, eax
        mov     ecx, -1
        repz scasb
        add     ecx, 2
        neg     ecx
        mov     eax, ecx
        mov     ecx, 8
        sub     ecx, eax

        ## Na potpuno isti nacin kao sto je procesiran string, procesira
	## se i niz bajtova nadodat na poruku.
        cld        
        lea     esi, pad
next_pad_byte:
        lodsb
        mov     ebx, ecx
        mov     ecx, 8
next_pad_bit:
        shl     al, 1
        rcl     edx
        jnc pad_bit_applied
        xor     edx, 0x04c11db7
pad_bit_applied:
        loop next_pad_bit
        mov     ecx, ebx
        loop    next_pad_byte

        ## Invertuje se dobijena CRC vrijednost i vraca kao rezultat
	## funkcije preko registra eax.
        not     edx
        mov     eax, edx
        
        ## Epilog funkcije.
        pop     ebx
        pop     edi
        pop     esi
        leave
        ret
