.intel_syntax noprefix

.text
.global distance2

	## Funkcija distance2() racuna kvadrat rastojanja izmedju dvije
	## tacke date odredjenim brojem koordinata.  Argumenti funkcije su:
	##   [ebp+8] - broj koordinata
	##   [ebp+12] - polje sa koordinatama prve tacke
        ##   [ebp+16] - polje sa koordinatama druge tacke
        ## Broj koordinata mora biti veci od 0. Funkcija vraca
        ## izracunatu vrijednost kvadrata rastojanja.  Funkcija se u 
	## potpunosti podvrgava C konvencijama pozivanja.
distance2:
        ## Prolog funkcije.
        enter	0, 0
        push    esi
        push    edi

        ## Registar esi ce drzati adresu prve koordinate prve tacke, a
	## registar edi adresu prve koordinate druge tacke.
        mov     esi, [ebp+12]
        mov     edi, [ebp+16]

        ## U registar edx se prebacuje broj koordinata.  Potom se u
	## registru eax izracunava ostatak pri dijeljenju toga broja sa 4 i
	## oduzima se taj broj od registra edx.  Na taj nacin, u registru
	## edx ostaje broj koordinata koje se mogu procesirati paralelno u
	## grupama po 4, dok u registru eax ostaje broj preostalih
	## koordinata.
        mov     edx, [ebp+8]
        mov     eax, edx
        shr     eax, 2
        shl     eax, 2
        sub     eax, edx
        neg     eax
        sub     edx, eax

        ## Gornja i donja polovina registra mm0 ce sadrzati parcijalne
	## sume kvadrata razlika koordinata.  Ove sume se inicijalizuju
	## na 0.
        pxor    mm0, mm0

        ## Registar ecx ce se koristiti kao indeks tekucih koordinata.
        xor     ecx, ecx
        
next_four_coords:
        ## Provjerava se da li se doslo do kraja petlje.
        cmp     ecx, edx
        je remaining_coords
        
        ## Ucitavaju se 4 koordinate prve tacke, zatim se od njih
	## oduzimaju 4 odgovarajuce koordinate druge tacke i potom se vrsi
        ## kvadriranje razlika i sumiranje kvadrata parova razlika
        ## susjednih koordinata.  Ovako dobijene sume se dodaju na
	## odgovarajuce parcijalne sume u registru mm0.
        movq    mm1, [esi+2*ecx]
        psubsw  mm1, [edi+2*ecx]
        pmaddwd mm1, mm1
        paddd   mm0, mm1

        ## Prelazi se na narednu grupu od 4 koordinate.
        add     ecx, 4
        jmp next_four_coords

remaining_coords:
        ## U registar edx se prebacuje broj preostalih koordinata
        mov     edx, eax

        ## Azuriraju se registri esi i edi tako da pokazuju na pocetak
	## grupe preostalih koordinata.
        shl     ecx, 1
        add     esi, ecx
        add     edi, ecx

        ## Preostale koordinate ce biti ucitane u registre mm2 i mm3,
	## koji se inicijalizuju na 0
        pxor    mm2, mm2
        pxor    mm3, mm3

        ## Registar ecx ce se opet koristiti kao indeks tekucih
	## koordinata.
        xor     ecx, ecx
next_coord:
        ## Ucitavaju se preostale koordinate preko registra ax (nije
        ## moguc pojedinacni 16-tobitni transfer u MMX registre) u
        ## registar mm1, a onda se nadodaju na ranije ucitane preostale
	## koordinate u registrima mm2 odnosno mm3.
        xor     eax, eax
        mov     ax, word ptr [esi+2*ecx]
        movd    mm1, eax
        psllq   mm2, 16
        paddw   mm2, mm1
        xor     eax, eax
        mov     ax, word ptr [edi+2*ecx]
        movd    mm1, eax
        psllq   mm3, 16
        paddw   mm3, mm1
        inc     ecx
        cmp     ecx, edx
        jb next_coord

        ## Na isti nacin kao gore, formiraju se sume kvadrata razlika
        ## susjednih koordinata za preostale koordinate, a onda se
	## dodaju te sume u odgovarajuce parcijalne sume u registru mm0.
        psubsw  mm2, mm3
        pmaddwd mm2, mm2
        paddd   mm0, mm2

        ## Sabiraju se donja i gornja polovina registra mm0, dajuci na
	## taj nacin konacan rezultat, koji se potom prebacuje u registar
	## eax.
        movq    mm1, mm0
        psrlq   mm1, 32
        paddd   mm0, mm1     
        movd    eax, mm0

        ## Naznacava se da je zavrseno sa koriscenjem MMX instrukcija.
        emms
        
        ## Epilog funkcije.
        pop     edi        
        pop     esi
        leave
        ret
