.intel_syntax noprefix

.text
.global intersection

	## Funkcija intersection() odredjuje da li se dvije prave u
        ## ravni odredjene datim tackama sijeku (ako se prave poklapaju,
	## uzima se da se sijeku).  Argumenti funkcije su:
	##   [ebp+8] - adresa prve koordinate prve tacke prve prave
        ##   [ebp+12] - adresa prve koordinate druge tacke prve prave
        ##   [ebp+16] - adresa prve koordinate prve tacke druge prave
        ##   [ebp+20] - adresa prve koordinate druge tacke druge prave
        ## Pretpostavlja se da se tacke iz parova koji odredjuju prave
	## ne poklapaju.  Funkcija vraca 1 ukoliko presjek postoji, a 0
	## inace. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
intersection:
        ## Prolog funkcije.
        enter	0, 0
        push    esi
        push    edi

        ## Postoji vise nacina da se rijesi dati problem, jedan od njih
	## (donekle formalizovan, ali u sustini iste kompleksnosti kao
        ## i ostali) bi bio sledeci. Ukoliko se napisu parametarske
        ## jednacine (respektivno po parametrima s i t) pravih
        ## odredjenih parovima datih tacaka (p0,p1) odn. (p2,p3), dolazi
        ## se do sistema jednacina po parametrima s i t oblika:
        ##   (p1[X]-p0[X])*s-(p3[X]-p2[X])*t=p2[X]-p0[X]
        ##   (p1[Y]-p0[Y])*s-(p3[Y]-p2[Y])*t=p2[Y]-p0[Y]
        ## Ovaj sistem ce biti rjesavan po parametru s koriscenjem
        ## Kramerovog pravila i na osnovu toga ce biti moguce zakljuciti
        ## da li se prave sijeku.

        ## Na stek se unose koeficijenti prve jednacine sistema.
        mov     esi, [ebp+20]
        fld     [esi]           # Stek: p3[X]
        mov     esi, [ebp+12]
        fld     [esi]           # Stek: p3[X], p1[X]
        mov     esi, [ebp+16]
        fld     [esi]           # Stek: p3[X], p1[X], p2[X]
        fsubr   st(2), st       # Stek: p2[X]-p3[X]=a01, p1[X], p2[X]
        mov     esi, [ebp+8]
        fld     [esi]           # Stek: a01, p1[X], p2[X], p0[X]
        fsub    st(2), st       # Stek: a01, p1[X]-p0[X]=a00, p2[X], p0[X]
        fsubp                   # Stek: a01, a00, p2[X]-p0[X]=b0

        ## Na stek se unose koeficijenti druge jednacine sistema.
        mov     esi, [ebp+20]
        fld     [esi+4]         # Stek: a01, a00, b0, p3[Y]
        mov     esi, [ebp+12]
        fld     [esi+4]         # Stek: a01, a00, b0, p3[Y], p1[Y]
        mov     esi, [ebp+16]
        fld     [esi+4]         # Stek: a01, a00, b0, p3[Y], p1[Y], p2[Y]
        fsubr   st(2), st       # Stek: a01, a00, b0, p2[Y]-p3[Y]=a11, p1[Y], p2[Y]
        mov     esi, [ebp+8]
        fld     [esi+4]         # Stek: a01, a00, b0, a11, p1[Y], p2[Y], p0[Y]
        fsub    st(2), st       # Stek: a01, a00, b0, a11, p1[Y]-p0[Y]=a10, p2[Y], p0[Y]
        fsubp                   # Stek: a01, a00, b0, a11, a10, p2[Y]-p0[Y]=b1

        ## Izracunava se determinanta po promjenljivoj s.
        fxch    st(3)           # Stek: a01, a00, b1, a11, a10, b0
        fmul    st(2)           # Stek: a01, a00, b1, a11, a10, b0*a11
        fxch    st(3)           # Stek: a01, a00, b0*a11, a11, a10, b1
        fmul    st, st(5)       # Stek: a01, a00, b0*a11, a11, a10, b1*a01
        fsubp   st(3), st       # Stek: a01, a00, b0*a11-b1*a01=Ds, a11, a10

        ## Izracunava se determinanta sistema.
        fmulp   st(4), st       # Stek: a01*a10, a00, Ds, a11
        fmulp   st(2), st       # Stek: a01*a10, a00*a11, Ds
        fxch    st(2)           # Stek: Ds, a00*a11, a01*a10
        fsubp                   # Stek: Ds, a00*a11-a01*a10=D

        ## Ispituje se da li je determinanta sistema jednaka 0.
        fldz                    # Stek: Ds, D, 0
        fcomip  st, st(1)       # Stek: Ds, D
        ffree   st              # Stek: Ds, invalidna vrijednost
        fincstp                 # Stek: Ds
        jne single_or_multiple_intersections

        ## Ispituje se da li je determinanta po s jednaka 0.
        fldz                    # Stek: Ds, 0
        fcomip  st, st(1)       # Stek: Ds
        ffree   st              # Stek: invalidna vrijednost
        fincstp                 # Stek:
        jne no_intersection

single_or_multiple_intersections:
        ## Vraca se 0 kao rezultat funkcije.
        mov     eax, 1
        jmp     finish

no_intersection:
        ## Vraca se 0 kao rezultat funkcije.
        xor     eax, eax
        
finish: 
        ## Epilog funkcije.
        pop     edi
        pop     esi
        leave
        ret
