.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.
        
.text
.global gauss_jordan

	## Funkcija gauss_jordan() rjesava dati sistem od 3 linearne
        ## jednacine sa 3 nepoznate.  Argumenti funkcije su:
        ##   [ebp+8] - matrica koeficijenata lijeve strane sistema
        ##   [ebp+12] - vektor koeficijenata desne strane sistema
        ##   [ebp+16] - vektor u koji treba upisati rjesenja sistema
        ## Pretpostavlja se da sistem ima tacno jedno rjesenje.
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
gauss_jordan:
        ## Prolog funkcije.
	enter   16, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Postavlja se direction fleg.
        cld

        ## Preko pomocnog niza, u registre xmm0, xmm1 i xmm2 se
        ## upisuju koeficijenti lijevih i desnih strana jednacina u
        ## sistemu (svaki registar na pozicijama manje tezine sadrzi
        ## koeficijente lijeve strane date jednacine, a na poziciji
        ## najvece tezine koeficijente desne strane).  Inicijalizuju se
	## registri esi i ebx, koji ce sadrzati adrese tekucih elemenata
        ## koeficijenata lijeve strane sistema odn. koeficijenta desne
        ## strane sistema.
        mov     esi, [ebp+8]
        mov     ebx, [ebp+12]

        ## Upisuju se u registar xmm0 koeficijenti prve jednacine.
        mov     ecx, 3
        lea     edi, [ebp-16]
        rep movsd
        xchg    esi, ebx
        movsd
        xchg    esi, ebx
        movups  xmm0, [ebp-16]

        ## Upisuju se u registar xmm1 koeficijenti druge jednacine.
        mov     ecx, 3
        lea     edi, [ebp-16]
        rep movsd
        xchg    esi, ebx
        movsd
        xchg    esi, ebx
        movups  xmm1, [ebp-16]

        ## Upisuju se u registar xmm2 koeficijenti trece jednacine.
        mov     ecx, 3
        lea     edi, [ebp-16]
        rep movsd
        xchg    esi, ebx
        movsd
        xchg    esi, ebx
        movups  xmm2, [ebp-16]

        ## Dijele se koeficijenti prve jednacine koeficijentom uz x[0] u
	## toj jednacini, cime isti koeficijent postaje jednak 1.  Ako
        ## je inicijalno ovaj koeficijent jednak 0, prethodno se
        ## zamjenjuju prva i druga ili prva i treca jednacina, kako bi
	## vrijednost ovog koeficijenta bila razlicita od 0.
        comiss  xmm0, zero
        jne scale_1st_equation_coefficients
        comiss  xmm1, zero
        je swap_1st_and_3rd_equation
        movaps  xmm4, xmm0
        movaps  xmm0, xmm1
        movaps  xmm1, xmm4
        jmp scale_1st_equation_coefficients
swap_1st_and_3rd_equation:        
        movaps  xmm4, xmm0
        movaps  xmm0, xmm2
        movaps  xmm2, xmm4
scale_1st_equation_coefficients:
        movaps  xmm4, xmm0
        shufps  xmm4, xmm4, 0x00
        divps   xmm0, xmm4

        ## Oduzimanjem prve jednacine pomnozene odgovarajucom
        ## vrijednoscu od druge odn. trece jednacine postize se da
        ## koeficijenti uz x[0] u ove dvije jednacine postanu jednaki 0.
        movaps  xmm4, xmm0
        movaps  xmm5, xmm1
        shufps  xmm5, xmm5, 0x00
        mulps   xmm4, xmm5
        subps   xmm1, xmm4
        movaps  xmm4, xmm0
        movaps  xmm5, xmm2
        shufps  xmm5, xmm5, 0x00
        mulps   xmm4, xmm5
        subps   xmm2, xmm4

        ## Dijele se koeficijenti druge jednacine koeficijentom uz x[1]
	## u toj jednacini, cime isti koeficijent postaje jednak 1.  Ako
        ## je inicijalno ovaj koeficijent jednak 0, prethodno se
	## zamjenjuju druga i treca jednacina, kako bi vrijednost ovog
	## koeficijenta bila razlicita od 0.
        movaps  xmm4, xmm1
        shufps  xmm4, xmm4, 0x55
        comiss  xmm4, zero
        jne scale_2nd_equation_coefficients
        movaps  xmm4, xmm1
        movaps  xmm1, xmm2
        movaps  xmm2, xmm4
scale_2nd_equation_coefficients:
        movaps  xmm4, xmm1
        shufps  xmm4, xmm4, 0x55
        divps   xmm1, xmm4

        ## Oduzimanjem druge jednacine pomnozene odgovarajucom
        ## vrijednoscu od prve odn. trece jednacine postize se da
        ## koeficijenti uz x[1] u ove dvije jednacine postanu jednaki 0.
        movaps  xmm4, xmm1
        movaps  xmm5, xmm0
        shufps  xmm5, xmm5, 0x55
        mulps   xmm4, xmm5
        subps   xmm0, xmm4
        movaps  xmm4, xmm1
        movaps  xmm5, xmm2
        shufps  xmm5, xmm5, 0x55
        mulps   xmm4, xmm5
        subps   xmm2, xmm4

        ## Dijele se koeficijenti trece jednacine koeficijentom uz x[2]
	## u toj jednacini, cime isti koeficijent postaje jednak 1.
        movaps  xmm4, xmm2
        shufps  xmm4, xmm4, 0xaa
        divps   xmm2, xmm4

        ## Oduzimanjem trece jednacine pomnozene odgovarajucom
        ## vrijednoscu od prve odn. druge jednacine postize se da
        ## koeficijenti uz x[2] u ove dvije jednacine postanu jednaki 0.
        movaps  xmm4, xmm2
        movaps  xmm5, xmm0
        shufps  xmm5, xmm5, 0xaa
        mulps   xmm4, xmm5
        subps   xmm0, xmm4
        movaps  xmm4, xmm2
        movaps  xmm5, xmm1
        shufps  xmm5, xmm5, 0xaa
        mulps   xmm4, xmm5
        subps   xmm1, xmm4

        ## U ovom trenutku matrica lijeve strane sistema je pretvorena u
        ## jedinicnu matricu, cime su rjesenja sistema jednaka
        ## vrijednostima koeficijenata na desnoj strani sistema; ova
        ## rjesenja se sada upisuju u za to predvidjene memorijske
	## lokacije.
        mov     edi, [ebp+16]
        shufps  xmm0, xmm0, 0xff
        movss   [edi], xmm0
        add     edi, 4
        shufps  xmm1, xmm1, 0xff
        movss   [edi], xmm1
        add     edi, 4
        shufps  xmm2, xmm2, 0xff
        movss   [edi], xmm2
        add     edi, 4
                
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
	## registre.
        fxrstor [esp]
        mov     esp, edx

done:        
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
