.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.
        
.text
.global intersection

	## Funkcija intersection() izracunava tacku presjeka izmedju
	## date prave i date ravni.  Argumenti funkcije su:
        ##   [ebp+8], [ebp+12] - adrese sa koordinatama tacaka koje
	##                       odredjuju pravu
        ##   [ebp+16] - adresa sa koordinatama tacke koja lezi u ravni
        ##   [ebp+20] - adresa sa koordinatama vektora normale ravni
        ##   [ebp+24] - adresa na koju treba smjestiti izracunate
	##              kordinate presjecne tacke
        ## Pretpostavka je da su tacke koje odreduju pravu razlicite,
        ## kao i da je vektor normale ravni ne-nulti vektor.  Funkcija
        ## vraca 1 u slucaju kada je presjek tacno jedna tacka, odn. 0
        ## ako presjek ne postoji ili ako prava lezi u ravni.  Funkcija
	## se u potpunosti podvrgava C konvencijama pozivanja.
intersection:
        ## Prolog funkcije.
	enter   16, 0
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Postavlja se direction fleg na odgovarajucu vrijednost.
        cld

        ## Preko pomocnog niza, u registar xmm0 se upisuju koordinate
        ## prve tacke date prave (u oznaci tacka P0).
        mov     ecx, 3
        mov     esi, [ebp+8]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm0, [ebp-16]

        ## Na isti nacin, u registre xmm1, xmm2 i xmm3 se upisuju
        ## koordinate druge tacke date prave odn. tacke koja lezi u
        ## datoj ravni, te vektora normale date ravni (u oznakama
	## respektivno P0, P i N).
        mov     ecx, 3
        mov     esi, [ebp+12]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm1, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+16]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm2, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+20]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm3, [ebp-16]

        ## U registrima xmm1 i xmm2 se formiraju vektor pravca date
	## prave P1-P0 (u oznaci vektor D), odn. vektor P-P0.
        subps   xmm1, xmm0
        subps   xmm2, xmm0

        ## Izracunava se skalarni proizvod vektora N i D.
        movaps  xmm4, xmm3
        mulps   xmm4, xmm1
        movaps  xmm5, xmm4
        shufps  xmm5, xmm5, 0x01
        movaps  xmm6, xmm4
        shufps  xmm6, xmm6, 0x02
        addss   xmm4, xmm5
        addss   xmm4, xmm6

        ## Ukoliko je ovaj skalarni proizvod jednak 0, prava je ili
        ## paralelna ravni ili lezi u njoj, tako da se vraca 0 kao
	## rezultat funkcije.
        comiss  xmm4, zero
        jne single_intersection
        xor     eax, eax 
        jmp     restore

single_intersection:
        ## Izracunava se skalarni proizvod vektora N i P-P0.
        mulps   xmm3, xmm2
        movaps  xmm5, xmm3
        shufps  xmm5, xmm5, 0x01
        movaps  xmm6, xmm3
        shufps  xmm6, xmm6, 0x02
        addss   xmm3, xmm5
        addss   xmm3, xmm6

        ## Vrijednost t koja odreduje polozaj presjecne tacke na pravoj
        ## je odredjena kolicnikom skalarnih proizvoda (N,P-P0) i (N,D).
        ## Izracunava se ova vrijednost, a potom i koordinate presjecne
	## tacke R=P0+t*D.
        divss   xmm3, xmm4
        shufps  xmm3, xmm3, 0x00
        mulps   xmm1, xmm3
        addps   xmm0, xmm1

        ## Preko pomocnog niza, koordinate presjecne tacke se iz
        ## registra xmm0 upisuju u za to predvidjene memorijske
	## lokacije.
        movups  [ebp-16], xmm0
        mov     ecx, 3
        lea     esi, [ebp-16]
        mov     edi, [ebp+24]
        rep movsd

        ## Vraca se 1 kao rezultat iz funkcije.
        mov     eax, 1
        
restore:        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor [esp]
        mov     esp, edx

done:        
	## Epilog funkcije.
        pop     edi
        pop     esi
	leave
        ret
