.intel_syntax noprefix

.text
.global value

	## Funkcija value() racuna vrijednost polinoma u datoj
	## tacki. Argumenti funkcije su:
	##   [ebp+8] - red polinoma
	##   [ebp+12] - pokazivac na polje sa koeficijentima polinoma
	##   [ebp+16] - tacka u kojoj se racuna vrijednost polinoma
	## Pretpostavlja se da je red polinoma veci ili jednak
	## 0. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
value:
        ## Prolog funkcije.
        enter	0, 0
	push	esi

	## Za racunanje polinoma se koristi Horner-ova sema:
	##   c[n]*x^(n)+c[n-1]*x^(n-1)+...+c[1]*x+c[0]=
	##     =((c[n]*x+c[n-1])*x+...+c[1])*x+c[0]

	## Registar ecx se koristi kao brojac.
	mov	ecx, [ebp+8]

	## Registar esi pokazuje na tekuci koeficijent polinoma pri
	## izracunavanju Horner-ovom semom. Registar se inicijalizuje
	## na adresu koeficijenta uz najveci stepen polinoma.
	mov	esi, [ebp+12]
	mov	eax, ecx
	shl	eax, 2
	add	esi, eax

	## Na stek se stavlja tacka u kojoj se racuna vrijednost
	## polinoma.
	fld	[ebp+16]	# Stek:	x

	## Na stek se stavlja koeficijent uz najveci stepen polinoma,
	## a registar esi se dekrementira tako da pokazuje na
	## koeficijent uz sledeci manji stepen polinoma.
	fld	[esi]
	sub	esi, 4		# Stek:	x, c[n]
	
next_coeff:
	## Provjerava se da li je racunanje zavrseno.
	cmp	ecx, 0
	je finish

	## Uracunava se tekuci koeficijent u sumu. Sa P(k) je oznacena
	## parcijalna suma u Horner-ovoj semi u kojoj su uracunati
	## koeficijenti polinoma sa stepenima n,n-1,...,k.
	fld	st(1)		# Stek:	x, ((c[n]*x+c[n-1])*x+...+c[k+1])=P(k+1), x
	fmulp			# Stek:	x, P(k+1)*x
	fld	[esi]		# Stek:	x, P(k+1)*x, c[k]
	faddp			# Stek:	x, P(k+1)*x+c[k]=P(k)

	## Registar esi se dekrementira tako da pokazuje na koeficijent
	## uz sledeci manji stepen polinoma.
	sub	esi, 4

	## Dekrementira se broja i nastavlja se sa petljom.
	dec	ecx
	jmp next_coeff

	## Cisti se stek. Sa P je oznacena trazena vrijednost polinoma.
	fxch			# Stek:	P, x
	ffree st		# Stek:	P, invalidna vrijednost
	fincstp			# Stek:	P

finish:
        ## Epilog funkcije.
	pop	esi
        leave
        ret
