.intel_syntax noprefix

.text
.global bernoulli

	## Funkcija bernoulli() racuna n-ti po redu Bernoulli-jev broj.
	## Argumenti funkcije su:
	##   [ebp+8] - indeks n broja koji se trazi
        ## Funkcija vraca trazeni broj.  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
bernoulli:
        ## Prolog funkcije.
        enter	0, 0
        push    esi
        push    edi

        ## Indeks n ce biti cuvan u registru edx.
        mov     edx, [ebp+8]

        ## Alocira se prostor na steku za polje Bernoulli-jevih brojeva,
	## odnosno polje binomnih koeficijenata.  Prvo polje ce se
        ## sastojati od n+1 realnih brojeva, a drugo polje od n+2
	## cijelih brojeva.  Adresa pocetnog elementa prvog polja ce se
	## cuvati u registru edi, a pocetnog elementa drugog polja u 
        ## registru esi. U drugo polje se odmah upisuju binomni
	## koeficijenti za vrijednost indeksa jednaku 0.
        mov     eax, edx
        inc     eax
        shl     eax, 2
        sub     esp, eax
        mov     edi, esp
        add     eax, 4
        sub     esp, eax
        mov     esi, esp
        mov     dword ptr [esi], 1
        mov     dword ptr [esi+4], 1

        ## U petlji se racunaju Bernoulli-jevi brojevi sa indeksima od 1
	## do n.  Registar ecx se koristi kao brojac.
        xor     ecx, ecx
next_number:
        ## Provjerava se da li je vrijednost brojaca prevazisla n i, ako
	## je to slucaj, prekida se petlja.
        inc     ecx
        cmp     ecx, edx
        ja done_numbers

        ## Na osnovu vrijednosti binomnih koeficijenata u formuli za
	## Bernoulli-jev broj B[k-1], racunaju se binomni koeficijenti u
	## formuli za Bernoulli-jev broj B[k]; za ovo izracunavanje
	## koristi se Pascal-ov trougao.
        push    ecx
        inc     ecx

        ## Binomni koeficijent "k+1 nad k+1" se postavlja na 1.
        mov     dword ptr [esi+4*ecx], 1

next_coeff:
        ## Provjerava se da li su izracunati svi binomni koeficijenti u
	## tekucem prolazu (binomni koeficijent "k+1 nad 0" je uvijek
	## jednak 1 i on se nikad ne mijenja) i, ako je to slucaj, prelazi
	## se na racunanje Bernoulli-jevog broja B[k].
        dec     ecx
        cmp     ecx, 0
        je done_coeffs

        ## Binomni koeficijent "k+1 nad j" se racuna kao zbir
	## koeficijenata "k nad j" i "k nad j+1".
        mov     eax, [esi+4*ecx-4]
        add     [esi+4*ecx], eax
        jmp next_coeff

done_coeffs:
        ## Sada se racuna broj B[k] pomocu datog izraza.
        pop     ecx

        ## Izracunava se suma S proizvoda brojeva B[j] (0<=j<k) i
        ## odgovarajucih binomnih koeficijenata za racunaje broja
        ## B[k] (bice koriscena oznaka S(j) za parcijalnu sumu
        ## prilikom ovog racunanja).  Registar ecx se opet koristi kao
	## brojac.
        fld1                            # Stek: 1=B[0]*"k+1 nad 0"=S(0)
        push    ecx
        dec     ecx
        jecxz done_terms
next_term:
        fld     dword ptr [edi+4*ecx]   # Stek: S(j), B[j+1]
        fimul   dword ptr [esi+4*ecx]   # Stek: S(j), B[j+1]*"k+1 nad j+1"
        faddp                           # Stek: S(j)+B[j+1]*"k+1 nad j+1"=S(j+1)
        loop next_term

done_terms:
        ## Mijenja se znak gornje sume i taj broj se dijeli sa "k+1 nad
	## k", cime se dobija vrijednost Bernoulli-jevog broja B[k].
        pop     ecx
        fchs                            # Stek: -S(k-1)
        fidiv   dword ptr [esi+4*ecx]   # Stek: -S(k-1)/"k+1 nad k"=B[k]
        fstp    dword ptr [edi+4*ecx]   # Stek:
        jmp next_number
        
done_numbers:
        ## Upisuje se rezultat funkcije natrag u odgovarajuci registar i
        ## oslobadja se prostor na steku koji je bio koriscen za polje
	## Bernoulli-jevih brojeva, odnosno polje binomnih
	## koeficijenata.
        fld     dword ptr [edi+4*edx]
        mov     eax, edx
        shl     eax, 1
        add     eax, 3
        shl     eax, 2
        add     esp, eax

finish:        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        leave
        ret
