.intel_syntax noprefix

.data
a_mask: .int    0b0             # Prvi bit u A klasi adresa.
b_mask: .int    0b10            # Prva dva bita u B klasi adresa.
c_mask: .int    0b110           # Prva tri bita u C klasi adresa.
d_mask: .int    0b1110          # Prva cetiri bita u D klasi adresa.
e_mask: .int    0b11110         # Prvih pet bitova u E klasi adresa.

.text
.global class
        
        ## Funkcija class() odredjuje kojoj klasi IP adresa pripada data
	## adresa.  Argumenti funkcije su:
        ##   [ebp+8] - adresa prvog elementa u polju od 4 cijela broja
        ##             koji predstavljaju elemente IP adrese u "dotted
        ##             quad" zapisu (pri cemu su elementi stavljeni u
        ##             polje pocev od elementa najvece tezine)
        ## Pretpostavka je da su svi elementi adrese validni, odnosno u
	## opsegu [0,255].  Funkcija vraca 0 ako data adresa nije
	## validna, odnosno neko od slova 'A', 'B', 'C', 'D' ili 'E' kao
	## indikator klase kojoj pripada adresa u suprotnom.  Funkcija
	## se u potpunosti podvrgava C konvencijama pozivanja.
class:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx

        ## Zadati problem se mogao rijesiti i na drugacije nacine od
        ## ovdje prezentiranog; najefikasnije rjesenje bi bilo sa
        ## pravljenjem jedne tabele mapiranja sa bajta najvece tezine
        ## adrese na oznaku klase u .data sekciji i uposljavanjem xlatb
        ## naredbe u tijelu procedure.

        ## Incijalizuje se rezultat koji vraca funkcija.
        xor     eax, eax

        ## U registar edx se ucitava element najvece tezine adrese.
	## Jedino ovaj element je relevantan prilikom odredjivanja klase
	## adrese, ostali elementi mogu biti ignorisani.
        mov     ebx, [ebp+8]
        mov     edx, [ebx]

        ## Provjerava se da li je u pitanju adresa iz E klase adresa;
	## ako jeste priprema se odgovarajuci rezultat i zavrsava se
	## funkcija.
        shr     edx, 3
        cmp     edx, e_mask
        jne check_d
        mov     al, 'E'
        jmp done

check_d:
        ## Provjerava se da li je u pitanju adresa iz D klase adresa;
	## ako jeste priprema se odgovarajuci rezultat i zavrsava se
	## funkcija.
        shr     edx
        cmp     edx, d_mask
        jne check_c
        mov     al, 'D'
        jmp done

check_c:
        ## Provjerava se da li je u pitanju adresa iz C klase adresa;
	## ako jeste priprema se odgovarajuci rezultat i zavrsava se
	## funkcija.
        shr     edx
        cmp     edx, c_mask
        jne check_b
        mov     al, 'C'
        jmp done

check_b:
        ## Provjerava se da li je u pitanju adresa iz B klase adresa;
	## ako jeste priprema se odgovarajuci rezultat i zavrsava se
	## funkcija.
        shr     edx
        cmp     edx, b_mask
        jne check_a
        mov     al, 'B'
        jmp done

check_a:
        ## Provjerava se da li je u pitanju adresa iz A klase adresa;
	## ako jeste priprema se odgovarajuci rezultat i zavrsava se
	## funkcija.
        shr     edx
        cmp     edx, a_mask
        jne done
        mov     al, 'A'
        jmp done

done:        
        ## Epilog funkcije.
        pop     ebx
        leave
        ret
