.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.
two:    .float  2               # Broj 2.
        
.text
.global area

	## Funkcija area() odredjuje povrsinu trougla kome su date
	## koordinate tjemena.  Argumenti funkcije su:
	##   [ebp+8] - vektor sa koordinatama prvog tjemena trougla
        ##   [ebp+12] - vektor sa koordinatama drugog tjemena trougla
        ##   [ebp+16] - vektor sa koordinatama treceg tjemena trougla
        ##   [ebp+20] - adresa na koju se smjesta izracunata povrsina
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
area:
        ## Prolog funkcije.
	enter   16, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## U krajnje desni element registra xmm7, kao i u poslednji
        ## element pomocnog niza od 4 realna broja koji je alociran kao
        ## lokalna promjenljiva, se upisuje 0.
        movss   xmm7, zero
        movss   [ebp-4], xmm7

        ## Postavlja se smjer za instrukcije za rad sa stringovima.
        cld

        ## Preko pomocnog niza, u registar xmm0 se upisuju koordinate
	## prvog tjemena trougla.
        mov     ecx, 3
        mov     esi, [ebp+8]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm0, [ebp-16]

        ## Na isti nacin, u registre xmm1 i xmm2 se upisuju koordinate
	## drugog i treceg tjemena trougla.
        mov     ecx, 3
        mov     esi, [ebp+12]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm1, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+16]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm2, [ebp-16]

        ## U registrima xmm1 i xmm2 se formiraju vektori stranica 
        ## trougla, a potom se prave njihove kopije u registrima xmm3 i
	## xmm4.
        subps   xmm1, xmm0
        subps   xmm2, xmm0
        movaps  xmm3, xmm1
        movaps  xmm4, xmm2

        ## Preuredjuju se ovi vektori i njihove kopije (koje se prave u
	## registrima xmm3 i xmm4) radi racunanja vektorskog proizvoda.
        shufps  xmm1, xmm1, 0xc9
        shufps  xmm2, xmm2, 0xd2
        shufps  xmm3, xmm3, 0xd2
        shufps  xmm4, xmm4, 0xc9

        ## Odredjuje se vektorski proizvod.
        mulps   xmm1, xmm2
        mulps   xmm3, xmm4
        subps   xmm1, xmm3

        ## Sabiranjem kvadrata komponenti vektorskog proizvoda,
	## odredjuje se kvadrat norme tog vektora.
        mulps   xmm1, xmm1
        movhlps xmm0, xmm1
        addps   xmm0, xmm1
        movaps  xmm1, xmm0
        shufps  xmm1, xmm1, 0x55
        addss   xmm0, xmm1
        
        ## Izracunava se sama norma, a zatim i povrsina trougla i
	## smjesta na odgovarajucu lokaciju u memoriji.
        sqrtss  xmm0, xmm0
        divss   xmm0, two
        mov     edi, [ebp+20]
        movss   [edi], xmm0
        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
	## registre.
        fxrstor [esp]
        mov     esp, edx

done:        
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
