.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.
        
.text
.global standard_deviation

	## Funkcija standard_deviation() izracunava standardnu
        ## devijaciju za dati niz vrijednosti.  Argumenti funkcije su:
        ##   [ebp+8] - broj vrijednosti u datom nizu
        ##   [ebp+12] - adresa na kojoj pocinje niz
        ##   [ebp+16] - adresa na koju treba upisati izracunatu
	##              standardnu devijaciju
        ## Pretpostavka je da su u nizu zadate bar dvije vrijednosti.
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
standard_deviation:
        ## Prolog funkcije.
	enter           0, 0
        push            esi
        push            edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov             eax, 1
        cpuid
        test            edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov             edx, esp
        and             esp, 0xfffffff0
        sub             esp, 512
        fxsave          [esp]

        ## Registar eax ce cuvati broj elemenata datog vektora.
        mov             eax, [ebp+8]

        ## Izracunava se srednja vrijednost datog vektora.  Sabiranje
        ## elemenata vektora ce biti vrseno za po 4 elementa istovremeno
        ## i parcijalni zbirovi ce biti cuvani u registru xmm0.
        ## Registar esi ce sadrzati adresu prvog narednog elementa
	## vektora. Registar ecx ce biti koriscen kao brojac.
        movss           xmm0, zero
        shufps          xmm0, xmm0, 0b00000000
        mov             esi, [ebp+12]        
        mov             ecx, eax
mean_next4:
        ## Ako je preostalo 4 ili vise elemenata datog vektora,
        ## uracunavaju se isti u parcijalne sume, registar esi se
        ## postavlja na adresu prvog narednog elementa vektora, a brojac
	## se umanjuje za 4.
        cmp             ecx, 4
        jl mean_next1
        movups          xmm1, [esi]
        addps           xmm0, xmm1
        add             esi, 16
        sub             ecx, 4
        jmp mean_next4

mean_next1:
        ## Zaracunavaju se u parcijalnu sumu preostali elementi vektora.
        movss           xmm1, [esi]
        addss           xmm0, xmm1
        add             esi, 4
        loop mean_next1

        ## Sabiraju se parcijalne sume i potom se izracunava srednja
        ## vrijednost elemenata vektora i smjesta u sva polja registra
	## xmm7.
        movhlps         xmm1, xmm0
        addps           xmm0, xmm1
        movaps          xmm1, xmm0
        shufps          xmm1, xmm1, 0b01010101
        addss           xmm0, xmm1
        cvtsi2ss        xmm1, eax
        divss           xmm0, xmm1
        shufps          xmm0, xmm0, 0b00000000
        movaps          xmm7, xmm0

        ## Parcijalne sume za racunanje standardne devijacije se
        ## formiraju na potpuno isti nacin kao parcijalne sume za
	## racunanje srednje vrijednosti.
        movss           xmm0, zero
        shufps          xmm0, xmm0, 0b00000000
        mov             esi, [ebp+12]
        mov             ecx, eax
deviation_next4:
        ## Ako je preostalo 4 ili vise elemenata datog vektora,
        ## uracunavaju se isti u parcijalne sume, registar esi se
        ## postavlja na adresu prvog narednog elementa vektora, a brojac
	## se umanjuje za 4.
        cmp             ecx, 4
        jl deviation_next1
        movups          xmm1, [esi]
        subps           xmm1, xmm7
        mulps           xmm1, xmm1
        addps           xmm0, xmm1
        add             esi, 16
        sub             ecx, 4
        jmp deviation_next4

deviation_next1:
        ## Zaracunavaju se u parcijalnu sumu preostali elementi vektora.
        movss           xmm1, [esi]
        subps           xmm1, xmm7
        mulss           xmm1, xmm1
        addss           xmm0, xmm1
        add             esi, 4
        loop deviation_next1

        ## Sabiraju se parcijalne sume i potom se izracunava vrijednost
        ## devijacije.
        movhlps         xmm1, xmm0
        addps           xmm0, xmm1
        movaps          xmm1, xmm0
        shufps          xmm1, xmm1, 0b01010101
        addss           xmm0, xmm1
        dec             eax
        cvtsi2ss        xmm1, eax
        divss           xmm0, xmm1
        sqrtss          xmm0, xmm0

        ## Smjesta se izracunata vrijednost u za to predvidjenu
	## memorijsku lokaciju.
        mov             edi, [ebp+16]
        movss           [edi], xmm0        
        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor         [esp]
        mov             esp, edx

done:        
	## Epilog funkcije.
        pop             edi
        pop             esi
	leave
        ret
