.intel_syntax noprefix

.text
.global intersection

	## Funkcija intersection() izracunava presjek datog skupa kocki.
	## Argumenti funkcije su:
        ##   [ebp+8] - broj kocki
        ##   [ebp+12] - adresa prvog elementa u polju sa koordinatama
	##              osnovnih tjemena kocki
        ##   [ebp+16] - adresa prvog elementa u polju sa duzinama
	##              stranica kocki
        ##   [ebp+20], [ebp+24] - adrese na koju treba smjestiti
        ##                        koordinate naspramnih tjemena
        ##                        pravougaonog paralelopipeda koji
        ##                        predstavlja presjek datih kocki
        ## Pretpostavka je da je broj datih kocki veci od 0, kao i da su
	## sve stranice kocki vece od 0.  Funkcija vraca 1 ako postoji
	## presjek, odn. 0 u suprotnom.  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
intersection:
        ## Prolog funkcije.
	enter   16, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Registar esi ce sadrzati adresu na kojoj se nalaze koordinate
        ## osnovnog tjemena tekuce kocke, a registar ebx adresu na kojoj
	## se nalazi duzina stranice tekuce kocke.
        cld
        mov     esi, [ebp+12]
        mov     ebx, [ebp+16]

        ## Preko pomocnog niza, u registar xmm0 se upisuju koordinate
        ## osnovnog tjemena prve kocke; registar xmm0 ce inace uvijek
        ## sadrzati koordinate osnovnog tjemena pravougaonog
	## paralelopipeda koji predstavlja presjek kocki.
        mov     ecx, 3
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm0, [ebp-16]

        ## U registar xmm1 se upisuje duzina stranice prve kocke, a
        ## zatim se istom registru formiraju koordinate naspramnog
	## tjemena prve kocke; registar xmm1 ce inace uvijek sadrzati
        ## koordinate naspramnog tjemena pravougaonog paralelopipeda
	## koji predstavlja presjek kocki.
        movss   xmm1, [ebx]
        add     ebx, 4
        shufps  xmm1, xmm1, 0b00000000
        addps   xmm1, xmm0

        ## Registar ecx ce se koristiti kao brojac, i sadrzace broj
	## kocki preostalih za procesiranje.
        mov     ecx, [ebp+8]        
        dec ecx
        jecxz write_results

next_cube:
        ## Preko pomocnog niza, u registar xmm0 se upisuju koordinate
        ## osnovnog tjemena tekuce kocke.
        push    ecx
        mov     ecx, 3
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm2, [ebp-16]
        pop     ecx

        ## U registar xmm1 se upisuje duzina stranice tekuce kocke, a
        ## zatim se istom registru formiraju koordinate naspramnog
	## tjemena tekuce kocke.
        movss   xmm3, [ebx]
        add     ebx, 4
        shufps  xmm3, xmm3, 0b00000000
        addps   xmm3, xmm2

        ## Azurira se presjek.
        maxps   xmm0, xmm2
        minps   xmm1, xmm3

        ## Porede se koordinate naspramnih tjemena pravougaonog
        ## paralelopipeda koji predstavlja presjek, i smjestaju se
	## rezultati poredjenja u memoriju.
        movaps  xmm2, xmm1
        cmpltps xmm2, xmm0
        movups  [ebp-16], xmm2

        ## Ispituje se da li je presjek postao prazan, i u tom slucaju
	## se vraca 0 kao rezultat funkcije.
        push    ecx
        push    esi
        mov     ecx, 3
        lea     esi, [ebp-16]
next_comparation:
        lodsd
        cmp     eax, 0
        je continue
        pop     esi        
        pop     ecx
        xor     eax, eax
        jmp restore
continue:
        loop next_comparation
        pop     esi
        pop     ecx

        ## Prelazi se na narednu kocku. 
        loop next_cube

write_results:
        ## Upisuju se u odgovarajuce lokacije koordinate naspramnih
        ## tjemena pravougaonog paralelopipeda koji predstavlja presjek,
	## i vraca se 1 kao rezultat funkcije.
        movups  [ebp-16], xmm0
        mov     ecx, 3
        lea     esi, [ebp-16]
        mov     edi, [ebp+20]
        rep movsd
        movups  [ebp-16], xmm1
        mov     ecx, 3
        lea     esi, [ebp-16]
        mov     edi, [ebp+24]
        rep movsd
        mov     eax, 1

restore:
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor [esp]
        mov     esp, edx

done:
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
