.intel_syntax noprefix

.text
.global echo

	## Funkcija echo() primjenjuje echo efekat na dati niz odbiraka.
	## Argumenti funkcije su:
        ##   [ebp+8] - broj odbiraka
        ##   [ebp+12] - adresa prvog elementa u nizu odbiraka datog
	##              signala
        ##   [ebp+16] - kasnjenje echo efekta, izrazeno u broju odbiraka
        ##   [ebp+20] - faktor skaliranja signala za echo efekat
        ##   [ebp+24] - adresa prvog elementa niza u koji treba
        ##              smjestiti izracuante odbirke, sa uracunatim echo
	##              efektom
        ## Pretpostavka je da je broj odbiraka veci od 0, kao i da je
        ## kasnjenje broj koji je veci od 0, a manji od broja odbiraka;
        ## takodje i da je faktor skaliranja signala za echo efekat broj
	## veci od 0.  Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
echo:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Inicijalizuje se direction fleg.
        cld

        ## Registar esi ce sadzati adresu tekuceg odbirka polaznog
        ## signala, a registar esi adresu tekuceg odbirka rezultujuceg
	## signala.  Odbirci sa pocetka polaznog signala, na koje se ne
	## moze primijeniti echo efekat, se kopiraju u rezultujuci
	## signal.
        mov     ebx, [ebp+12]
        mov     esi, ebx
        mov     ecx, [ebp+16]
        mov     edi, [ebp+24]
        rep movsd

        ## Registar ecx ce sadrzati broj odbiraka koji su preostali za
        ## procesiranje.  U sve 4 pozicije registra xmm0 se upisuje
	## faktor skaliranja.
        mov     ecx, [ebp+8]
        sub     ecx, [ebp+16]
        movss   xmm0, [ebp+20]
        shufps  xmm0, xmm0, 0b00000000
        
next_four:
        ## Provjerava se da li je preostalo 4 ili vise odbiraka
	## rezultujuceg signala za izracunavanje.
        cmp     ecx, 4
        jl next_one

        ## Izracunavaju se vrijednosti naredna 4 odbirka rezultujuceg
        ## signala, i upisuju njihove vrijednosti u odgovarajuce
	## lokacije u memoriji.
        movups  xmm1, [ebx]
        mulps   xmm1, xmm0        
        movups  xmm2, [esi]
        addps   xmm2, xmm1
        movups  [edi], xmm2

        ## Prelazi se na narednu iteraciju.
        add     ebx, 16
        add     esi, 16
        add     edi, 16
        sub     ecx, 4
        jmp next_four

next_one:
        ## Provjerava se da li je preostalo jos odbiraka rezultujuceg
	## signala za izracunavanje.
        jecxz restore

        ## Izracunava se vrijednost narednog odbirka rezultujuceg
        ## signala, i upisuje ista u odgovarajucu lokaciju u memoriji.
        movss   xmm1, [ebx]
        mulss   xmm1, xmm0        
        movss   xmm2, [esi]
        addss   xmm2, xmm1
        movss   [edi], xmm2

        ## Prelazi se na narednu iteraciju.
        add     ebx, 4
        add     esi, 4
        add     edi, 4
        dec     ecx
        jmp next_one        

restore:
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor [esp]
        mov     esp, edx

done:
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
