.intel_syntax noprefix

.data
two:	.int	2		# Broj 2.
six:	.int	6		# Broj 6.

.text
.global runge_kutta

	## Funkcija runge_kutta() racuna rjesenje Cauchy-jevog problema
	## u'=sqrt(x*u-1) u datoj tacki i sa datim pocetnim
	## uslovom. Argumenti funkcije su:
	##   [ebp+8] - tacka u kojoj je dat pocetni uslov
	##   [ebp+12] - vrijednost pocetnog uslova
	##   [ebp+16] - rastojanje tacke u kojoj se trazi rjesenje od
	##              tacke pocetnog uslova
	## Pretpostavlja se da je treci argument razlicit od 0. 
	## Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
runge_kutta:
        ## Prolog funkcije.
        enter	0, 0

	## Ucitavaju se argumenti funkcije u registre koprocesora.
	fld	[ebp+12]	# Stek:	u0
	fld	[ebp+16]	# Stek:	u0, h
	fld	[ebp+8]		# Stek:	u0, h, x0
	fld	st(1)		# Stek:	u0, h, x0, h
	fidiv	dword ptr two	# Stek:	u0, h, x0, h/2

	## Izracunava se koeficijent k0.
	fld	st(3)		# Stek:	u0, h, x0, h/2, u0
	fmul	st, st(2)	# Stek:	u0, h, x0, h/2, x0*u0
	fld1			# Stek:	u0, h, x0, h/2, x0*u0, 1
	fsubp			# Stek:	u0, h, x0, h/2, x0*u0-1
	fsqrt			# Stek:	u0, h, x0, h/2, sqrt(x0*u0-1)=f(x0,u0)
	fmul	st, st(3)	# Stek  u0, h, x0, h/2, h*f(x0,u)=k0
	fxch	st(2)		# Stek  u0, h, k0, h/2, x0

	## Izracunava se koeficijent k1.
	fadd	st, st(1)	# Stek  u0, h, k0, h/2, x0+h/2
	fld	st(2)		# Stek:	u0, h, k0, h/2, x0+h/2, k0
	fidiv	dword ptr two	# Stek:	u0, h, k0, h/2, x0+h/2, k0/2
	fadd	st, st(5)	# Stek:	u0, h, k0, h/2, x0+h/2, u0+k0/2
	fmul	st, st(1)	# Stek:	u0, h, k0, h/2, x0+h/2, (x0+h/2)*(u0+k0/2)
	fld1			# Stek:	u0, h, k0, h/2, x0+h/2, (x0+h/2)*(u0+k0/2), 1
	fsubp			# Stek:	u0, h, k0, h/2, x0+h/2, (x0+h/2)*(u0+k0/2)-1
	fsqrt			# Stek:	u0, h, k0, h/2, x0+h/2, sqrt((x0+h/2)*(u0+k0/2)-1)=f(x0+h/2,u0+k0/2)
	fmul	st, st(4)	# Stek:	u0, h, k0, h/2, x0+h/2, h*f(x0+h/2,u0+k0/2)=k1
	fxch	st(2)		# Stek:	u0, h, k0, k1, x0+h/2, h/2

	## Izracunava se koeficijent k2.
	fld	st(2)		# Stek:	u0, h, k0, k1, x0+h/2, h/2, k1
	fidiv	dword ptr two	# Stek:	u0, h, k0, k1, x0+h/2, h/2, k1/2
	fadd	st, st(6)	# Stek:	u0, h, k0, k1, x0+h/2, h/2, u0+k1/2
	fmul	st, st(2)	# Stek:	u0, h, k0, k1, x0+h/2, h/2, (x0+h/2)*(u0+k1/2)
	fld1			# Stek:	u0, h, k0, k1, x0+h/2, h/2, (x0+h/2)*(u0+k1/2), 1
	fsubp			# Stek:	u0, h, k0, k1, x0+h/2, h/2, (x0+h/2)*(u0+k1/2)-1
	fsqrt			# Stek:	u0, h, k0, k1, x0+h/2, h/2, sqrt((x0+h/2)*(u0+k1/2)-1)=f(x0+h/2,u0+k1/2)
	fmul	st, st(5)	# Stek  u0, h, k0, k1, x0+h/2, h/2, x0*f(x0+h/2,u0+k1/2)=k2
	fxch	st(2)		# Stek:	u0, h, k0, k1, k2, h/2, x0+h/2

	## Izracunava se koeficijent k3.
	faddp			# Stek  u0, h, k0, k1, k2, x0+h/2+h/2=x+h
	fld	st(1)		# Stek  u0, h, k0, k1, k2, x+h, k2
	fadd	st, st(6)	# Stek:	u0, h, k0, k1, k2, x0+h, u0+k2
	fmulp			# Stek:	u0, h, k0, k1, k2, (x0+h)*(u0+k2)
	fld1			# Stek:	u0, h, k0, k1, k2, (x0+h)*(u0+k2), 1
	fsubp			# Stek:	u0, h, k0, k1, k2, (x0+h)*(u0+k2)-1
	fsqrt			# Stek:	u0, h, k0, k1, k2, sqrt((x0+h)*(u0+k2)-1)=f(x0+h,u0+k2)
	fmulp	st(4), st	# Stek:	u0, h*f(x0+h,u0+k2)=k3, k0, k1, k2

	## Izracunava se rjesenje problema.
	faddp			# Stek:	u0, k0, k3, k1+k2
	fimul	dword ptr two	# Stek:	u0, k0, k3, 2*k1+2*k2
	faddp			# Stek:	u0, k0, 2*k1+2*k2+k3
	faddp			# Stek:	u0, k0+2*k1+2*k2+k3
	fidiv	dword ptr six	# Stek:	u0, (k0+2*k1+2*k2+k3)/6
	faddp			# Stek:	u0+(k0+2*k1+2*k2+k3)/6

        ## Epilog funkcije.
        leave
        ret
