.intel_syntax noprefix

.text
.global refract

	## Funkcija refract() racuna izlazni pravac svjetlosti na
	## granici izmedju dviju sredina.  Argumenti funkcije su:
	##   [ebp+8] - vektor koji predstavlja upadni pravac svjetlosti
	##   [ebp+12] - vektor normale povrsi koja razdvaja sredine
        ##   [ebp+16] - odnos koeficijenata prelamanja svjetlosti
        ##   [ebp+20] - vektor u koji treba upisati rezultat
        ## Funkcija vraca 0 ukoliko dodje do potpune interne refleksije,
	## odnosno 1 inace.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
refract:
        ## Prolog funkcije.
        enter	0, 0
        push    ebx
        push    esi
        push    edi

        ## U komentarima ce biti koriscene oznake I, N, eta i T
	## respektivno za argumente funkcije, kao i c1 i c2 za izraze
	## cos(theta1) odnosno cos(theta2).

        ## Izracunava se vrijednost c1 kao skalarni proizvod vektora -I
	## i N.  Registar ecx se koristi kao brojac, registri ebx i esi
	## cuvaju adrese tekucih elemenata ovih vektora.
        fld dword ptr [ebp+16]          # Stek: eta
        fldz                            # Stek: eta, 0 = c1
        mov     ecx, 3
        mov     ebx, [ebp+8]
        mov     esi, [ebp+12]
next_term:
        fld     dword ptr [ebx]         # Stek: eta, c1, I[i]
        add     ebx, 4
        fchs                            # Stek: eta, c1, -I[i]
        fmul    dword ptr [esi]         # Stek: eta, c1, -I[i]*N[i]
        add     esi, 4
        faddp                           # Stek: eta, c1-I[i]*N[i]=c1
        loop next_term

        ## Izracunava se vrijednost c2^2 prema datoj formuli.
        fld1                            # Stek: eta, c1, 1
        fld     st(2)                   # Stek: eta, c1, 1, eta
        fmul    st, st                  # Stek: eta, c1, 1, eta^2
        fld1                            # Stek: eta, c1, 1, eta^2, 1
        fld     st(3)                   # Stek: eta, c1, 1, eta^2, 1, c1
        fmul    st, st                  # Stek: eta, c1, 1, eta^2, 1, c1^2
        fsubp                           # Stek: eta, c1, 1, eta^2, 1-c1^2
        fmulp                           # Stek: eta, c1, 1, eta^2*(1-c1^2)
        fsubp                           # Stek: eta, c1, 1-eta^2*(1-c1^2)

        ## Ispituje se da li je doslo do potpune interne refleksije i
	## ako jeste, odmah se zavrsava funkcija.
        fldz                            # Stek: eta, c1, 1-eta^2*(1-c1^2), 0
        fcomip  st, st(1)               # Stek: eta, c1, 1-eta^2*(1-c1^2)
        ja total_internal_reflection

        ## Izracunava se vrijednost c2.
        fsqrt                           # Stek: eta, c1, sqrt(1-eta^2*(1-c1^2))=c2

        ## Izracunava se vektor T.  Registar ecx se koristi kao brojac,
	## registri ebx i esi cuvaju adrese tekucih elemenata vektora I
	## i N, a registar edi vektora T.
        fxch                            # Stek: eta, c2, c1
        fmul    st(2)                   # Stek: eta, c2, eta*c1
        fsubrp                          # Stek: eta, eta*c1-c2
        mov     ecx, 3
        mov     ebx, [ebp+8]
        mov     esi, [ebp+12]
        mov     edi, [ebp+20]
next_element:
        fld     dword ptr [ebx]         # Stek: eta, eta*c1-c2, I[i]
        add     ebx, 4
        fmul    st(2)                   # Stek: eta, eta*c1-c2, eta*I[i]
        fld     dword ptr [esi]         # Stek: eta, eta*c1-c2, eta*I[i], N[i]
        add     esi, 4
        fmul    st(2)                   # Stek: eta, eta*c1-c2, eta*I[i], (eta*c1-c2)*N[i]
        faddp                           # Stek: eta, eta*c1-c2, eta*I[i]+(eta*c1-c2)*N[i]=T[i]
        fstp    dword ptr [edi]         # Stek: eta, eta*c1-c2
        add     edi, 4
        loop next_element

        ## Odbacuju se vrijednosti koje se nalaze na vrhu steka.
        ffree   st              # Stek: eta, invalidna vrijednost
        fincstp                 # Stek: eta
        ffree   st              # Stek: invalidna vrijednost
        fincstp                 # Stek: 

        ## Vraca se 1 kao rezultat i zavrsava se funkcija.
        mov     eax, 1
        jmp finish

total_internal_reflection:
        ## Vraca se 0 kao rezultat.
        xor     eax, eax

finish:        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
