.intel_syntax noprefix

.data
den:    .float  180
r:      .float  6378.14

.text
.global distance

	## Funkcija distance() racuna najkrace rastojanje izmedju dvije
	## tacke na Zemlji.  Argumenti funkcije su:
	##   [ebp+8], [ebp+12] - latituda i longitude prve tacke (u
        ##                       stepenima)
        ##   [ebp+16], [ebp+20] - latituda i longitude prve tacke (u
        ##                        stepenima)
        ## Pretpostavka je da su latitude u intervalu [-90,90], a 
        ## longitude u intervalu [-180,180].  Funkcija vraca izracunato
	## rastojanje.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
distance:
        ## Prolog funkcije.
        enter	0, 0

        ## Izracunava se vrijednost kojom treba pomnoziti ugao u
	## stepenima da bi se dobio ugao u radijanima
        fldpi                           # Stek: pi
        fdiv    dword ptr den           # Stek: pi/180=rad

        ## Izracunavaju se Dekartove koordinate (x0,y0,z0) prve tacke na
	## osnovu date latitude i longitude.
        fld     dword ptr [ebp+12]      # Stek: rad, long0
        fmul    st(1)                   # Stek: rad, long0*rad
        fsincos                         # Stek: rad, sin(long0*rad), cos(long0*rad)
        fld     dword ptr [ebp+8]       # Stek: rad, sin(long0*rad), cos(long0*rad), lat0
        fmul    st(3)                   # Stek: rad, sin(long0*rad), cos(long0*rad), lat0*rad
        fsincos                         # Stek: rad, sin(long0*rad), cos(long0*rad), sin(lat0*rad)=z0, cos(lat0*rad)
        fmul    st(3), st               # Stek: rad, sin(long0*rad)*cos(lat0*rad)=y0, cos(long0*rad), z0, cos(lat0*rad)
        fmulp   st(2), st               # Stek: rad, y0, cos(long0*rad)*cos(lat0*rad)=x0, z0

        ## Izracunavaju se Dekartove koordinate (x1,y1,z1) druge tacke
	## na osnovu date latitude i longitude.
        fld     dword ptr [ebp+20]      # Stek: rad, y0, x0, z0, long1
        fmul    st(4)                   # Stek: rad, y0, x0, z0, long1*rad
        fsincos                         # Stek: rad, y0, x0, z0, sin(long1*rad), cos(long1*rad)
        fld     dword ptr [ebp+16]      # Stek: rad, y0, x0, z0, sin(long1*rad), cos(long1*rad), lat1
        fmul    st(6)                   # Stek: rad, y0, x0, z0, sin(long1*rad), cos(long1*rad)
        fsincos                         # Stek: rad, y0, x0, z0, sin(long1*rad), cos(long1*rad), sin(lat1*rad)=z1, cos(lat1*rad)
        fmul    st(3), st               # Stek: rad, y0, x0, z0, sin(long1*rad)*cos(lat1*rad)=y1, cos(long1*rad), z1, cos(lat1*rad)
        fmulp   st(2), st               # Stek: rad, y0, x0, z0, y1, cos(long1*rad)*cos(lat1*rad)=x1, z1

        ## Racuna se skalarni proizvod vektora Dekartovih koordinata
	## datih tacaka, odn. time i kosinus ugla izmedju tih vektora.
        fmulp   st(3), st               # Stek: rad, y0, x0, z0*z1, y1, x1
        fmulp   st(3), st               # Stek: rad, y0, x0*x1, z0*z1, y1
        fmulp   st(3), st               # Stek: rad, y0*y1, x0*x1, z0*z1
        faddp                           # Stek: rad, y0*y1, x0*x1+z0*z1
        faddp                           # Stek: rad, y0*y1+x0*x1+z0*z1=cos(theta)
        fxch                            # Stek: cos(theta), rad
        ffree   st                      # Stek: cos(theta), invalidna vrijednost
        fincstp                         # Stek: cos(theta)

        ## Za racunanje inverznog kosinusa koristi se identitet:
	##   acos(x)=atan(sqrt(1-x^2)/x)
        ## Vrijednost x je cos(theta) odn. u intervalu [-1,1], tako da
	## je onda 1-x^2 sigurno vece ili jednako od 0 i na tu
	## vrijednost se moze primijeniti sqrt instrukcija.
        fld     st                      # Stek: cos(theta)=x, x
        fmul    st, st                  # Stek: x, x*x=x^2
        fld1                            # Stek: x, x^2, 1
        fsubrp                          # Stek: x, 1-x^2
        fsqrt                           # Stek: x, sqrt(1-x^2)
        fxch                            # Stek: sqrt(1-x^2), x
        fpatan                          # Stek: atan(sqrt(1-x^2)/x)=acos(x)=theta

        ## Izracunava se najkrace rastojanje medju tackama.
        fld     dword ptr r             # Stek: theta, r
        fmulp                           # Stek: r*theta
        
        ## Epilog funkcije.
        leave
        ret
