.intel_syntax noprefix
        
.text
.global projection

	## Funkcija projection() izracunava projekciju date tacke na
	## datu pravu.  Argumenti funkcije su:
        ##   [ebp+8] - adresa na kojoj se nalaze koordinate tacke P
        ##   [ebp+12], [ebp+16] - adrese na kojima se nalaze koordinate
        ##                        tacaka P0 i P1 kojima je odredjena
	##                        prava
        ##   [ebp+20] - adresa na koju treba smjestiti koordinate tacke
	##              P' koja predstavlja projekciju date tacke na
	##              datu pravu
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
projection:
        ## Prolog funkcije.
	enter   0, 0
        push    esi
        push    edi

        ## Registri esi, ecx i edx ce respektivno sadrzati adrese sa
        ## koordinatama date tacke P, odn. tacaka P0 i P1 kojima je
        ## odredjena data prava.  Registar edi ce sadrzati adresu na
	## koju treba smjestiti koordinate izracunate tacke P'.
        mov     esi, [ebp+8]
        mov     ecx, [ebp+12]
        mov     edx, [ebp+16]
        mov     edi, [ebp+20]

        ## Na stek se ucitavaju koordinate tacke P0.
        fld     dword ptr [ecx]         # Stek: P0[X]
        fld     dword ptr [ecx+4]       # Stek: P0[X], P0[Y]

        ## Na steku se formira vektor pravca D date prave i potom se
        ## isti vektor normalizuje.  Norma ovog vektora se oznacava sa
	## n, a dobijeni jedxnicni vektor sa D1.
        fld     dword ptr [edx]         # Stek: P0[X], P0[Y], P1[X]
        fsub    st(2)                   # Stek: P0[X], P0[Y], P1[X]-P0[X]=D[X]
        fld     dword ptr [edx+4]       # Stek: P0[X], P0[Y], D[X], P1[Y]
        fsub    st(2)                   # Stek: P0[X], P0[Y], D[X], P1[Y]-P0[Y]=D[Y]
        fld     st(1)                   # Stek: P0[X], P0[Y], D[X], D[Y], D[X]
        fmul    st(2)                   # Stek: P0[X], P0[Y], D[X], D[Y], D[X]^2
        fld     st(1)                   # Stek: P0[X], P0[Y], D[X], D[Y], D[X]^2, D[Y]
        fmul    st(2)                   # Stek: P0[X], P0[Y], D[X], D[Y], D[X]^2, D[Y]^2
        faddp                           # Stek: P0[X], P0[Y], D[X], D[Y], D[X]^2+D[Y]^2
        fsqrt                           # Stek: P0[X], P0[Y], D[X], D[Y], sqrt(D[X]^2+D[Y]^2)=n
        fdiv    st(2), st               # Stek: P0[X], P0[Y], D[X]/n=D1[X], D[Y], n
        fdivp                           # Stek: P0[X], P0[Y], D1[X], D[Y]/n=D1[Y]

        ## Odredjuje se skalarni proizvod vektora od tacke P0 na pravoj
        ## do date tacke P sa normalizovanim vektorom pravca D1 date
	## prave; vrijednost ovog skalarnog proizvoda se oznacava sa t.
        fld     dword ptr [esi]         # Stek: P0[X], P0[Y], D1[X], D1[Y], P[X]
        fsub    st(0), st(4)            # Stek: P0[X], P0[Y], D1[X], D1[Y], P[X]-P0[X]
        fmul    st, st(2)               # Stek: P0[X], P0[Y], D1[X], D1[Y], (P[X]-P0[X])*D1[X]
        fld     dword ptr [esi+4]       # Stek: P0[X], P0[Y], D1[X], D1[Y], (P[X]-P0[X])*D1[X], P[Y]
        fsub    st(0), st(4)            # Stek: P0[X], P0[Y], D1[X], D1[Y], (P[X]-P0[X])*D1[X], P[Y]-P0[Y]
        fmul    st, st(2)               # Stek: P0[X], P0[Y], D1[X], D1[Y], (P[X]-P0[X])*D1[X], (P[Y]-P0[Y])*D1[Y]
        faddp                           # Stek: P0[X], P0[Y], D1[X], D1[Y], (P[X]-P0[X])*D1[X]+(P[Y]-P0[Y])*D1[Y]=t

        ## Izracunava se projekcija P' date tacke P na datu pravu i
	## njene koordinate smjestaju u za to predvidjene lokacije.
        fmul    st(2), st               # Stek: P0[X], P0[Y], t*D1[X], D1[Y], t
        fmulp                           # Stek: P0[X], P0[Y], t*D1[X], t*D1[Y]
        faddp   st(2), st               # Stek: P0[X], P0[Y]+t*D1[Y]=P'[X], t*D1[X]
        faddp   st(2), st               # Stek: P0[X]+t*D1[X]=P'[X], P'[Y]
        fstp    dword ptr [edi+4]
        fstp    dword ptr [edi]

	## Epilog funkcije.
        pop     edi
        pop     esi
	leave
        ret
