.intel_syntax noprefix

.text
.global convert

       	## Funkcija convert() konvertuje dati broj u reprezentaciju u
	## "bazi faktorijela".  Argumenti funkcije su:
        ##   [ebp+8] - dati broj
        ##   [ebp+12] - adresa prvog elementa u nizu u koji treba
        ##              smjestiti cifre reprezentacije datog broja u
	##              "bazi faktorijela".
        ##   [ebp+16] - adresa na koju treba smjestiti broj cifara
        ##              reprezentacije datog broja u "bazi faktorijela"
        ## Pretpostavka je da je dati broj nenegativan, kao i da je niz
        ## u koji treba smjestiti cifre reprezentacije datog broja u
        ## "bazi faktorijela" takve duzine da moze da prihvati sve
        ## izracunate cifre.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
convert:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Inicijalizuje se direction fleg.
        cld

        ## Dati broj se smjesta u registar ebx, i procesira se slucaj
	## kada je taj broj jednak 0 kao specijalni slucaj.
        mov     ebx, [ebp+8]
        cmp     ebx, 0
        ja positive
        xor     eax, eax
        mov     edi, [ebp+12]
        stosd
        inc     eax
        mov     edi, [ebp+16]
        mov     [edi], eax
        jmp done
        
positive:
        ## Odredjuje se najmanji faktorijel koji je veci od datog broja.
        ## U registru eax se izracunava faktorijel, dok registar ecx
	## sadrzi broj ciji je to faktorijel.
        mov     ecx, 1
        mov     eax, 1
next_factoriel:
        cmp     eax, ebx
        ja update_factoriel
        inc     ecx
        mul     ecx
        jmp next_factoriel

update_factoriel:
        ## Azurira se vrijednost faktorijela, i broja ciji je to
	## faktorijel, tako da se dobije najveci faktorijel manji ili
	## jednak datom broju.
        xor     edx, edx
        div     ecx
        dec     ecx

        ## Upisuje se, u za to predvidjenu lokaciju u memoriji, broj
	## cifara reprezentacije datog broja u "bazi faktorijela".
        mov     edi, [ebp+16]
        mov     [edi], ecx

        ## Odredjuju se cifre datog broja u bazi faktorijela.  Registar
        ## edi ce pokazivati na lokaciju u memoriji na koju treba
	## upisati tekucu cifru.
        mov     edi, [ebp+12]
next_digit:
        ## Odredjuje se koliko puta se tekuci faktorijel sadrzi u datom
        ## broju, i ta vrijednost predstavlja tekucu cifru
        ## reprezentacije broja u "bazi faktorijela".  Ostatak pri ovom
	## dijeljenju predstavlja broj kome se dalje trazi reprezentacija u
	## "bazi faktorijela".
        xchg    eax, ebx
        xor     edx, edx
        div     eax, ebx
        stosd
        mov     eax, edx

        ## U narednoj iteraciji, prelazi se na prvi manji faktorijel od
	## tekuceg faktorijela.
        xchg    eax, ebx
        xor     edx, edx
        div     ecx
        loop next_digit

done:        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
