.intel_syntax noprefix

.text
.global collisions

	## Funkcija collisions() provjerava koliko kolizija sa datom hash 
	## funkcijom pravi dati niz brojeva. Argumenti funkcije su:
	##   [ebp+8] - duzina hash tabele
	##   [ebp+12] - broj identifikatora koje treba unijetu u hash tabelu
	##   [ebp+16] - adresa prvog elementa u polju identifikatora
	## Funkcija vraca broj ulaza hash tabele na kojima se javila 
	## kolizija. Funkcija se u potpunosti podvrgava C konvencijama 
	## pozivanja.
collisions:
        ## Prolog funkcije.
        enter	0, 0
	push	ebx
        push	esi
	push	edi

	## Alocira se memorija na steku za polje koje ce sadrzati broj
	## pogodaka za svaki ulaz hash tabele. Registar esp ce drzati
	## adresu prvog elementa ovog polja.
	mov	eax, [ebp+8]
	shl	eax, 2
	sub	esp, eax

	## Popunjava se gornje polje nulama.
	cld
	mov	edi, esp
	xor	eax, eax
	mov	ecx, [ebp+8]
	rep stosd

	## Simulira se unosenje identifikatora u hash tabelu i pritom se
	## za svaki ulaz tabele broji koliko je identifikatora uneseno u
	## njega. Registar ecx ce biti koriscen kao brojac i inicijalizuje
	## se na broj identifikatora. Registar esi ce drzati adresu na kojoj
	## se nalazi tekuci identifikator. Registar ebx ce drzati adresu 
	## prvog elementa polja koje odgovara hash tabeli.
	mov	ecx, [ebp+12]
	mov	esi, [ebp+16]
	mov	ebx, esp

next_id:
	## Ucitava se identifikator.
	lodsd

	## Izracunava se data hash funkcija.
	mov	edx, eax
	shr	eax, 8
	xor	eax, edx
	xor	edx, edx
	div	dword ptr [ebp+8]

	## Inkrementira se broj pogodaka za odgovarajuci element polja
	## koje odgovara hash tabeli.
	inc	dword ptr [ebx+4*edx]

	## Prelazi se na naredni identifikator.
	loop next_id

	## Broje se kolizije u hash tabeli, tj. odredjuje se broj elemenata
	## polja koje odgovara hash tabeli koji su veci od 1. Registar ecx
	## se koristi kao brojac i inicijalizuje se na duzinu tabele.
	## Registar esi ce drzati adresu polja koje odgovara tabeli. U
	## registru edi ce biti akumuliran broj elemenata vecih od 1.
	mov	ecx, [ebp+8]
	mov	esi, esp
	xor	edx, edx

next_entry:
	## Ucitava se broj pogodaka za dati ulaz hash tabele.
	lodsd

	## Ukoliko je broj pogodaka veci od 1 inkrementira se broj kolizija.
	cmp	eax, 1
	jle continue
	inc	edx
	
continue:
	## Prelazi se na naredni element polja koje odgovara hash tabeli.
	loop next_entry

	## Oslobadja se memorija koriscena za polje koje odgovara hash
	## tabeli.
	mov	eax, [ebp+8]
	shl	eax, 2
	add	esp, eax

	## Broj kolizija se smjesta u registar eax.
	mov	eax, edx

	## Epilog funkcije.
	pop	edi
	pop	esi
	pop	ebx
        leave
        ret
