.intel_syntax noprefix

.text
.global intersect

	## Funkcija intersect() odredjuje da li se data prava presijeca 
	## sa datom kruznicom sa centrom u koordinatnom pocetku.
	## Argumenti funkcije su:
	##   [ebp+8] - poluprecnik kruznice
	##   [ebp+12], [ebp+16] - koordinate tacke na pravoj
	##   [ebp+20], [ebp+24] - koordinate vektora pravca prave
	## Pretpostavlja se da je poluprecnik kruznice pozitivan, kao i
	## da je vektor pravca prave razlicit od 0. Funkcija vraca 1 ako
	## presjek postoji, odn. 0 u suprotnom. Funkcija se u potpunosti 
	## podvrgava C konvencijama pozivanja.
intersect:
        ## Prolog funkcije.
        enter	0, 0

	## Ukoliko se napise parametarska jednacina prave x=x0+t*dx,
	## y=y0+t*dx i potom racuna presjek sa kruznicom x^2+y^2=r^2
	## dobija se da su parametarske koordinate presjeka odredjene
	## kvadratnom jednacinom po t oblika a*t^2+b*t+c=0, pri cemu je
	## a=dx^2+dy^2, b=2*(x0*dx+y0*dy) i c=x0^2+y0^2-r^2. Ukoliko
	## jednacina ima realnih rjesenja, onda presjek postoji.

	## Odredjuje se koeficijenat b/2 u gore opisanoj kvadratnoj
	## jednacini.
	fld	dword ptr [ebp+12]	# Stek:	x0
	fld	dword ptr [ebp+20]	# Stek:	x0, dx
	fmulp   			# Stek:	x0*dx
	fld	dword ptr [ebp+16]	# Stek:	x0*dx, y0
	fld	dword ptr [ebp+24]	# Stek:	x0*dx, y0, dy
	fmulp   			# Stek:	x0*dx, y0*dy
	faddp   			# Stek:	x0*dx+y0*dy=b/2

	## Odredjuje se (b/2)^2 kao prvi korak pri racunanju diskriminante.
	fld	st		# Stek:	b/2, b/2
	fmulp   		# Stek:	(b/2)^2

	## Odredjuje se koeficijent a kvadratne jednacine.
	fld	dword ptr [ebp+20]	# Stek:	(b/2)^2, dx
	fld	st			# Stek:	(b/2)^2, dx, dx
	fmulp   			# Stek:	(b/2)^2, dx^2
	fld	dword ptr [ebp+24]	# Stek:	(b/2)^2, dx^2, dy
	fld	st			# Stek:	(b/2)^2, dx^2, dy, dy
	fmulp   			# Stek:	(b/2)^2, dx^2, dy^2
	faddp   			# Stek:	(b/2)^2, dx^2+dy^2=a

	## Odredjuje se koeficijent c kvadratne jednacine.
	fld	dword ptr [ebp+12]	# Stek:	(b/2)^2, a, x0
	fld	st			# Stek:	(b/2)^2, a, x0, x0
	fmulp   			# Stek:	(b/2)^2, a, x0^2
	fld	dword ptr [ebp+16]	# Stek:	(b/2)^2, a, x0^2, x1
	fld	st			# Stek:	(b/2)^2, a, x0^2, x1, x1
	fmulp   			# Stek:	(b/2)^2, a, x0^2, x1^2
	faddp   			# Stek:	(b/2)^2, a, x0^2+x1^2
	fld	dword ptr [ebp+8]	# Stek:	(b/2)^2, a, x0^2+x1^2, r
	fld	st			# Stek:	(b/2)^2, a, x0^2+x1^2, r, r
	fmulp   			# Stek:	(b/2)^2, a, x0^2+x1^2, r^2
	fsubp   			# Stek:	(b/2)^2, a, x0^2+x1^2-r^2=c

	## Odredjuje se cetvrtina diskriminanta jednacine.
	fmulp   		# Stek:	(b/2)^2, a*c
	fsubp   		# Stek:	(b/2)^2-4*a*c=(b^2-4*a*c)/4

	## Na osnovu znaka diskriminante jednacine (znak cetvrtine
        ## diskriminante jednak je znaku diskriminante) odredjuje se
	## koliko jednacina ima realnih korijena. Ako jednacina ima
        ## realnih korijena vraca se 1 kao rezultat, u suprotnom vraca
	## se 0.
	fldz			# Stek:	(b^2-4*a*c)/4, 0
	fcomip st, st(1)	# Stek:	(b^2-4*a*c)/4
	jbe has_roots
	xor	eax, eax
	jmp finish
has_roots:
	mov	eax, 1

        ## Prazni se stek.
        ffree st                # Stek:	invalidna vrijednost
	fincstp                 # Stek:

finish:	
        ## Epilog funkcije.
        leave
        ret
