.intel_syntax noprefix
			
.text
.global quicksort

        ## Funkcija quicksort() sortira segment datog niza bajtova
        ## quicksort algoritmom. Argumenti funkcije su:
        ##   [ebp+8] - niz koga treba sortirati
        ##   [ebp+12], [ebp+16] - indeksi koji odredjuju dio niza koji
        ##                        se sortira
        ## Funkcija ne vraca nikakvu vrijednost. Funkcija se u
        ## potpunosti podvrgava C konvencijama pozivanja.
quicksort:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Prebacuju se argumenti funkcije u registre. Registar ebx ce
        ## drzati adresu polja, registri esi i edi indekse koji
        ## ogranicavaju segment niza koga treba sortirati.
        mov     ebx, [ebp+8]
        mov     esi, [ebp+12]
        mov     edi, [ebp+16]

        ## Ukoliko je donji indeks veci ili jednak gornjem, odmah se
        ## izlazi iz funkcije.
        cmp     esi, edi
        jge sort_done

        ## Bira se prvi element niza da bude pivotirajuci element i isti
        ## se stavlja u edx registar.
        mov	edx, [ebx+esi*4]
	
        ## Quicksort algoritam je implementiran tako sto se prolazi kroz
        ## citav niz pocev od elementa iza pivotirajuceg elementa i svi
        ## elementi koji su manji ili jednaki pivotirajucem se prebacuju
        ## prema pocetku niza. Registar ecx ce drzati indeks tekuceg
        ## elementa niza, a registar eax ce drzati indeks elementa sa
        ## pocetka niza iza koga moze da se ubaci naredni element manji
        ## ili jednak pivotirajucem. Na pocetku se inicijalizuju ovi
        ## registri.
        mov     eax, esi
        mov     ecx, esi
        inc     ecx

next_element:
        ## Ako je pivotirajuci element manji od tekuceg, odmah se
        ## prelazi na naredni element.
        cmp     edx, [ebx+ecx*4]
        jl swap_done

        ## Ako je tekuci element manji ili jednak od pivotirajuceg, onda
        ## se taj element stavlja na kraj grupe elemenata na pocetku
        ## niza koji su svi manji ili jednaki pivotirajucem. Kako je vec
        ## pomenuto, indeks poslednjeg elementa iz te grupe drzi se u 
        ## registru eax. Prebacivanje se vrsi swap-ovanjem, pri cemu
        ## to swap-ovanje ide preko registra edx, da bi se potom
	## pivotirajuci element vratio u registar edx.
        inc     eax
        mov     edx, [ebx+ecx*4]
        xchg    edx, [ebx+eax*4]
        mov     [ebx+ecx*4], edx
        mov	edx, [ebx+esi*4]

swap_done:
        ## Prelazi se na sledeci elemenat niza i provjerava uslov
        ## izlaska iz petlje.
        inc     ecx
        cmp     ecx, edi
        jle next_element

        ## Pivotirajuci element se postavlja na svoje mjesto u nizu u
        ## smislu sortiranja, sto se sprovodi swap-ovanjem sa elementom
        ## koji se trenutno tamo nalazi. Pivotirajuci element se uzima
        ## iz registra edx.
        xchg    edx, [ebx+eax*4]
        mov     [ebx+esi*4], edx

        ## Rekurzivno se poziva funkcija quicksort() da sortira dio niza
        ## koji je ostao lijevo od pivotirajuceg elementa.
        push    eax
        dec     eax
        push    eax
        push    esi
        mov     ebx, [ebp+8]
        push    ebx
        call    quicksort
        add     esp, 12
        pop     eax

        ## Rekurzivno se poziva funkcija quicksort() da sortira dio niza
        ## koji je ostao desno od pivotirajuceg elementa.
        push    edi
        inc     eax
        push    eax
        mov     ebx, [ebp+8]
        push    ebx
        call    quicksort
        add     esp, 12
	
sort_done:
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
