.intel_syntax noprefix

.text
.global newton

	## Funkcija newton() izracunava n-ti korijen datog broja a
        ## koriscenjem Newton-ove iterativne metode za rjesavanje
	## jednacine x^n-a=0. Argumenti funkcije su:
	##   [ebp+8] - korijen koji se racuna
	##   [ebp+12] - broj ciji se korijen racuna
	##   [ebp+16] - pocetna iteracija
	##   [ebp+20] - trazena tacnost
       	##   [ebp+24] - maksimalni dozvoljeni broj iteracija
        ## Pretpostavlja se da je korijen broj veci ili jednak 2, da su
	## broj ciji se korijen racuna i pocetna iteracija nenegativni
	## brojevi, kao i da su tacnost i maksimalni broj iteracija
        ## pozitivni brojevi.  Funkcija vraca izracunatu vrijednost
	## korijena. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
newton:
        ## Prolog funkcije.
        enter	0, 0

        ## Registar ecx ce sluziti kao brojac za iteracije, dok ce se u
	## registru edx cuvati vrijednost n-1.
        mov     ecx, [ebp+24]
        mov     edx, [ebp+8]
        dec     edx

        ## Ucitavaju se na stek argumenti procedure koji se koriste 
	## tokom racunanja.  Sa x(k) je oznacena vrijednost dobijena u
	## k-toj iteraciji Newton-ovog iterativnog postupka.
        fild    dword ptr [ebp+8]       # Stek: n
        fld     [ebp+12]                # Stek: n, a
        fld     [ebp+16]                # Stek: n, a, x0=x(k)
        fld     [ebp+20]                # Stek: n, a, x(k), eps

        ## Sprovodi se Newton-ov iterativni postupak.
next_iteration:
        ## Na steku se formira vrijednost x(k)^(n-1).  Ova vrijednost ce
	## biti izracunata kao produkt P=b(0)*x(k)*b(1)*x(k)^2+..., gdje su
	## b(0), b(1),... bitovi (gledano od bita najmanje tezine) broja
	## n-1.  U oznakama ce sa P(i) biti oznacen parcijalni produkt u
        ## koji su ukljuceni stepeni broja x(k) do 2^i.  Radi izdvajanja
	## bitova iz broja n-1, ovaj broj ce biti kopiran iz registra
	## edx u registar eax.
        fld1                            # Stek: n, a, x(k), eps, 1=P(0)
        fld     st(2)                   # Stek: n, a, x(k), eps, P(0), x(k)=x(k)^(2^(i+1))
        mov     eax, edx
        
next_bit:
        ## Siftuje se registar eax za jedan bit udesno i ukoliko je taj
	## bit jednak 1, parcijalni produkt se mnozi sa x(k)^(2^(i+1)).
        shr     eax, 1
        jnc square
        fmul    st(1), st               # Stek: n, a, x(k), eps, P(i)*x(k)^(2^(i+1))=P(i+1), x(k)^(2^(i+1))

square:
        ## Kvadrira se tekuci stepen broja x(k), da bi se dobio stepen
	## potreban za naredni element parcijalnog produkta.
        fmul    st, st                  # Stek: n, a, x(k), eps, P(i+1), x(k)^(2^(i+1))*x(k)^(2^(i+1))=x(k)^(2^((i+1)+1))

        ## Provjerava se da li je racunanje parcijalnog produkta
	## zavrseno.
        cmp     eax, 0
        jne next_bit

        ## Skida se sa vrha steka pomocna vrijednost koja je bila
	## koriscena pri racunanju x(k)^(n-1).
        ffree   st                      # Stek: n, a, x(k), eps, x(k)^(n-1), invalidna vrijednost
        fincstp                         # Stek: n, a, x(k), eps, x(k)^(n-1)

        ## Izracunava se vrijednost x(k+1) prema Newton-ovoj formuli.
	## Pritom, na vrhu steka ostaje razlika x(k)-x(k+1), koja je
	## potrebna za provjeru da li je dostignuta zeljena tacnost.
        fld     st                      # Stek: n, a, x(k), eps, x(k)^(n-1), x(k)^(n-1)
        fmul    st, st(3)               # Stek: n, a, x(k), eps, x(k)^(n-1), x(k)^(n-1)*x(k)=x(k)^n
        fsub    st, st(4)               # Stek: n, a, x(k), eps, x(k)^(n-1), x(k)^n-a
        fdivrp                          # Stek: n, a, x(k), eps, (x(k)^n-a)/x(k)^(n-1)
        fdiv    st, st(4)               # Stek: n, a, x(k), eps, (x(k)^n-a)/(n*x(k)^(n-1))
        fsub    st(2), st               # Stek: n, a, x(k)-(x(k)^n-a)/(n*x(k)^(n-1))=x(k+1), eps, (x(k)^n-a)/(n*x(k)^(n-1))=x(k)-x(k+1)

        ## Provjerava se da li je dostignuta zeljena tacnost; ako jeste,
	## prekida se petlja, a ako nije nastavlja se sa iteracijama.
        fabs                            # Stek: n, a, x(k+1), eps, abs(x(k)-x(k+1))
        fcomip  st, st(1)               # Stek: n, a, x(k+1), eps
        jbe finish
        loop next_iteration

finish:
        ## Brisu se pomocne vrijednosti koje su koriscene tokom
	## iterativnog postupka sa steka i na steku ostaje samo rezultat
	## izracunavanja.
        ffree   st                      # Stek: n, a, x, invalidna vrijednost
        fincstp                         # Stek: n, a, x
        fxch    st(2)                   # Stek: x, a, n
        ffree   st                      # Stek: x, a, invalidna vrijednost
        fincstp                         # Stek: x, a
        ffree   st                      # Stek: x, invalidna vrijednost
        fincstp                         # Stek: x

        ## Epilog funkcije.
        leave
        ret
