.intel_syntax noprefix
                
.text
.global substring

        ## Funkcija substring() odredjuje najduzi zajednicki 
	## podstring dva data stringa. Argumenti funkcije su:
        ##   [ebp+8] - prvi string
        ##   [ebp+12] - drugi string
        ##   [ebp+16] - adresa na koju treba upisati adresu prvog karaktera
	##              najduzeg zajednickog podstringa
        ##   [ebp+20] - adresa na koju treba upisati duzinu najduzeg
	##              zajednickog podstringa
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
substring:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Inicijalizuje se direction flag, a registar esi se postavlja
	## na adresu prvog karaktera prvog stringa.
        cld
        mov     esi, [ebp+8]

        ## Incijalizuju se rezultati funkcije.
        mov     eax, [ebp+16]
        mov     dword ptr [eax], 0
        mov     eax, [ebp+20]
        mov     dword ptr [eax], 0

        ## Radi izracunavanja duzine stringova, registar eax se
	## postavlja na 0.
        xor     eax, eax

        ## Izracunava se duzina drugog stringa i pamti u registru edx.
	## Ako je u pitanju prazan string, odmah se zavrsava funkcija.
        mov     edi, [ebp+12]
        mov     ecx, -1
        repnz scasb
        add     ecx, 2
        neg     ecx
        jecxz finish
        mov     edx, ecx

        ## Izracunava se duzina prvog stringa.  Ako je u pitanju prazan
	## string, odmah se zavrsava funkcija.
        mov     edi, [ebp+8]
        mov     ecx, -1
        repnz scasb
        add     ecx, 2
        neg     ecx
        jecxz finish

        ## Odredjuje se najduzi zajednicki podstring.  Za svaki karakter
        ## prvog stringa redom se pronalaze sve pojave tog karaktera u
	## drugom stringu, a zatim se ispituje koliko je narednih
	## karaktera identicno u oba stringa i na taj nacin se dolazi do
	## zajednickih podstringova.  Pamti se najduzi od tih
	## podstringova.
next_char:
        ## Ucitava se tekuci karakter prvog stringa.
        lodsb

        ## Pamte se broj preostalih karaktera i adresa narednog
	## karaktera za prvi string.
        push    ecx
        push    esi

        ## U registar edi se smjesta adresa prvog karaktera drugog
	## stringa.
        mov     edi, [ebp+12]
        
next_match:
        ## Odredjuje se naredna pojava tekuceg karaktera prvog stringa u
        ## drugom stringu.  Ukoliko nema pojava tekuceg karaktera prvog
        ## stringa u drugom stringu, prelazi se na naredni karakter
	## prvog stringa.
        mov     ecx, edx
        repnz scasb
        jnz continue

        ## Pamte se broj preostalih karaktera i adresa narednog
        ## karaktera za drugi string, kao i adresa narednog karaktera za
	## prvi string.
        push    ecx
        push    edi
        push    esi
        
        ## Odredjuje se koliko se karaktera koji slijede uparene 
        ## karaktere poklapaju u oba stringa.  Poredi se maksimalno
        ## onoliko karaktera koliko je preostalo u drugom stringu; treba
	## primijetiti da pritom ne moze doci do citanja karaktera iza
	## kraja prvog stringa u slucaju da je taj string kraci od prvog,
	## jer ce tada poredjenje biti zaustavljeno na terminirajucem
        ## karakteru prvog stringa.  Takodje, treba primijetiti da se
        ## u slucaju kada se uparivanje prekine zato sto se doslo do
        ## kraja drugog stringa mora (zbog nacina kako funkcionise repz
        ## prefiks - brojac se uvijek dekrementira i potom se obavlja
	## komparacija, a tek onda se provjerava da li treba zavrsiti
        ## petlju) eksplicitno provjeriti rezultat poslednje komparacije
        ## i azurirati brojac u skladu sa istim.
        repz cmpsb
        jz calculate_length
        inc     ecx

calculate_length:       
        ## U registru ebx se izracunava kolika je duzina pronadjenog
        ## zajednickog podstringa.  Prvo se zaracunava inicijalno
        ## upareni karakter, a zatim se zaracunavaju i upravo odredjeni
	## karakteri koji se poklapaju iza ovih karaktera.
        mov     ebx, 1
        add     ebx, [esp+8]
        sub     ebx, ecx

        ## Vracaju se broj preostalih karaktera i adresa narednog
        ## karaktera za drugi string, kao i adresa narednog karaktera za
	## prvi string.
        pop     esi
        pop     edi
        pop     ecx

        ## Poredi se duzina upravo odredjenog podstringa sa duzinom
	## najduzeg do sada pronadjenog podstringa.  Ukoliko nije u
	## pitanju podstring duzi od najduzeg do sada pronadjenog
        ## podstringa, prelazi se na odredjivanje naredne pojave tekuceg
	## karaktera prvog stringa u drugom stringu.
        push    eax
        mov     eax, [ebp+20]
        cmp     ebx, [eax]
        pop     eax
        jng next_match

        ## Ukoliko je pak u pitanju podstring duzi od najduzeg do sada
        ## pronadjenog podstringa, pamte se adresa prvog karaktera i
        ## duzina tog podstringa i opet se prelazi na odredjivanje
	## naredne pojave tekuceg karaktera prvog stringa u drugom
	## stringu.
        push    eax
        mov     eax, [ebp+16]
        mov     [eax], esi
        dec     dword ptr [eax]
        mov     eax, [ebp+20]
        mov     [eax], ebx
        pop     eax
        jmp next_match
        
continue:
        ## Prelazi se na naredni karakter prvog stringa.
        pop     esi
        pop     ecx
        loop next_char

finish: 
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
