.intel_syntax noprefix

.data
gamma:  .int    5               # Koeficijent gamma.
N:      .int    6               # Duzina polja koeficijenata c.
c:      .float  1.0000000002, 76.1800917295, -86.5053203294, 24.0140982408, -1.2317395725, 0.0012086510, 0.0000053952 # Polje koeficijenata c.
two:    .int    2               # Broj 2.
        
.text
.global lanczos

	## Funkcija lanczos() racuna Lanczos-ovu ocjenu gama-funkcije za
	## vrijednost z+1 pri cemu je vrijednost z data.
	## Argumenti funkcije su:
	##   [ebp+8] - vrijednost z
        ## Pretpostavlja se da je vrijednost argumenta u intervalu
        ## (0,10]. Funkcija vraca izracunatu vrijednost Lanczos-ove
	## ocjene.  Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
lanczos:
        ## Prolog funkcije.
        enter	0, 0

        ## Inicijalizuje se stek.
        fld1                            # Stek: 1
        fidiv   dword ptr two           # Stek: 1/2
        fadd    dword ptr [ebp+8]       # Stek: z+1/2
        fld     st                      # Stek: z+1/2, z+1/2
        fiadd   dword ptr gamma         # Stek: z+1/2, z+gamma+1/2
        fxch                            # Stek: z+gamma+1/2, z+1/2
        fld     st(1)                   # Stek: z+gamma+1/2, z+1/2, z+gamma+1/2

        ## Izracunava se vrijednost (z+gamma+1/2)^(z+1/2) kao prvi
	## faktor Lanczos-ove ocjene.
        fyl2x                           # Stek: z+gamma+1/2, (z+1/2)*log2(z+gamma+1/2)=log2((z+gamma+1/2)^(z+1/2))=L
        fld     st                      # Stek: z+gamma+1/2, L, L
        frndint                         # Stek: z+gamma+1/2, L, int(L)
        fsub    st(1), st               # Stek: z+gamma+1/2, L-int(L)=frac(L), int(L)
        fxch                            # Stek: z+gamma+1/2, int(L), frac(L)
        f2xm1                           # Stek: z+gamma+1/2, int(L), 2^frac(L)-1
        fld1                            # Stek: z+gamma+1/2, int(L), 2^frac(L)-1, 1
        faddp                           # Stek: z+gamma+1/2, int(L), 2^frac(L)
        fscale                          # Stek: z+gamma+1/2, int(L), 2^int(L)*2^frac(L)=2^(int(L)+frac(L))=2^L
        fxch                            # Stek: z+gamma+1/2, 2^L, int(L)
        fcomp                           # Stek: z+gamma+1/2, 2^L=2^log2((z+gamma+1/2)^(z+1/2))=(z+gamma+1/2)^(z+1/2)
        fxch                            # Stek: (z+gamma+1/2)^(z+1/2), z+gamma+1/2

        ## Izracunava se vrijednost e^(-(z+gamma+1/2)) kao drugi
	## faktor Lanczos-ove ocjene i mnozi se sa prvim faktorom.
        fchs                            # Stek: (z+gamma+1/2)^(z+1/2), -(z+gamma+1/2)
        fldl2e                          # Stek: (z+gamma+1/2)^(z+1/2), -(z+gamma+1/2), log2(e)
        fmulp                           # Stek: (z+gamma+1/2)^(z+1/2), -(z+gamma+1/2)*log2(e)=log2(e^(-(z+gamma+1/2)))=M
        fld     st                      # Stek: (z+gamma+1/2)^(z+1/2), M, M
        frndint                         # Stek: (z+gamma+1/2)^(z+1/2), M, int(M)
        fsub    st(1), st               # Stek: (z+gamma+1/2)^(z+1/2), M-int(M)=frac(M), int(M)
        fxch                            # Stek: (z+gamma+1/2)^(z+1/2), int(M), frac(M)
        f2xm1                           # Stek: (z+gamma+1/2)^(z+1/2), int(M), 2^frac(M)-1
        fld1                            # Stek: (z+gamma+1/2)^(z+1/2), int(M), 2^frac(M)-1, 1
        faddp                           # Stek: (z+gamma+1/2)^(z+1/2), int(M), 2^frac(M)
        fscale                          # Stek: (z+gamma+1/2)^(z+1/2), int(M), 2^int(M)*2^frac(M)=2^(int(M)+frac(M))=2^M
        fxch                            # Stek: (z+gamma+1/2)^(z+1/2), 2^M, int(M)
        fcomp                           # Stek: (z+gamma+1/2)^(z+1/2), 2^M=2^log2(e^(-(z+gamma+1/2)))=e^(-(z+gamma+1/2))
        fmulp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))

        ## Izracunava se vrijednost sqrt(2*pi) kao treci faktor
	## Lanczos-ove ocjene i mnozi se sa prva dva faktora.
        fild    dword ptr two           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2)), 2
        fldpi                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2)), 2, pi
        fmulp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2)), 2*pi
        fsqrt                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2)), sqrt(2*pi)
        fmulp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi)

        ## Izracunava se suma c[0]+c[1]/(z+1)+...+c[6]/(z+6) kao cetvrti
	## faktor Lanczos-ove ocjene i mnozi se sa prva tri faktora.  U
	## komentarima je sa i oznacen indeks koraka pri izracunavanju
	## sume (i se krece od 0 do 6), a sa S(i) izracunata parcijalna
	## suma u tom koraku.
        xor     ecx, ecx
        fld     dword ptr c             # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), c[0]=S(0)
        fld     [ebp+8]                 # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(0), z=z+i
next_coefficient:
        inc     ecx
        cmp     ecx, N
        jg continue
        fld1                            # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(i), z+i, 1
        faddp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(i), z+(i+1)
        fld     dword ptr [4*ecx+c]     # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(i), z+(i+1), c[i+1]
        fdiv    st, st(1)               # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(i), z+(i+1), c[i+1]/(z+(i+1))
        faddp   st(2), st               # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(i)+c[i+1]/(z+(i+1))=S(i+1), z+(i+1)
        jmp next_coefficient
continue:
        fcomp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi), S(6)
        fmulp                           # Stek: (z+gamma+1/2)^(z+1/2)*e^(-(z+gamma+1/2))*sqrt(2*pi)*S(6)
        
        ## Epilog funkcije.
        leave
        ret
