.intel_syntax noprefix

.text
.global farey
        
        ## Funkcija farey() odredjuje brojilac i imenilac razlomka koji
        ## je k-ti po redu element u Farey-evoj seriji reda n.
	## Argumenti funkcije su:
        ##   [ebp+8] - red serije n
        ##   [ebp+12] - redni broj (pocev od 1) trazenog elementa serije
        ##   [ebp+16], [ebp+20] - adrese lokacija na koje treba upisati
        ##                        brojilac, odn. imenilac trazenog
	##                        elementa
        ## Pretpostavka je da je n > 0 i k > 0.  Funkcija vraca 1 ako
        ## pronadje trazeni element, odn. 0 u suprotnom (ako je k vece
	## od broja elemenata u seriji).  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
farey:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Registar ecx ce sluziti kao brojac i u njega se prebacuje
	## redni broj trazenog elementa sekvence.
        mov     ecx, [ebp+12]

        ## Provjerava se da li je trazen prvi element sekvence i u tom
	## slucaju se kao rezultat vraca 0/1.
        cmp     ecx, 1
        ja k_above_1
        mov     ebx, [ebp+16]
        xor     eax, eax
        mov     [ebx], eax
        inc     eax
        mov     ebx, [ebp+20]
        mov     [ebx], eax
        mov     eax, 1
        jmp     done

k_above_1:
        ## Provjerava se da li je trazen drugi element sekvence i u tom
	## slucaju se kao rezultat vraca 0/n, gdje je n red sekvence.
        cmp     ecx, 2
        ja k_above_2
        mov     ebx, [ebp+16]
        mov     eax, 1
        mov     [ebx], eax
        mov     eax, [ebp+8]
        mov     ebx, [ebp+20]
        mov     [ebx], eax
        mov     eax, 1
        jmp     done

k_above_2:
        ## Registar eax ce cuvati vrijednost x[k], a registar ebx
        ## vrijednost y[k].  Registar esi ce cuvati vrijednost x[k+1], a
        ## registar edi vrijednost y[k+1].  Inicijalizuju se vrijednosti
	## u ovim registrima.
        mov     eax, 0
        mov     ebx, 1
        mov     esi, 1
        mov     edi, [ebp+8]

        ## Obzirom da su ovim odredjena prva dva elementa sekvence,
	## oduzima se 2 od rednog broja trazenog elementa sekvence.
        sub     ecx, 2

again:
        ## Provjerava se da li su x[k+1] i y[k+1] postali jednaki 1; ako
        ## jesu, to znaci da je sekvenca zavrsena, odn. da je dati redni
	## broj veci od duzine sekvence.
        cmp     edi, esi
        jne continue
        cmp     esi, 1
        je not_found
        
continue:
        ## Vrijednost registra eax (koji sadrzi x[k]) se pamti na steku,
	## a u istom registru se izracunava vrijednost (y[k]+n)/y[k+1].
        push    eax
        mov     eax, ebx
        add     eax, [ebp+8]
        xor     edx, edx
        div     edi

        ## U registru eax se izracunava vrijednost y[k+2], koja se
        ## odlaze na mjesto vrijednosti y[k+1] (odn. u registar edi),
        ## dok se vrijednost y[k+1] stavlja na mjesto vrijednost y[k]
        ## (odn. u registar ebx).  Kolicnik (y[k]+n)/y[k+1] se tokom
        ## ovog izracunavanja cuva na steku, jer je potreban i za
	## izracunavanje vrijednosti x[k+2].
        push    eax
        mul     edi
        sub     eax, ebx
        mov     ebx, edi
        mov     edi, eax
        pop     eax

        ## U registru eax se izracunava vrijednost x[k+2] (za
        ## izracunavanje ove vrijednosti se koristi vrijednost x[k],
        ## koja je bila odlozena na stek).  Po zavrsenom izracunavanju
        ## se ova vrijednost stavlja na mjesto vrijednosti x[k+1]
        ## (odn. u registar edi), dok se vrijednost x[k+1] stavlja u
        ## registar eax (odn. prakticno na mjesto vrijednosti x[k]).  Na
        ## taj nacin, vrijednosti su po registrima rasporedjene tako da
	## je sve spremno za narednu iteraciju.
        mul     esi
        pop     edx
        sub     eax, edx
        xchg    esi, eax
        loop again

        ## Upisuju se brojilac i imenilac razlomka koji predstavlja
        ## trazeni element sekvence na zadate lokacije i vraca se 1 kao
	## rezultat funkcije.
        mov     ebx, [ebp+16]
        mov     [ebx], esi
        mov     ebx, [ebp+20]
        mov     [ebx], edi
        mov     eax, 1
        jmp done

not_found: 
        ## Vraca se 0 kao rezultat funkcije.
        xor     eax, eax

done:   
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
