.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.
one:    .float  1               # Broj 1.
three:  .float  3               # Broj 3.
        
.text
.global bezier

	## Funkcija bezier() odredjuje koordinate tacke na datoj
        ## Bezier-ovoj krivoj za datu vrijednost parametra t.  Argumenti
	## funkcije su:
        ##   [ebp+8], [ebp+12], [ebp+16], [ebp+20] - adrese vektora
        ##                                           koordinata
        ##                                           kontrolnih tacaka
        ##                                           kojima je odredjena
        ##                                           Bezier-ova kriva
        ##   [ebp+24] - vrijednost parametra t
        ##   [ebp+28] - adresa vektora u koji treba smjestiti koordinate
	##              trazene tacke
        ## Pretpostavlja se da je vrijednost parametra t unutar
        ## intervala [0,1]. Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
bezier:
        ## Prolog funkcije.
	enter   16, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## U polje najmanje tezine registara xmm4, xmm5, xmm1 i xmm2
        ## respektivno se upisuju vrijednosti 1-t, t, (1-t)^2 i t^2.
        movss   xmm4, one
        movss   xmm5, [ebp+24]
        subss   xmm4, xmm5
        movss   xmm1, xmm4
        mulss   xmm1, xmm4
        movss   xmm2, xmm5
        mulss   xmm2, xmm5

        ## U poljima najmanje tezine registara xmm0, xmm1, xmm2 i xmm3
        ## respektivno se izracunavaju vrijednosti (1-t)^3, 3*t*(1-t)^2,
	## 3*t^2*(1-t) i t^3.
        movss   xmm0, xmm4
        mulss   xmm0, xmm1
        movss   xmm3, xmm5
        mulss   xmm3, xmm2
        mulss   xmm1, xmm5
        mulss   xmm1, three
        mulss   xmm2, xmm4
        mulss   xmm2, three
        
        ## Sva polja registara xmm0, xmm1, xmm2 i xmm3 se postavljaju na
        ## vrijednosti polja najmanje tezine u svakom od tih registara.
        shufps  xmm0, xmm0, 0x00
        shufps  xmm1, xmm1, 0x00
        shufps  xmm2, xmm2, 0x00
        shufps  xmm3, xmm3, 0x00

        ## U poslednji element pomocnog niza od 4 realna broja, koji je
        ## alociran kao lokalna promjenljiva, se upisuje 0.
        movss   xmm7, zero
        movss   [ebp-4], xmm7

        ## Postavlja se smjer za instrukcije za rad sa stringovima.
        cld

        ## Preko pomocnog niza, u registre xmm4, xmm5, xmm6 i xmm7 se
	## upisuju koordinate tacaka P0, P1, P2 i P3.
        mov     ecx, 3
        mov     esi, [ebp+8]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm4, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+12]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm5, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+16]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm6, [ebp-16]
        mov     ecx, 3
        mov     esi, [ebp+20]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm7, [ebp-16]

        ## U registrima xmm0, xmm1, xmm2 i xmm3 respektivno se formiraju
        ## faktori (1-t)^3*P0, 3*t*(1-t)^2*P1, 3*t^2*(1-t)*P2 i t^3*P3.
        mulps   xmm0, xmm4
        mulps   xmm1, xmm5
        mulps   xmm2, xmm6
        mulps   xmm3, xmm7

        ## Sabiranjem gornjih faktora, dobijaju se koordinate trazene
	## tacke P.
        addps   xmm0, xmm1
        addps   xmm0, xmm2
        addps   xmm0, xmm3

        ## Preko pomocnog niza, koordinate tacke P se upisuju u date
	## memorijske lokacije.
        movups  [ebp-16], xmm0
        mov     ecx, 3
        lea     esi, [ebp-16]
        mov     edi, [ebp+28]
        rep movsd
        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
	## registre.
        fxrstor [esp]
        mov     esp, edx

done:        
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
