.intel_syntax noprefix

.data
table:
        ## Tabela koja na indeksima koji odgovaraju velikim slovima
        ## sadrzi Soundex kodove, odn. vrijednost -1 u svim ostalim
	## elementima.
        .fill 65, 1, -1
        .byte 0, 1, 2, 3, 0, 1, 2, 0, 0, 2, 2, 4, 5
        .byte 5, 0, 1, 2, 6, 2, 3, 0, 1, 0, 2, 0, 2
        .fill 165, 1, -1
        
.text
.global soundex

       	## Funkcija soundex() odredjuje Soundex kod za datu
	## rijec.  Argumenti funkcije su:
        ##   [ebp+8] - adresa na kojoj je smjesten prvi karakter rijeci
        ##   [ebp+12] - adresa na kojoj treba smjestiti Soundex kod
	##              rijeci
        ## Pretpostavka je da se rijec sastoji samo od velikih slova,
        ## kao i da je na adresi na kojoj treba staviti Soundex kod
        ## rijeci alocirano 4 karaktera.  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
soundex:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Registar ebx ce sadrzati adresu prvog elementa gornje tabele.
        ## Registar esi ce sadrzati adresu tekuceg karaktera date
        ## rijeci, a registar edi adresu na koju treba upisati naredni
        ## karakter Soundex koda.  Registar ecx ce sadrzati broj
	## karaktera Soundex koda preostalih za izracunavanje.
        cld
        lea     ebx, table
        mov     esi, [ebp+8]
        mov     edi, [ebp+12]
        mov     ecx, 4

        ## Kopira se prvi karakter rijeci u odredisni niz, i pamti se
	## njegov Soundex kod.
        lodsb
        stosb
        dec     ecx
        xlatb
        
next_character:
        ## Pamti se Soundex kod prethodnog karaktera rijeci u registru
	## ah, a u registar al se ucitava tekuci karakter rijeci.
	## Petlja se prekida ako su procesirani svi karakteri rijeci.
        mov     ah, al
        lodsb
        cmp     al, 0
        je finalize

        ## Odredjuje se Soundex kod tekuceg karaktera rijeci, i ako je u
        ## pitanju kod koji se ignorise, ili ako je u pitanju isti kod
        ## kao za prethodni karakter rijeci, odmah se prelazi na naredni
	## karakter rijeci.
        xlatb
        cmp     al, ah
        je next_character
        cmp     al, 0
        je next_character

        ## Upisuje se cifra koja odgovara Soundex kodu tekuceg karaktera
        ## rijeci u niz karaktera predvidjena za smjestanje Soundex koda
        ## citave rijeci; potom se provjerava da li je Soundex kod
	## rijeci kompletiran i, ako jeste, prekida se petlja.
        mov     dl, al
        add     al, '0'
        stosb
        mov     al, dl
        dec     ecx
        jz done
        jmp next_character

finalize:
        ## Po potrebi se dopunjava Soundex kod citave rijeci, tako da
	## uvijek sadrzi 4 karaktera.
        mov     al, '0'
        rep stosb

done:        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
