.intel_syntax noprefix

.text
.global arbitrary_vector

	## Funkcija arbitrary_vector() izracunava najvecu po modulu
	## sopstvenu vrijednost date matrice metodom proizvoljnog
	## vektora. Argumenti funkcije su:
	##   [ebp+8] - dimenzija matrice
	##   [ebp+12] - polje sa elementima matrice (po redovima)
	##   [ebp+16] - broj iteracija
        ## Pretpostavlja se da su dimenzija matrice i broj iteracija
	## veci od 0. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
arbitrary_vector:
        ## Prolog funkcije.
        enter	0, 0
	push	esi
	push	edi

	## Alocira se prostor na steku za drzanje tekuceg i narednog
	## vektora. Adresa tekuceg vektora se pamti u registru esi, a
	## adresa narednog vektora u registru edi. U komentarima ce ovi
	## vektori biti oznacavani sa curr odn. next.
	mov	ecx, [ebp+8]
	shl	ecx, 2
	sub	esp, ecx
	mov	esi, esp
	sub	esp, ecx
	mov	edi, esp

	## Inicijalizuje se vektor curr na vektor jedinica. Registar
	## esi sadrzi adresu tekuceg elementa, a registar ecx se koristi
	## kao brojac.
	push	esi
	mov	ecx, [ebp+8]
	fld1			# Stek: 1
next_one:
	fst	dword ptr [esi]
	add	esi, 4
	loop next_one
	ffree	st		# Stek: invalidna vrijednost
	fincstp			# Stek:
	pop	esi

	## Registar ecx se koristi kao brojac za iteracije.
	mov	ecx, [ebp+16]

next_iteration:
	## Mnozi se vektor curr matricom da bi se dobio vektor next.
	## Na steku se cuvaju vrijednosti brojaca, kao i adrese
	## vektora curr i next.
	push ecx
	push esi
	push edi

	## U registar eax se ucitava adresa matrice, a registar ecx ce
	## biti koriscen kao brojac kroz redove matrice.
	mov	ecx, [ebp+8]
	mov	eax, [ebp+12]

next_row:
	## Pamte se brojac i adresa vektora curr.
	push	ecx
	push	esi

	## Registar ecx ce biti koriscen kao brojac kroz kolone
	## matrice. Pomocu steka ce biti racunata suma A[i][j]*curr[j]
	## za j=0,...,n-1 koja predstavlja element next[i]. Parcijalna
	## suma koja ukljucuje elemente iz intervala [0,j] ce biti
	## oznacavana sa S(j).
	mov	ecx, [ebp+8]	
	fldz			# Stek:	0=S(-1)

next_column:
	## Mnozi se svaki element tekuceg reda matrice sa odgovarajucim
	## elementom vektora curr i na taj nacin se izracunava tekuci
	## element vektora next.
	fld	dword ptr [eax]	# Stek: S(j-1), A[i][j]
	add	eax, 4
	fld	dword ptr [esi]	# Stek:	S(j-1), A[i][j], curr[j]
	add	esi, 4
	fmulp			# Stek:	S(j-1), A[i][j]*curr[j]
	faddp			# Stek:	S(j-1)+A[i][j]*curr[j]=S(j)
	loop next_column

	## Pamti se upravo izracunata vrijednost tekuceg elementa
	## vektora next.
	fstp	dword ptr [edi]	# Stek:
	add	edi, 4

	## Prelazi se na izracunavanje narednog elementa vektora next.
	pop	esi
	pop	ecx
	loop next_row

	## Vektor next postaje vektor curr za narednu iteraciju i
	## obrnuto.
	pop	edi
	pop	esi
	xchg	esi, edi
	pop	ecx
	loop next_iteration

	## Racuna se kolicnik prvih elemenata vektora curr i next (koji
	## u ovom trenutku predstavlja vektor iz prethodne iteracije)
	## koji predstavlja ocjenu najvece po modulu sopstvene
	## vrijednosti matrice.
	fld	dword ptr [esi]	# Stek:	 curr[0]
	fld	dword ptr [edi]	# Stek:	 curr[0], next[0]
	fdivp			# Stek:	 curr[0]/next[0]=lambda

	## Dealocira se prostor na steku koga su zauzimali tekuci i
	## naredni vektor.
	mov	ecx, [ebp+8]
	shl	ecx, 2
	add	esp, ecx
	add	esp, ecx

        ## Epilog funkcije.
	pop	edi
	pop	esi
        leave
        ret
