.intel_syntax noprefix

.text
.global monte_carlo

	## Funkcija monte_carlo() racuna odredjeni integral funkcije x^2
	## na intervalu od 0 do 1. Argumenti funkcije su:
	##   [ebp+8] - broj slucajnih tacaka
        ##   [ebp+12] - maksimalni slucajni broj koji se moze generisati
        ##              biblioteckom funkcijom rand()
        ## Funkcija vraca vrijednost trazenog integrala.  Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
monte_carlo:
        ## Prolog funkcije.
        enter	0, 0
        push    ebx

        ## Trazeni integral ce biti izracunat tako sto ce se u trouglu 
        ## omedjenom pravima x=1, y=0 i x=y, koji potpuno obuhvata
	## povrsinu jednaku trazenom integralu, generisati dati broj
        ## slucajnih tacaka i prebrojati koliko njih se nalazi unutar
	## ove povrsi.  Kolicnik ovog broja i ukupnog broja slucajnih
        ## tacaka, pomnozen sa povrsinom trougla (koja iznosi 0.5), daje
	## trazenu povrsinu, odnosno trazeni integral.

        ## Registar ecx ce se koristiti kao brojac i isti se 
	## inicijalizuje na dati broj koraka.  Registar ebx ce brojati
	## koliko se generisanih slucajnih tacaka nalazi unutar trazene
        ## povrsine i inicijalizuje se na 0.  Na registarski stek
        ## koprocesora se stavlja maksimalni slucajni broj (stavljanje
	## cijelih brojeva na registarski stek ce uvijek ici preko
	## obicnog steka).
        mov     ecx, [ebp+8]
        xor     ebx, ebx
        mov     eax, [ebp+12]
        push    eax
        fild    dword ptr [esp]         # Stek: RAND_MAX
        add     esp, 4

again:
        ## Generise se y koordinata slucajne tacke u intervalu [0,1].
        push    ecx
        call rand
        pop     ecx
        push    eax
        fild    dword ptr [esp]         # Stek: RAND_MAX, rand()
        add     esp, 4
        fdiv    st, st(1)               # Stek: RAND_MAX, rand()/RAND_MAX=y

        ## Generise se x koordinata slucajne tacke u intervalu [0,1].
        push    ecx
        call rand
        pop     ecx
        push    eax
        fild    dword ptr [esp]         # Stek: RAND_MAX, y, rand()
        add     esp, 4
        fdiv    st, st(2)               # Stek: RAND_MAX, y, rand()/RAND_MAX=x

        ## Provjerava se da li je x>=y odnosno da li je tacka unutar
        ## pomenutog trougla.  Ako to nije slucaj, ponovo se generisu
	## koordinate tacke.
        fcomi   st, st(1)               # Stek: RAND_MAX, y, x
        jae valid
        ffree   st                      # Stek: RAND_MAX, y, invalidna vrijednost
        fincstp                         # Stek: RAND_MAX, y
        ffree   st                      # Stek: RAND_MAX, invalidna vrijednost
        fincstp                         # Stek: RAND_MAX
        jmp again

valid:
        ## Ispituje se da li se tacka nalazi unutar trazene povrsine i
	## ako je to slucaj inkrementira se sadrzaj registra ebx.
        fmul    st, st                  # Stek: RAND_MAX, y, x*x
        fcomip  st, st(1)               # Stek: RAND_MAX, y
        jb continue
        inc     ebx

continue:
        ## Skida se y koordinata sa registarskog steka i prelazi se na
	## narednu iteraciju.
        ffree   st                      # Stek: RAND_MAX, invalidna vrijednost
        fincstp                         # Stek: RAND_MAX
        loop again

        ## Skida se maksimalni slucajni broj sa registarskog steka.
        ffree   st                      # Stek: invalidna vrijednost
        fincstp                         # Stek: 

        ## Izracunava se rezultat.  Broj slucajnih tacaka koje se nalaze
        ## unutar trazene povrsine je oznacen sa m, a ukupan broj
	## slucajnih tacaka sa n.
        push    ebx
        fild    dword ptr [esp]         # Stek:  m
        add     esp, 4
        mov     ecx, [ebp+8]
        shl     ecx, 1
        push    ecx
        fidiv   dword ptr [esp]         # Stek: m/(2*n)=0.5*m/n
        add     esp, 4
        
        ## Epilog funkcije.
        pop     ebx
        leave
        ret
