.intel_syntax noprefix

.text
.global prover
        
        ## Funkcija prover() vrsi provjeru da li je data logicka
        ## formula, izrazena u postfiksnoj notaciji, tautologija.
        ## Argumenti funkcije su:
        ##   [ebp+8] - string koji predstavlja datu formulu
        ## Funkcija vraca 1 ako formula jeste tautologija, odn. 0 u
        ## suprotnom; ako je teorema sintaksno neispravna, funkcija
        ## vraca -1.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
prover:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi        

        ## Inicijalizuje se direction fleg za citanje formule.
        cld

        ## Registar ebx ce sadrzati tekucu vrijednost promjenljive x;
	## ovaj registar u petlji uzima vrijednosti 0 i 1.
        xor     ebx, ebx
next_x:

        ## Registar edx ce sadrzati tekucu vrijednost promjenljive y;
        ## ovaj registar u petlji uzima vrijednosti 0 i 1.
        xor     edx, edx
next_y:

        ## Registar edi ce sadrzati tekucu vrijednost promjenljive z;
        ## ovaj registar u petlji uzima vrijednosti 0 i 1.
        xor     edi, edi
next_z:

        ## Registar ecx ce sadrzati broj operanada na steku tokom
        ## izracunavanja formule, a registar esi ce sadrzati memorijsku
	## adresu na kojoj se nalazi sledeci simbol u formuli.
        xor     ecx, ecx
        mov     esi, [ebp+8]
        
next_symbol:
        ## U registar al ucitava se naredni simbol formule.
        lodsb

        ## Provjerava se da li se doslo do kraja formule.
        cmp     al, 0
        je check_result

        ## Preskacu se blanko karakteri.
        cmp     al, ' '
        je next_symbol

        ## U zavisnosti od toga sta je naredni simbol u formuli, obavlja
	## se odgovarajuca akcija.
        cmp     al, 'x'
        je found_x
        cmp     al, 'y'
        je found_y
        cmp     al, 'z'
        je found_z
        cmp     al, '~'
        je found_not
        cmp     al, '&'
        je found_and
        cmp     al, '|'
        je found_or
        jmp error

found_x:
        ## Stavlja se vrijednost promjenljive x na stek.
        push    ebx
        inc     ecx
        jmp next_symbol

found_y:
        ## Stavlja se vrijednost promjenljive y na stek.
        push    edx
        inc     ecx
        jmp next_symbol

found_z:
        ## Stavlja se vrijednost promjenljive z na stek.
        push    edi
        inc     ecx
        jmp next_symbol

found_not:
        ## Provjerava se da li na steku ima jedan operand i invertuje se
	## isti.
        cmp     ecx, 1
        jl error
        not     dword ptr [esp]
        jmp next_symbol

found_and:
        ## Provjerava se da li na steku imaju dva operanda; potom se
        ## skidaju operandi sa steka, obavlja se operacija logicko "i"
	## nad istima i vraca se rezultat na stek.
        cmp     ecx, 2
        jl error
        pop     eax
        and     [esp], eax
        dec     ecx
        jmp next_symbol

found_or:
        ## Provjerava se da li na steku imaju dva operanda; potom se
        ## skidaju operandi sa steka, obavlja se operacija logicko "ili"
	## nad istima i vraca se rezultat na stek.
        cmp     ecx, 2
        jl error
        pop     eax
        or     [esp], eax
        dec     ecx
        jmp next_symbol

check_result:
        ## Provjerava se da li nakon zavrsenog izracunavanja ostala
	## tacno jedna vrijednost na steku.
        cmp     ecx, 1
        jne error

        ## Skida se rezultat sa steka i, ako je isti jednak 0, odmah se
        ## prekida sa iteracijama.  Logicke operacije su u prethodnom
        ## kodu izvodjene nad 32-bitnim vrijednostima, mada je samo
        ## vrijednost bita najmanje tezine bila znacajna.  Zato se prije
	## provjere rezultata maskiraju svi ostali bitovi.
        pop     eax
        and     eax, 0x1
        cmp     eax, 0
        je done

        ## Prelazi se na narednu vrijednost promjenljive z.
        inc     edi
        cmp     edi, 2
        jl next_z

        ## Prelazi se na narednu vrijednost promjenljive y.
        inc     edx
        cmp     edx, 2
        jl next_y

        ## Prelazi se na narednu vrijednost promjenljive x.
        inc     ebx
        cmp     ebx, 2
        jl next_x

        ## Ako se proslo kroz sve moguce vrijednosti svih promjenljivih
        ## i ako je vrijednost formule bila uvijek jednaka 1, znaci da
        ## je formula tautologija.  Registar eax, ciji je sadrzaj
        ## povratna vrijednost iz funkcije, u ovoj tacki vec ima
        ## odgovarajucu vrijednost - 1.
        jmp done

error:
        ## U registar eax, ciji je sadrzaj povratna vrijednost iz
        ## funkcije, upisuje se -1 kao indikacija greske.  Skidaju se sa
	## steka zaostale vrijednosti.
        mov     eax, -1
        shl     ecx, 2
        add     esp, ecx        

done:                
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
