.intel_syntax noprefix

.text
.global ackerman

       	## Funkcija ackerman() odredjuje cio broj iz datog intervala
        ## koji ima najduzu Ackerman-ovu sekvencu, kao i duzinu te
	## sekvence.  Argumenti funkcije su:
        ##   [ebp+8], [ebp+12] - granice datog intervala
        ##   [ebp+16] - adresa na kojoj treba smjestiti broj iz datog
        ##              intervala sa najduzom Ackerman-ovom sekvencom
        ##   [ebp+20] - adresa na kojoj treba smjestiti duzinu sekvence
	##              za gornji broj
        ## Pretpostavka je da je interval ne-prazan, kao i da je donja
        ## granica intervala veca od 1; takodje i da ce prilikom
        ## racunanja sekvenci za svaki broj iz datog intervala, svaka
        ## vrijednost u tim sekvencama moci biti predstavljena kao
	## 32-bitni cio broj.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
ackerman:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Registar ebx ce sadrzati tekuci broj iz datog intervala.
        ## Registar esi ce sadrzati broj sa trenutno najduzom sekvencom,
	## a registar edi duzinu te sekvence.
        mov     ebx, [ebp+8]
        xor     edi, edi
        
number_next:
        ## Tokom odredjivanja duzine sekvence za tekuci broj iz datog
	## intervala, registar eax ce sadrzati vrijednost x[i], a
	## registar ecx duzinu sekvence.
        mov     eax, ebx
        xor     ecx, ecx

x_next:
        ## Provjerava se da li je sekvenca zavrsena.
        cmp     eax, 1
        je x_done

        ## Odredjuje se naredna vrijednost u sekvenci.
        test    eax, 1
        jz even
        mov     edx, 3
        mul     edx
        inc     eax
        jmp x_continue
even:
        shr     eax, 1

x_continue:
        ## Nastavlja se sa izracunavanjem sekvence.
        inc     ecx        
        jmp x_next

x_done:
        ## Provjerava se da li je sekvenca za tekuci broj duza od
        ## trenutno najduze sekvence i, ako je to slucaj, pamte se
	## tekuci broj i odgovarajuca duzina sekvence.
        cmp     ecx, edi
        jbe number_continue
        mov     esi, ebx     
        mov     edi, ecx

number_continue:
        ## Provjerava se da li se proslo kroz citav interval i, ako to
	## nije slucaj, nastavlja se sa narednim brojem iz intervala.
        inc     ebx
        cmp     ebx, [ebp+12]
        jbe number_next

finish:
        ## Pamte se broj sa najduzom sekvencom iz datog intervala, kao i
        ## sama duzina sekvence, u za to predvidjenim lokacijama.
        mov     ebx, [ebp+16]
        mov     [ebx], esi
        mov     ebx, [ebp+20]
        mov     [ebx], edi

        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
