.intel_syntax noprefix

.data
	## Karakteri koji predstavljaju jedinice, petice i desetice u
	## rimskom zapisu broja respektivno za opsege [100,1000],
	## [10,100] i [1,10].
range:	.byte	'C','D','M'
	.byte	'X','L','C'
	.byte	'I','V','X'

.text
.global dec2rom

	## Funkcija convert_digit() konvertuje jednu decimalnu cifru u
	## rimski zapis. Arumenti funkcije su:
	##   [ebp+8] - cifra koju treba konvertovati
	##   [ebp+12] - string u koji treba smjestiti konvertovanu 
	##              reprezentaciju broja
	##  [ebp+16] - adresa prvog elementa polja od tri karaktera
	##             koji predstavljaju jedinice, petice i desetice 
	##             u rimskom zapisu	za dati opseg
	## Pretpostavka je da je data cifra u opsegu [0,9]. Funkcija
	## vraca broj karaktera u rimskom zapisu cifre. Funkcija se u 
	## potpunosti podvrgava C konvencijama pozivanja.
convert_digit:
	## Prolog funkcije.
        enter	0, 0
	push	ebx
	push	edi

        ## Inicijalizuje se direction fleg.
        cld

	## U registru ecx ce se cuvati cifra, registar edi ce sadrzati
	## adresu na koju treba upisati naredni karakter u rimskom
	## zapisu cifre. Registar ebx ce drzati adresu polja karaktera
	## koji opisuju rimski zapis cifre u datom opsegu. Registar edx
	## ce brojati sa koliko karaktera je predstavljena cifra.
	mov	ecx, [ebp+8]
	mov	edi, [ebp+12]
	mov	ebx, [ebp+16]
	xor	edx, edx

	## Ako je cifra jednaka 0, odmah se skace na kraj funkcije.
	cmp	ecx, 0
	je done

	## Poredi se cifra sa 9.
	cmp	ecx, 9
	jb less_than_9

	## Ukoliko je cifra jednaka 9, upisuje se odgovarajuca
	## reprezentacija u string.
	mov	al, [ebx]
	stosb
	mov	al, [ebx+2]
	stosb
	add	edx, 2
	jmp done
	
less_than_9:
	## Poredi se cifra sa 5
	cmp	ecx, 5
	jb less_than_5

	## Ukoliko je cifra veca ili jednaka 5, upisuje se
	## petica u string i skace se na upis ostatka cifre.
	mov	al, [ebx+1]
	stosb
	inc	edx
	sub	ecx, 5
	jmp less_than_4

less_than_5:
	## Poredi se cifra sa 4.
	cmp	ecx, 4
	jb less_than_4

	## Ukoliko je cifra jednaka 4, upisuje se odgovarajuca 
	## reprezentacija u string.
	mov	al, [ebx]
	stosb
	mov	al, [ebx+1]
	stosb
	add	edx, 2
	jmp done

less_than_4:
	## Upisuje se odgovarajuci broj jedinica u string.
	add	edx, ecx
	mov	al, [ebx]
	rep stosb
	
done:
	## Epilog funkcije.
	mov	eax, edx
	pop	edi
	pop	ebx
        leave
        ret
	
	## Funkcija dec2rom() prevodi dati broj iz decimalnog u rimski
	## zapis. Argumenti funkcije su:
	##   [ebp+8] - broj u decimalnom zapisu
	##   [ebp+12] - string koji predstavlja broj u rimskom zapisu
	## Pretpostavka je da je dati broj u opsegu [1,999]. Funkcija se
	## u potpunosti podvrgava C konvencijama pozivanja.
dec2rom:
        ## Prolog funkcije.
        enter	0, 0
	push	ebx
	push	esi
	push	edi

	## Izdvajaju se jedinice, desetice i stotice iz datog broja kao
	## ostaci pri uzastoponom dijeljenju sa 10 i stavljaju se na
	## stek. U registru eax se drzi broj (odn. rezultat dijeljenja
	## nakon uzastopnih dijeljenja sa 10), registar ebx cuva
	## djelilac 10, a registar ecx se koristi kao brojac.
	mov	eax, [ebp+8]
	mov	ebx, 10
	mov	ecx, 3
extract_next_digit:
	xor	edx, edx
	div	ebx
	push	edx
	loop extract_next_digit

	## Pozivanjem funkcije convert_next_digit() konvertuju se redom
	## stotice, desetice i jedinice. Registar ecx se koristi kao
	## brojac, registar esi cuva adresu prvog u nizu od tri
	## karaktera koji predstavljaju rimski zapis za jedinice, petice
	## i desetice za dati opseg, registar edi sadrzi adresu narednog
	## karaktera u rimskom zapisu broja.
	mov	ecx, 3
	mov	edi, [ebp+12]
	lea	esi, range
convert_next_digit:
	pop	eax
	push	ecx
	push	esi
	push	edi
	push	eax
	call convert_digit
	add	esp, 12
	pop	ecx
	add	esi, 3
	add	edi, eax
	loop convert_next_digit
	
	## String koji predstavlja rimski zapis broja se terminira
	## nulom.
	xor	eax, eax
	stosb
	
	## Epilog funkcije.
	pop	edi
	pop	esi
	pop	ebx
        leave
        ret
