.intel_syntax noprefix
		
.text
.global iteration

	## Funkcija iteration() racuna tabelu inverznih tangensa od 1/2^i
	## (za i=0,...,n-1). Argumenti funkcije su:
	##   [ebp+8] - duzina tabele
	##   [ebp+12] - pokazivac na pocetak tabele
	## Pretpostavlja se da je duzina tabele veca od 0. Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
iteration:
        ## Prolog funkcije.
        enter	0, 0
	push	ebx

	## Inicijalizuje se iterativni postupak. Registar ecx ce biti
	## koriscen kao brojac iteracija.
	fld	dword ptr [ebp+12]	# Stek:	epsilon
	fld	dword ptr [ebp+8]	# Stek:	epsilon, x(0)
	mov	ecx, [ebp+16]

next_iteration:
	
	## Izracunava se x(i+1)=exp(-x(i)). U komentarima je sa int()
	## oznacen cjelobrojni, a sa frac() razlomljeni dio realnog
	## broja.
	fld	st		# Stek:	epsilon, x(i), x(i)
	fchs			# Stek:	epsilon, x(i), -x(i)
	fldl2e			# Stek:	epsilon, x(i), -x(i), log2(e)
	fmulp			# Stek:	epsilon, x(i), -x(i)*log2(e)=X
	fld	st		# Stek:	epsilon, x(i), X, X
	frndint			# Stek:	epsilon, x(i), X, int(X)
	fsub	st(1), st	# Stek:	epsilon, x(i), X-int(X)=frac(X), int(X)
	fxch			# Stek:	epsilon, x(i), int(X), frac(X)
	f2xm1			# Stek:	epsilon, x(i), int(X), 2^frac(X)-1
	fld1			# Stek:	epsilon, x(i), int(X), 2^frac(X)-1, 1
	faddp			# Stek:	epsilon, x(i), int(X), 2^frac(X)
	fscale			# Stek:	epsilon, x(i), int(X), 2^int(X)*2^frac(X)=2^(int(X)+frac(X))=2^X
	fxch			# Stek:	epsilon, x(i), 2^X, int(X)
	ffree	st		# Stek:	epsilon, x(i), 2^X, invalidna vrijednost
	fincstp			# Stek:	epsilon, x(i), 2^X=2^(-x(i)*log2(e))=exp(-x(i))=x(i+1)

	## Odredjuje se |x(i+1)-x(i)| i poredi sa trazenom tacnoscu.
	fxch			# Stek:	epsilon, x(i+1), x(i)
	fsub	st, st(1)	# Stek:	epsilon, x(i+1), x(i)-(x+1)
	fabs			# Stek:	epsilon, x(i+1), |x(i)-(x+1)|=|x(i+1)-x(i)|
	fcomip	st, st(2)	# Stek:	epsilon, x(i+1)

	## Ukoliko je postignuta trazena tacnost izlazi se iz petlje, u
	## suprotnom se nastavlja sa itracijama.
	jb found
	loop next_iteration

	## Ako metoda nije konvergirala vraca se 0 kao rezultat.
	xor	eax, eax
	jmp finish

found:
	## Ako je metoda konvergirala, pamti se vrijednost rjesenja,
	## cisti se stek i vraca se 1 kao rezultat.
	mov	ebx, [ebp+20]
	fstp	dword ptr [ebx]	# Stek:	epsilon
	ffree	st		# Stek:	invalidna vrijednost
	fincstp
	mov	eax, 1
	
finish:
	## Epilog funkcije.
	pop	ebx
        leave
        ret
