.intel_syntax noprefix

.data
scale:  .int    -4              # Broj -4.

.text
.global dct

	## Funkcija dct() racuna diskretnu kosinusnu transformaciju date
	## matrice.  Argumenti funkcije su:
	##   [ebp+8] - data matrica
	##   [ebp+12] - transformacija date matrice
        ## Funkcija ne vraca nikakvu vrijednost.  Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
dct:
        ## Prolog funkcije.  Alocira se memorija za lokalne promjenljive
	## x, y, u i v.
        enter	16, 0
        push    esi
        push    edi

        ## Registar edi ce cuvati adresu prvog elementa, a registar edx
	## index tekuceg elementa rezultujuce matrice.
        mov     edi, [ebp+12]
        xor     edx, edx

        ## Na stek se stavlja vrijednost 1/sqrt(2).
        fld1                            # Stek: 1
        fld1                            # Stek: 1, 1
        fscale                          # Stek: 1, 2
        fsqrt                           # Stek: 1, sqrt(2)
        fdivp                           # Stek: 1/sqrt(2)

        ## Petlja po u.
        mov     dword ptr [ebp-12], 0
next_u:

        ## Petlja po v.
        mov     dword ptr [ebp-16], 0
next_v: 
        ## Na stek se stavlja 0 kao inicijalna vrijednost za sumu po x
	## iz izraza za DCT (ova suma ce u komentarima biti oznacavana sa
	## Sx).  Potom se na stek stavlja i broj -4.
        fldz                            # Stek: 1/sqrt(2), 0=Sx
        fild    dword ptr scale         # Stek: 1/sqrt(2), Sx, -4

        ## Registar esi ce cuvati adresu prvog elementa, a registar ecx
	## index tekuceg elementa date matrice.
        mov     esi, [ebp+8]
        xor     ecx, ecx

        ## Izraz za DCT se racuna za svaki DCT koeficijent u dvostrukoj
	## petlji.  Izraz je prethodno potrebno tranfsormisati tako da
	## se sto je moguce vise izracunavanja izvuce izvan unutrasnje,
	## odnosno i spoljasnje petlje.

        ## Petlja po x.
        mov     dword ptr [ebp-4], 0
next_x:
        ## Na stek se stavlja 0 kao inicijalna vrijednost za sumu po y
	## iz izraza za DCT (ova suma ce u komentarima biti oznacavana sa
	## Sy).  Broj -4 se potom opet dovodi na vrh steka.
        fldz                            # Stek: 1/sqrt(2), Sx, -4, 0=Sy
        fxch                            # Stek: 1/sqrt(2), Sx, Sy, -4

        ## Petlja po y.
        mov     dword ptr [ebp-8], 0
next_y:
        ## Izracunava se izraz u sumi po y za tekuce y i dodaje u tu sumu.
        fld1                            # Stek: 1/sqrt(2), Sx, Sy, -4, 1
        fild    dword ptr [ebp-8]       # Stek: 1/sqrt(2), Sx, Sy, -4, 1, y
        fscale                          # Stek: 1/sqrt(2), Sx, Sy, -4, 1, 2*y
        faddp                           # Stek: 1/sqrt(2), Sx, Sy, -4, 2*y+1
        fimul   dword ptr [ebp-16]      # Stek: 1/sqrt(2), Sx, Sy, -4, (2*y+1)*v
        fldpi                           # Stek: 1/sqrt(2), Sx, Sy, -4, (2*y+1)*v, pi
        fmulp                           # Stek: 1/sqrt(2), Sx, Sy, -4, (2*y+1)*v*pi
        fscale                          # Stek: 1/sqrt(2), Sx, Sy, -4, (2*y+1)*v*pi/16
        fcos                            # Stek: 1/sqrt(2), Sx, Sy, -4, cos((2*y+1)*v*pi/16)
        fmul    dword ptr [esi+4*ecx]   # Stek: 1/sqrt(2), Sx, Sy, -4, f(x,y)*cos((2*y+1)*v*pi/16)
        faddp   st(2), st               # Stek: 1/sqrt(2), Sx, Sy+f(x,y)*cos((2*y+1)*v*pi/16)=Sy, -4

        ## Provjerava se da li se doslo do kraja petlje po y.
        inc     ecx
        inc     dword ptr [ebp-8]
        cmp     [ebp-8], 8
        jb next_y

        ## Izracunava se izraz u sumi po x za tekuce x i dodaje u tu sumu.
        fld1                            # Stek: 1/sqrt(2), Sx, Sy, -4, 1
        fild    dword ptr [ebp-4]       # Stek: 1/sqrt(2), Sx, Sy, -4, 1, x
        fscale                          # Stek: 1/sqrt(2), Sx, Sy, -4, 1, 2*x
        faddp                           # Stek: 1/sqrt(2), Sx, Sy, -4, 2*x+1
        fimul   dword ptr [ebp-12]      # Stek: 1/sqrt(2), Sx, Sy, -4, (2*x+1)*u
        fldpi                           # Stek: 1/sqrt(2), Sx, Sy, -4, (2*x+1)*u, pi
        fmulp                           # Stek: 1/sqrt(2), Sx, Sy, -4, (2*x+1)*u*pi
        fscale                          # Stek: 1/sqrt(2), Sx, Sy, -4, (2*x+1)*u*pi/16
        fcos                            # Stek: 1/sqrt(2), Sx, Sy, -4, cos((2*x+1)*u*pi/16)
        fmulp   st(2), st               # Stek: 1/sqrt(2), Sx, cos((2*x+1)*u*pi/16)*Sy, -4

        fxch                            # Stek: 1/sqrt(2), Sx, -4, cos((2*x+1)*u*pi/16)*Sy
        faddp   st(2), st               # Stek: 1/sqrt(2), Sx*cos((2*x+1)*u*pi/16)*Sy=Sx, -4

        ## Provjerava se da li se doslo do kraja petlje po x.
        inc     dword ptr [ebp-4]
        cmp     [ebp-4], 8
        jb next_x

        ## Dovrsava se racunanje koeficijenta F(u,v) i izracunati
	## koeficijent se smjesta u matricu F.
        fchs                            # Stek: 1/sqrt(2), Sx, 4
        fdivp                           # Stek: 1/sqrt(2), Sx/4
        cmp     [ebp-12], 0
        ja check_v
        fmul    st, st(1)               # Stek: 1/sqrt(2), Sx/4*Cu
check_v:
        cmp     [ebp-16], 0
        ja store
        fmul    st, st(1)               # Stek: 1/sqrt(2), Sx/4*Cu*Cv
store:
        fstp    dword ptr [edi+4*edx]   # Stek: 1/sqrt(2)

        ## Provjerava se da li se doslo do kraja petlje po v.        
        inc     edx
        inc     dword ptr [ebp-16]
        cmp     [ebp-16], 8
        jb next_v

        ## Provjerava se da li se doslo do kraja petlje po u.
        inc     dword ptr [ebp-12]
        cmp     [ebp-12], 8
        jb next_u

        ## Skida se vrijednost 1/sqrt(2) sa steka.
        ffree   st                      # Stek: invalidna vrijednost
        fincstp
        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        leave
        ret
