.intel_syntax noprefix

.data
two:    .int    2               # Broj 2.
        
.text
.global ln

	## Funkcija ln() racuna vrijednost prirodnog logaritma datog
        ## broja pomocu aproksimacije Taylor-ovim razvojem sa datim
	## brojem faktora.  Argumenti funkcije su:
	##   [ebp+8] - vrijednost za koju se izracunava funkcija
	##   [ebp+12] - broj faktora u aproksimaciji
        ## Pretpostavlja se da su vrijednost za koju se izracunava
	## funkcija, kao i broj faktora u aproksimaciji veci od 0.
	## Funkcija vraca izracunatu ocjenu vrijednosti logaritma.
	## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
ln:
        ## Prolog funkcije.
        enter	0, 0

        ## Registar ecx ce biti koriscen kao brojac u petlji.
        mov     ecx, [ebp+12]

        ## Pripremaju se na steku sve vrijednosti koje su potrebne za
        ## racunanje razvoja.  Razvoj ce biti izracunat kao suma
        ## elemenata oblika 1/(2*i+1)*((x-1)/(x+1))^(2*i+1), pri cemu je
	## i=0,1,...
        fldz                    # Stek: 0=S(0)
        fild    two             # Stek: S(0), 2
        fld1                    # Stek: S(0), 2, 1
        fld     [ebp+8]         # Stek: S(0), 2, 1, x
        fld     st              # Stek: S(0), 2, 1, x, x
        fsub    st, st(2)       # Stek: S(0), 2, 1, x, x-1
        fxch                    # Stek: S(0), 2, 1, x-1, x
        fadd    st, st(2)       # Stek: S(0), 2, 1, x-1, x+1
        fdivp                   # Stek: S(0), 2, 1, (x-1)/(x+1)
        fld     st              # Stek: S(0), 2, 1, (x-1)/(x+1), (x-1)/(x+1)
        fmul    st(1), st       # Stek: S(0), 2, 1, ((x-1)/(x+1))^2, (x-1)/(x+1)

next_factor:
        ## Dodaje se naredni element u razvoju u sumu.
        fld     st              # Stek: S(i), 2, 2*i+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1), ((x-1)/(x+1))^(2*i+1)
        fld1                    # Stek: S(i), 2, 2*i+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1), ((x-1)/(x+1))^(2*i+1), 1
        fdiv    st, st(4)       # Stek: S(i), 2, 2*i+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1), ((x-1)/(x+1))^(2*i+1), 1/(2*i+1)
        fmulp                   # Stek: S(i), 2, 2*i+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1), 1/(2*i+1)*((x-1)/(x+1))^(2*i+1)
        faddp   st(5), st       # Stek: S(i)+1/(2*i+1)*((x-1)/(x+1))^(2*i+1)=S(i+1), 2, 2*i+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1)

        ## Izracunava se vrijednost 2*(i+1)+1 za naredni korak razvoja.
        fxch    st(2)           # Stek: S(i+1), 2, ((x-1)/(x+1))^(2*i+1), ((x-1)/(x+1))^2, 2*i+1
        fadd    st, st(3)       # Stek: S(i+1), 2, ((x-1)/(x+1))^(2*i+1), ((x-1)/(x+1))^2, 2*i+3=2*(i+1)+1
        fxch    st(2)           # Stek: S(i+1), 2, 2*(i+1)+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+1)

        ## Izracunava se vrijednost ((x-1)/(x+1))^(2*(i+1)+1) za naredni
	## korak razvoja.
        fmul    st, st(1)       # Stek: S(i+1), 2, 2*(i+1)+1, ((x-1)/(x+1))^2, ((x-1)/(x+1))^(2*i+3)=((x-1)/(x+1))^(2*(i+1)+1)

        ## Prelazi se na naredni korak razvoja.
        loop next_factor

        ## Skidaju se sa steka nepotrebne vrijednosti.
        ffree   st              # Stek: S, 2, 2*(n+1)+1, ((x-1)/(x+1))^2, invalidna vrijednost
        fincstp                 # Stek: S, 2, 2*(n+1)+1, ((x-1)/(x+1))^2
        ffree   st              # Stek: S, 2, 2*(n+1)+1, invalidna vrijednost
        fincstp                 # Stek: S, 2, 2*(n+1)+1
        ffree   st              # Stek: S, 2, invalidna vrijednost
        fincstp                 # Stek: S, 2

        ## Izracunava se finalna vrijednost funkcije.
        fmulp                   # Stek: 2*S
        
        ## Epilog funkcije.
        leave
        ret
