.intel_syntax noprefix

.text
.global sequence
        
        ## Funkcija sequence() odredjuje indeks prvog karaktera u prvoj
        ## najduzoj sekvenci istih karaktera u datom stringu, kao i
	## duzinu te sekvence.  Argumenti funkcije su:
        ##   [ebp+8] - adresa prvog karaktera stringa
        ##   [ebp+12] - adresa na koju treba upisati indeks prvog
	##              karaktera sekvence
        ##   [ebp+16] - adresa na koju treba upisati duzinu sekvence
        ## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
sequence:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Postavlja se direction fleg i inicijalizuje se registar esi
        ## koji ce cuvati adresu tekuceg karaktera tokom procesiranja
	## stringa.
        cld
        mov     esi, [ebp+8]

        ## Registar ebx ce cuvati indeks karaktera kojim pocinje dosad
        ## najduza sekvenca u stringu, a registar ecx duzinu sekvence.
	## Inicijalizuju se vrijednosti ovih registara.
        xor     ebx, ebx
        xor     ecx, ecx

        ## Registar ah ce sadrzati prethodni karakter stringa.  Registar
        ## edx ce sadrzati duzinu trenutne sekvence istih karaktera u
        ## stringu, a registar edi indeks prvog karaktera u toj
	## sekvenci.  Inicijalizuju se vrijednosti ovih registara.
        xor     ah, ah
        xor     edi, edi
        xor     edx, edx

        ## U petlji se procesiraju karakteri datog stringa.
next_character:
        ## Ucitava se naredni karakter.
        lodsb

        ## Porede se prethodni i tekuci karakter.
        cmp     al, ah
        je equal

        ## Ako karakteri nisu jednaki, znaci da je zavrsena jedna
        ## sekvenca i da pocinje nova.  Poredi se duzina tekuce sekvence
        ## sa duzinom dosad najduze sekvence i po potrebi se azuriraju
	## duzina najduze sekvence i indeks njenog prvog karaktera.
        cmp     ecx, edx
        jge reinitialize_sequence
        mov     ebx, edi
        mov     ecx, edx

reinitialize_sequence:
        ## Inicijalizuje se indeks prvog karaktera nove sekvence i njena
	## duzina.
        add     edi, edx
        xor     edx, edx

equal:  
        ## Uvecava se duzina tekuce sekvence.
        inc     edx

        ## Razmjenjuju se tekuci i prethodni karakter, cime tekuci
        ## karakter postaje prethodni karakter za narednu iteraciju.
        xchg    al, ah

        ## Provjerava se da li se doslo do kraja stringa i ako to nije
	## slucaj nastavlja se sa iteracijama.
        cmp     ah, 0
        jne next_character

        ## Upisuju se izracunate vrijednosti u za to predvidjene
	## memorijske lokacije.
        mov     eax, [ebp+12]
        mov     [eax], ebx
        mov     eax, [ebp+16]
        mov     [eax], ecx

        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
