.intel_syntax noprefix
        
.text
.global fit

	## Funkcija fit() izracunava koeficijenta aproksimirajuce prave
        ## za data niz tacaka, niz vrijednosti izmjerenih u datim
        ## tackama i niz nepreciznosti mjerenja.  Argumenti funkcije su:
        ##   [ebp+8] - duzina nizova
        ##   [ebp+12] - niz mjernih tacaka
        ##   [ebp+16] - niz izmjerenih vrijednosti
        ##   [ebp+20] - niz nepreciznosti mjerenja
        ##   [ebp+24] - adresa na koju treba upisati koeficijent a
	##              aproksimirajuce prave
        ##   [ebp+28] - adresa na koju treba upisati koeficijent b
	##              aproksimirajuce prave
        ## Pretpostavlja se da je duzina nizova broj veci od 2, kao i da
        ## su vrijednosti nepeciznosti razlicite od 0.  Funkcija vraca 0
        ## ukoliko ne moze izracunati koeficijente aproksimirajuce
        ## prave, odn. 0 u suprotnom.  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
fit:
        ## Prolog funkcije.
	enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Razvojem datih jednacina dolazi se do sistema jednacina po a
	## i b oblika:
        ##   a * S + b * Sx = Sy
        ##   a * Sx + b * Sxx = Sxy
        ## Ovdje su S, Sx, Sy, Sxx i Sxy sume po i od 0 do n - 1 (gdje
        ## je n duzina datih nizova), pri cemu je svaki element sume
        ## kolicnik ciji je imenilac sigma[i]^2, dok mala slova u oznaci
        ## sume odredjuju sta se nalazi u brojiocu, pa tako za S u
        ## brojiocu stoji 1, za Sx stoji x[i], za Sy stoji y[i], za Sxx
        ## stoji x[i]^2 i za Sxy stoji x[i]*y[i].  Dati problem se
	## rjesava tako sto se izracunaju ove sume, a zatim rijesi gornji
	## sistem jednacina.

        ## Registar ebx ce sadrzati adresu tekuceg elementa niza x,
        ## registar esi adresu tekuceg elementa niza y, a registar edi
        ## adresu tekuceg elementa niza sigma.  Registar ecx ce biti
	## koriscen kao brojac.
        mov     ebx, [ebp+12]
        mov     esi, [ebp+16]
        mov     edi, [ebp+20]
        mov     ecx, [ebp+8]

        ## Inicijalizuju se vrijednosti suma koje predstavljaju
	## koeficijente sistema jednacina po a i b.
        fldz                    # Stek: 0=S
        fldz                    # Stek: S, 0=Sx
        fldz                    # Stek: S, Sx, 0=Sy
        fldz                    # Stek: S, Sx, Sy, 0=Sxx
        fldz                    # Stek: S, Sx, Sy, Sxx, 0=Sxy

next:
        ## U petlji se izracunavaju vrijednosti suma; indeksom [i] je
        ## oznacena vrijednost date sume u tekucoj iteraciji, a indeksom
	## [i+1] vrijednost sume u narednoj iteraciji.
        fld1                    # Stek: S[i], Sx[i], Sy[i], Sxx[i], Sxy[i], 1
        fld     dword ptr [edi] # Stek: S[i], Sx[i], Sy[i], Sxx[i], Sxy[i], 1, sigma[i]
        add     edi, 4
        fmul    st, st          # Stek: S[i], Sx[i], Sy[i], Sxx[i], Sxy[i], 1, sigma[i]^2
        fdivp                   # Stek: S[i], Sx[i], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2
        fadd    st(5), st       # Stek: S[i]+1/sigma[i]^2=S[i+1], Sx[i], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2
        fld     dword ptr [ebx] # Stek: S[i+1], Sx[i], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i]
        add     ebx, 4
        fld     st              # Stek: S[i+1], Sx[i], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i], x[i]
        fmul    st, st(2)       # Stek: S[i+1], Sx[i], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i], x[i]/sigma[i]^2
        fadd    st(6), st       # Stek: S[i+1], Sx[i]+x[i]/sigma[i]^2=Sx[i+1], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i], x[i]/sigma[i]^2
        fxch                    # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i]/sigma[i]^2, x[i]
        fmul    st, st(1)       # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i], Sxy[i], 1/sigma[i]^2, x[i]/sigma[i]^2, x[i]^2/sigma[i]^2
        faddp   st(4), st       # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i]+x[i]^2/sigma[i]^2=Sxx[i+1], Sxy[i], 1/sigma[i]^2, x[i]/sigma[i]^2
        fxch                    # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i+1], Sxy[i], x[i]/sigma[i]^2, 1/sigma[i]^2
        fld     dword ptr [esi] # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i+1], Sxy[i], x[i]/sigma[i]^2, 1/sigma[i]^2, y[i]
        add     esi, 4
        fmul    st(1), st       # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i+1], Sxy[i], x[i]/sigma[i]^2, y[i]/sigma[i]^2, y[i]
        fmulp   st(2), st       # Stek: S[i+1], Sx[i+1], Sy[i], Sxx[i+1], Sxy[i], x[i]*y[i]/sigma[i]^2, y[i]/sigma[i]^2
        faddp   st(4), st       # Stek: S[i+1], Sx[i+1], Sy[i]+y[i]/sigma[i]^2=Sy[i+1], Sxx[i+1], Sxy[i], x[i]*y[i]/sigma[i]^2
        faddp                   # Stek: S[i+1], Sx[i+1], Sy[i+1], Sxx[i+1], Sxy[i]+x[i]*y[i]/sigma[i]^2=Sxy[i+1]
        loop next

        ## Izracunava se determinanta sistema i provjerava da li je ista
	## razlicita od 0.
        fld     st(4)           # Stek: S, Sx, Sy, Sxx, Sxy, S
        fmul    st, st(2)       # Stek: S, Sx, Sy, Sxx, Sxy, S*Sxx
        fld     st(4)           # Stek: S, Sx, Sy, Sxx, Sxy, S*Sxx, Sx
        fmul    st, st          # Stek: S, Sx, Sy, Sxx, Sxy, S*Sxx, Sx*Sx
        fsubp                   # Stek: S, Sx, Sy, Sxx, Sxy, S*Sxx-Sx*Sx=D
        fldz                    # Stek: S, Sx, Sy, Sxx, Sxy, D, 0
        fcomip  st, st(1)       # Stek: S, Sx, Sy, Sxx, Sxy, D
        je no_solution

        ## Izracunava se vrijednost a i upisuje u odgovarajucu
	## memorijsku lokaciju.
        fld     st(2)           # Stek: S, Sx, Sy, Sxx, Sxy, D, Sxx
        fmul    st, st(4)       # Stek: S, Sx, Sy, Sxx, Sxy, D, Sxx*Sy
        fld     st(5)           # Stek: S, Sx, Sy, Sxx, Sxy, D, Sxx*Sy, Sx
        fmul    st, st(3)       # Stek: S, Sx, Sy, Sxx, Sxy, D, Sxx*Sy, Sx*Sxy
        fsubp                   # Stek: S, Sx, Sy, Sxx, Sxy, D, Sxx*Sy-Sx*Sxy=Da
        fdiv    st, st(1)       # Stek: S, Sx, Sy, Sxx, Sxy, D, Da/D=a
        mov     ebx, [ebp+24]
        fstp    dword ptr [ebx] # Stek: S, Sx, Sy, Sxx, Sxy, D

        ## Izracunava se vrijednost b i upisuje u odgovarajucu
	## memorijsku lokaciju.
        fld     st(5)           # Stek: S, Sx, Sy, Sxx, Sxy, D, S
        fmul    st, st(2)       # Stek: S, Sx, Sy, Sxx, Sxy, D, S*Sxy
        fld     st(5)           # Stek: S, Sx, Sy, Sxx, Sxy, D, S*Sxy, Sx
        fmul    st, st(5)       # Stek: S, Sx, Sy, Sxx, Sxy, D, S*Sxy, Sx*Sy
        fsubp                   # Stek: S, Sx, Sy, Sxx, Sxy, D, S*Sxy-Sx*Sy=Db
        fdiv    st, st(1)       # Stek: S, Sx, Sy, Sxx, Sxy, D, Db/D=b
        mov     ebx, [ebp+28]
        fstp    dword ptr [ebx] # Stek: S, Sx, Sy, Sxx, Sxy, D

        ## Vraca se 1 kao rezultat funkcije.
        mov     eax, 1
        jmp finalize

no_solution:
        ## Vraca se 0 kao rezultat funkcije
        xor     eax, eax

finalize:
        ## Brisu se suvisne vrijednosti sa steka.
        fcompp                  # Stek: S, Sx, Sy, Sxx
        fcompp                  # Stek: S, Sx
        fcompp                  # Stek:
                
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
