.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.

.text
.global invest

	## Funkcija invest() simulira proces investiranja na berzi.
	## Argumenti funkcije su:
        ##   [ebp+8] - duzina perioda investiranja
        ##   [ebp+12] - adresa prvog elementa polja sa vrijednostima
	##              akcija tokom gornjeg perioda
        ##   [ebp+16] - pokazivac na lokaciju sa kolicinom novca sa
        ##              kojom se inicijalno raspolaze, a na koju takodje
        ##              treba upisati finalni iznos sa kojim se
        ##              raspolaze (u novcu i akcijama ukupno) na kraju
	##              datog perioda investiranja).
        ## Pretpostavke su da je period investiranja bar jedan dan, da
        ## su vrijednosti akcija brojevi veci ili jednaki 0, kao i da je
        ## inicijalna kolicina novca sa kojom se raspolaze veca od 0.
	## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
invest:
        ## Prolog funkcije.
	enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Registar ecx ce sluziti kao brojac i inicijalizuje se na
        ## duzinu perioda investiranja.  Registar esi ce sadrzati adresu
        ## tekuceg elementa iz polja sa vrijednostima akcija.  Registar
        ## edi ce sadrzati adresu lokacije sa iznosom sa kojim se krece
	## u investiranje, odn. na koju treba upisati finalni iznos.
        mov     ecx, [ebp+8]
        mov     esi, [ebp+12]
        mov     edi, [ebp+16]

        ## Registar xmm6 ce sadrzati kolicinu akcija, a registar xmm7
	## kolicinu novca koja se posjeduje.
        movss   xmm6, zero
        movss   xmm7, [edi]

        ## Registar eax ce sluziti kao brojac koliko dana uzastopno su
        ## akcije rasle (ako sadrzi pozitivan broj), odn. opadale (ako
	## sadrzi negativan broj).  Registar ebx ce sadrzati broj 0.
        xor     eax, eax
        xor     ebx, ebx

        ## Registar xmm0 ce sadrzati prethodnu, a registar xmm1 tekucu
        ## vrijednost akcija; drugi registar se inicijalizuje na
	## vrijednost akcija na pocetku perioda investiranja.
        movss   xmm1, [esi]
        add     esi, 4
        dec     ecx
        jecxz restore
next_stock:
        ## Pamti se prethodna vrijednost akcija u registru xmm0 i
        ## ucitava se tekuca vrijednost u registar xmm1.  Potom se
        ## porede ove dvije vrijednosti i preduzima se odgovarajuca
	## akcija.
        movss   xmm0, xmm1
        movss   xmm1, [esi]
        comiss  xmm1, xmm0
        ja stock_up
        jb stock_down
        jmp continue

stock_up:
        ## Ako su akcije prethodno padale 3 ili vise dana, obavlja se
        ## kupovina akcija.  U suprotnom, azurira se broj uzastopnih
	## dana za koje su akcije rasle.
        cmp     eax, -3
        jle buy_stock
        cmp     eax, 0
        cmovl   eax, ebx
        inc     eax
        jmp continue

buy_stock:
        ## Kupuju se akcije za sav raspolozivi novac.
        movss   xmm5, xmm7
        divss   xmm5, xmm1
        addss   xmm6, xmm5
        movss   xmm7, zero
        mov     eax, 1
        jmp continue

stock_down:
        ## Ako su akcije prethodno rasle 3 ili vise dana, obavlja se
        ## prodaja akcija.  U suprotnom, azurira se broj uzastopnih
	## dana za koje su akcije padale.
        cmp     eax, 3
        jge sell_stock
        cmp     eax, 0
        cmovg   eax, ebx
        dec     eax
        jmp continue

sell_stock:
        ## Prodaju se sve raspolozive akcije.
        movss   xmm5, xmm6
        mulss   xmm5, xmm1
        addss   xmm7, xmm5
        movss   xmm6, zero
        mov     eax, -1
        jmp continue

continue:
        ## Prelazi se na narednu vrijednost akcija.
        add     esi, 4
        loop next_stock

        ## Izracuva se finalni iznos sa kojim se raspolaze u trenutku
	## isteka perioda investiranja.
        mulss   xmm6, xmm1
        addss   xmm7, xmm6
        movss   [edi], xmm7

restore:        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor [esp]
        mov     esp, edx

done:        
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
