.intel_syntax noprefix

.data
two:    .float  2               # Broj 2.
mask:   .int    0x7fffffff, 0, 0, 0 # Maska za izracunavanje apsolutne vrijednosti broja na poziciji najmanje tezine u nekom SSE registru.

.text
.global newton

	## Funkcija newton() odredjuje relativnu gresku izracunavanja
        ## kvadratnog korijena datog broja Newton-ovom metodom sa
        ## trazenom preciznoscu u odnosu na kvadratni korijen istog
        ## broja izracunat direktno koriscenjem odgovarajuce instrukcije
	## mikroprocesora.  Argumenti funkcije su:
        ##   [ebp+8] - dati broj
        ##   [ebp+12] - trazena preciznost
        ##   [ebp+16] - adresa lokacije na koju treba upisati izracunatu
	##              relativnu gresku
        ## Pretpostavka je da je su dati broj, kao i trazena preciznost,
        ## veci od 0.  Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
newton:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Registar xmm0 ce sadrzati dati broj, a registar xmm1 tekucu
        ## vrijednost kvadratnog korijena koji se racuna.  Registar xmm4
        ## ce cuvati trazenu preciznost, registar xmm5 ce sadrzati broj
        ## 2, registar xmm6 ce sadrzati masku za izracunavanje apsolutne
        ## vrijednosti broja na poziciji najmanje tezine u SSE registru,
	## a registar xmm7 ce sadrzati vrijednost 0.
        movss   xmm0, [ebp+8]
        movss   xmm1, xmm0
        movss   xmm4, [ebp+12]
        movss   xmm5, two
        movups  xmm6, mask
        xorps   xmm7, xmm7

next_iteration:
        ## U registar xmm2 se kopira tekuca vrijednost kvadratnog
        ## korijena, a u registru xmm1 se izracunava njegova nova
	## vrijednost.
        movss   xmm2, xmm1
        movss   xmm1, xmm0
        divss   xmm1, xmm2
        addss   xmm1, xmm2
        divss   xmm1, xmm5

        ## Provjerava se da li je apsolutna vrijednost (obzirom da bit
        ## najvece tezine kod realnih brojeva predstavlja bit znaka, za
        ## racunanje apsolutne vrijednosti nekog realnog broja dovoljno
        ## je postaviti mu bit najvece tezine na 0) razlike tekuce i
        ## nove vrijednosti kvadratnog korijena manja ili jednaka od
        ## trazene preciznosti i, ako je to slucaj, prekida se sa daljim
	## iteracijama.
        subss   xmm2, xmm1
        andps   xmm2, xmm6
        comiss  xmm2, xmm4
        ja next_iteration

        ## Izracunava se kvadratni korijen datog broja koriscenjem
        ## odgovarajuce instrukcije mikroprocesora, i odredjuje se
        ## relativna greska gornjeg izracunavanja prema Newton-ovoj
        ## metodi u odnosu na ovu vrijednost; ova relativna greska se
	## potom upisuje u za to predvidjenu lokaciju u memoriji.
        sqrtss  xmm0, xmm0
        subss   xmm1, xmm0
        divss   xmm1, xmm0
        andps   xmm1, xmm6
        mov     ebx, [ebp+16]
        movss   [ebx], xmm1

        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor [esp]
        mov     esp, edx

done:
	## Epilog funkcije.
        pop     ebx
	leave
        ret
