.intel_syntax noprefix

.global midpoint
	
	## Funkcija midpoint() izracunava koordinate piksela date linije
	## midpoint algoritmom. Argumenti funkcije su:
	##   [ebp+8], [ebp+12] - koordinate prve krajnje tacke linije
	##   [ebp+16], [ebp+20] - koordinate druge krajnje tacke linije
	##   [ebp+24], [ebp+28] - polja u koja treba smjestiti izracunate
	##                        koordinate piksela
	## Pretpostavka je su druge koordinate vece od prvih po obje
	## ose, kao i da je razlika koordinata po x osi veca od razlike
	## koordinata po y osi. Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
midpoint:
        ## Prolog funkcije.
        enter	0, 0
	push	ebx
	push	esi
	push	edi

	## Analizom algoritma moze se ustanoviti da x koordinate rastu
	## za 1, tako da se u prvom prolazu postave vrijednosti ovih
	## koordinata u polju x. Registar esi cuva adresu tekuceg
	## elementa polja x, registar eax drzi vrijednost tog elementa,
	## a registar ecx sluzi kao brojac.
	cld
	mov	edi, [ebp+24]
	mov	eax, [ebp+8]
	mov	ecx, [ebp+16]
	sub	ecx, eax
	inc	ecx
next_x:
	stosd
	inc	eax
	loop next_x

	## Pri odredjivanju koordinata piksela po y osi, opet registar
	## esi cuva adresu tekuceg elementa polja y, registar eax
	## vrijednost tog elementa, a registar ecx sluzi kao
	## brojac. Registar ebx drzi vrijednost promjenljive d, a
	## registri edx i esi inkremente ove promjenljive respektivno za
	## slucajeve kada je d<0 i kada je d>=0. Prvo se inicijalizuju
	## svi registri.
	mov	edi, [ebp+28]
	mov	eax, [ebp+12]

	## Registar ecx se prvo postavlja na razliku koordinata o x osi.
	mov	ecx, [ebp+16]
	sub	ecx, [ebp+8]

	## Registar edx se prvo postavlja na razliku koordinata po y osi.
	mov	edx, [ebp+20]
	sub	edx, eax

	## Koristeci gornje vrijednosti, izracunavaju se vrijednsti koje
	## ce biti smjestene u registrima ebx i esi.
	mov	ebx, edx
	shl	ebx, 1
	sub	ebx, ecx
	mov	esi, edx
	sub	esi, ecx
	shl	esi, 1

	## Registri ecx i edx se dovode na potrebne vrijednosti.
	inc	ecx
	shl	edx, 1

next_y:
	## Upisuje se tekuca y koordinata.
	stosd

	## Ispituje se vrijednost pomocne promjenljive d;...
	cmp	ebx, 0
	jl d_less_than_zero

	## ...ukoliko je ta vrijednost veca ili jednaka 0, inkrementira
	## se tekuca y koordinata i dodaje se odgovarajuca vrijednost
	## promjenljivoj d,...
	inc	eax
	add	ebx, esi
	jmp continue

d_less_than_zero:
	## ...a u suprotonom se samo uvecava promjenljiva d za
	## odgovarajucu vrijednost.
	add	ebx, edx
	
continue:
	## U oba slucaja, nastavlja se sa iteracijama.
	loop next_y
	
	## Epilog funkcije.
	pop	edi
	pop	esi
	pop	ebx
        leave
        ret
