.intel_syntax noprefix

.data
two:	.int	2	# Broj 2.
four:	.int	4	# Broj 4.
	
.text
.global step

	## Funkcija step() izracunava vrijednost parcijalne sume
	## Fourier-ovog reda step funkcije u datoj tacki. Argumenti
	## funkcije su:
	##   [ebp+8] - vrijednost za koju se racuna funkcija
	##   [ebp+12] - broj sabiraka koji ce biti uracunati
	## Pretpostavlja se da je vrijednost za koju se racuna funkcija
	## u opsegu [-pi,pi], kao i da je broj sabiraka veci od
	## 0. Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
step:
        ## Prolog funkcije.
        enter	0, 0

	## Registar ecx ce biti koriscen kao brojac.
	mov	ecx, [ebp+12]

	## Inicijalizuje se suma koja predstavlja aproksimaciju step
	## funkcije za datu vrijednost. Ova suma ce u komentarima biti
	## oznacavana sa S.
	fldz			# Stek:	 0=S

	## Izracunava se faktor 4/pi.
	fldpi			# Stek:	 S, pi
	fidivr dword ptr four	# Stek:	 S, 4/pi

	## Dodaju se vrijednost za koju se racuna funkcija i term
	## 2*k+1 na stek.
	fld	dword ptr [ebp+8]	# Stek:	 S, 4/pi, x
	fld1				# Stek:	 S, 4/pi, x, 1=2*0+1

next_term:
	## Izracunava se naredni sabirak iz Fourier-ove transformacije
	## step funkcije i dodaje u sumu.
	fld	st		# Stek:	 S, 4/pi, x, 2*k+1, 2*k+1
	fmul	st(2)		# Stek:	 S, 4/pi, x, 2*k+1, (2*k+1)*x
	fsin			# Stek:	 S, 4/pi, x, 2*k+1, sin((2*k+1)*x)
	fdiv	st(1)		# Stek:	 S, 4/pi, x, 2*k+1, sin((2*k+1)*x)/((2*k+1)*x)
	fmul	st(3)		# Stek:	 S, 4/pi, x, 2*k+1, 4*pi/sin((2*k+1)*x)/((2*k+1)*x)
	faddp	st(4), st	# Stek:	S+4*pi/sin((2*k+1)*x)/((2*k+1)*x)->S, 4/pi, x, 2*k+1

	## Prelazi se na naredni sabirak.
	fiadd	dword ptr two	# Stek:	S, 4/pi, x, 2*k+1+2=2*(k+1)+2
	loop next_term
	
done:
	## Cisti se stek.
	ffree	st		# Stek:	S, 4/pi, x, invalidna vrijednost
	fincstp			# Stek:	S, 4/pi, x
	ffree	st		# Stek:	S, 4/pi, invalidna vrijednost
	fincstp			# Stek:	S, 4/pi
	ffree	st		# Stek:	S, invalidna vrijednost
	fincstp			# Stek:	S
	
	## Epilog funkcije.
	leave
        ret
