.intel_syntax noprefix
			
.text
.global convert

        ## Funkcija convert() vrsi konvertovanje datog broja iz jedne
	## baze u drugu. Argumenti funkcije su:
        ##   [ebp+8] - string sa ciframa broja koji se konvertuje
        ##   [ebp+12] - baza iz koje se vrsi konverzija
        ##   [ebp+16] - string u koji se smjestaju cifre broja dobijenog
        ##              konverzijom
        ##   [ebp+20] - baza iz koje se vrsi konverzija
        ## Funkcija vraca 1 ako konverzija prodje u redu, odnosno 0 ako
	## naidje na neku sintaksnu gresku u broju koji se konvertuje.
        ## Smatra se da su obje baze brojevi u intervalu [2,36], kao i
	## da su cifre od 10 do 35 predstavljene odgovarajuci malim ili
        ## velikim slovima engleskog alfabeta.  Funkcija se u
        ## potpunosti podvrgava C konvencijama pozivanja.
        
convert:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Sve operacije nad stringovima se izvode od pocetka stringova.
        cld

        ## Odredjuje se broj koji je predstavljen datim stringom.
        ## Registar esi pokazuje na tekuci karakter u stringu.  U
	## registru ebx se cuva baza u kojoj je predstavljen broj.  U
	## registru ecx se izdvaja tekuca cifra.  Broj koji je
	## predstavljen stringom se akumulira u registru edi.
        mov     esi, [ebp+8]
        mov     ebx, [ebp+12]
        xor     ecx, ecx
        xor     edi, edi

parse_next_digit:
        ## Ucitava se naredni karakter i provjerava se da li se doslo do
	## kraja stringa.
        lodsb
        cmp     al, 0
        je convert_number

        ## Ako je karakter cifra, pretvara se u odgovarajuci broj.
        cmp     al, '0'
        jb check_lower_letter
        cmp     al, '9'
        ja check_lower_letter
        sub     al, '0'
        jmp check_digit
     
check_lower_letter:
        ## Ako je karakter malo slovo, pretvara se u odgovarajuci broj.
        cmp     al, 'a'
        jb check_upper_letter
        cmp     al, 'z'
        ja check_upper_letter
        sub     al, 'a'
        add     al, 10
        jmp check_digit

check_upper_letter:
        ## Ako je karakter veliko slovo, pretvara se u odgovarajuci broj.
        cmp     al, 'A'
        jb error
        cmp     al, 'Z'
        ja error
        sub     al, 'A'
        add     al, 10

check_digit:
        ## Prebacuje se cifra u registar ecx i provjerava da nije
        ## eventualno veca ili jednaka bazi, u kom slucaju se
	## prijavljuje greska.
        mov     cl, al
        cmp     ecx, ebx
        jae error

        ## Mnozi se tekuca vrijednost broja sa bazom i toj vrijednosti
	## se dodaje upravo parsirana cifra, cime se cifra zaracunava u
	## broj.
        mov     eax, edi
        mul     ebx
        add     eax, ecx
        mov     edi, eax
        jmp parse_next_digit

convert_number:
        ## Sada se pretvara dati broj u zeljenu bazu.  Broj se
	## sukcesivno dijeli sa bazom, cime se izdvajaju cifre u datoj bazi
	## kao ostaci dijeljenja.  Cifre se medjutim izdvajaju pocev od
	## cifre najmanje tezine, tako da se iste prvo smjestaju na stek.
        ## U registru eax se drzi broj, a u registru ebx baza u koju se
	## zeli konvertovati broj.  Registar ecx broji koliko broj ima
	## cifara.
        mov     eax, edi
        xor     ecx, ecx
        mov     ebx, [ebp+20]

push_next_digit:
        ## Dijeli se broj sa bazom i ostatak dijeljenja, odnosno
	## izdvojena cifra, se stavlja na stek.
        cmp     eax, 0
        je store_number
        xor     edx, edx
        div     ebx
        push    edx
        inc     ecx
        jmp push_next_digit

store_number:
        ## Smjesta se konvertovani broj u dati string.  Registar edi ce
        ## pokazivati na tekuci karakter stringa.  Jedna po jedna cifra
        ## stringa se skida sa steka, pretvara u odgovarajucu cifru
	## odnosno slovo i smjesta u dati string.
        mov     edi, [ebp+16]
store_next_digit:
        pop     eax
        cmp     eax, 10
        jae convert_to_letter
        add     al, '0'
        jmp do_store
convert_to_letter:
        sub     al, 10
        add     al, 'a'
do_store:
        stosb        
        loop store_next_digit

        ## Terminira se nulom rezultujuci string.
        xor     eax, eax
        stosb
        jmp ok

error:
        ## U slucaju greske, funkcija vraca 0.
        xor     eax, eax
        jmp     finish

ok:
        ## Ako sve prodje u redu, funkcija vraca 1.
        xor     eax, eax
        inc     eax
       
finish: 	
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
