.intel_syntax noprefix

.text
.global correlation

	## Funkcija correlation() racuna koeficijent korelacije datih
	## nizova.  Argumenti funkcije su:
	##   [ebp+8] - duzina nizova
	##   [ebp+12], [ebp+16] - adrese nizova
        ## Duzina nizova mora biti broj veci od 0.  Funkcija vraca
	## izracunati koeficijent korelacije. Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
correlation:
        ## Prolog funkcije.
        enter	0, 0
        push    esi
        push    edi

        ## Poziva se funkcija mean() da se izracuna aritmeticka sredina
	## prvog niza (u oznaci xm).
        mov     eax, [ebp+12]
        push    eax
        mov     eax, [ebp+8]
        push    eax
        call mean               # Stek: xm
        add     esp, 8

        ## Poziva se funkcija mean() da se izracuna aritmeticka sredina
	## drugog niza (u oznaci ym).
        mov     eax, [ebp+16]
        push    eax
        mov     eax, [ebp+8]
        push    eax
        call mean               # Stek: xm, ym
        add     esp, 8

        ## Na stek se stavljaju pocetne vrijednosti za dvije sume u
        ## imeniocu i sumu u brojiocu izraza za korelaciju (ove sume ce
	## biti oznacene sa Sx2, Sy2 i Sxy).
        fldz                    # Stek: xm, ym, 0=Sx2
        fldz                    # Stek: xm, ym, Sx2, 0=Sy2
        fldz                    # Stek: xm, ym, Sx2, Sy2, 0=Sxy

        ## Registar ecx ce biti koriscen kao brojac.  Registar esi ce
	## pokazivati na tekuci element prvog niza, a registar edi na
	## tekuci element drugog niza.
        mov     ecx, [ebp+8]
        mov     esi, [ebp+12]
        mov     edi, [ebp+16]

        ## U petlji se formiraju suma u brojiocu, odnosno sume u
	## imeniocu izraza za korelaciju.
next_elements:
        ## Izracunava se x(i)-xm.  Pritom se azurira registar esi da
	## pokazuje na naredni element prvog niza.
        fld     dword ptr [esi] # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)
        add     esi, 4
        fsub    st, st(5)       # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm

        ## Izracunava se (x(i)-xm)^2 i dodaje u sumu Sx2.
        fld     st              # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, x(i)-xm
        fmul    st, st          # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, (x(i)-xm)^2
        faddp   st(4), st       # Stek: xm, ym, Sx2+(x(i)-xm)^2=Sx2, Sy2, Sxy, x(i)-xm

        ## Izracunava se y(i)-ym.  Pritom se azurira registar edi da
	## pokazuje na naredni element drugog niza.
        fld     dword ptr [edi] # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, y(i)
        add     edi, 4
        fsub    st, st(5)       # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, y(i)-ym

        ## Izracunava se (y(i)-ym)^2 i dodaje u sumu Sy2.
        fld     st              # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, y(i)-ym, y(i)-ym
        fmul    st, st          # Stek: xm, ym, Sx2, Sy2, Sxy, x(i)-xm, y(i)-ym, (y(i)-ym)^2
        faddp   st(4), st       # Stek: xm, ym, Sx2, Sy2+(y(i)-ym)^2=Sy2, Sxy, x(i)-xm, y(i)-ym

        ## Izracunava se (x(i)-xm)*(y(i)-ym) i dodaje u sumu Sxy.
        fmulp                   # Stek: xm, ym, Sx2, Sy2, Sxy, (x(i)-xm)*(y(i)-ym)
        faddp                   # Stek: xm, ym, Sx2, Sy2, Sxy+(x(i)-xm)*(y(i)-ym)=Sxy
        loop next_elements

        ## Zamjenjuje se suma u brojiocu sa aritmetickom sredinom prvog
	## niza na steku i brise se ova aritmeticka sredina sa steka.
        fxch    st(4)           # Stek: Sxy, ym, Sx2, Sy2, xm
        ffree   st              # Stek: Sxy, ym, Sx2, Sy2, invalidna vrijednost
        fincstp                 # Stek: Sxy, ym, Sx2, Sy2

        ## Izracunava se imenilac izraza za korelaciju i zamjenjuje sa
        ## aritmetickom sredinom drugog niza na steku, a potom se brise
	## ova aritmeticka sredina sa steka.
        fmulp                   # Stek: Sxy, ym, Sx2*Sy2
        fsqrt                   # Stek: Sxy, ym, sqrt(Sx2*Sy2)
        fxch                    # Stek: Sxy, sqrt(Sx2*Sy2), ym
        ffree   st              # Stek: Sxy, sqrt(Sx2*Sy2), invalidna vrijednost
        fincstp                 # Stek: Sxy, sqrt(Sx2*Sy2)

        ## Izracunava se vrijednost korelacije.
        fdivp                   # Stek: Sxy/sqrt(Sx2*Sy2)

        ## Epilog funkcije.
        pop     edi
        pop     esi
        leave
        ret

        ## Funkcija mean() racuna aritmeticku sredinu datog
	## niza. Argumenti funkcije su:
	##   [ebp+8] - duzina niza
	##   [ebp+12] - adresa niza
        ## Duzina niza mora biti broj veci od 0.  Funkcija vraca
	## izracunatu aritmeticku sredinu. Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
mean:
        ## Prolog funkcije.
        enter   0, 0

        ## Registar ecx ce biti koriscen kao brojac, registar edx ce
	## sadrzati adresu tekuceg elementa niza.  Na steku se
	## inicijalizuje suma elemenata niza (u oznaci S).
        mov     ecx, [ebp+8]
        mov     edx, [ebp+12]
        fldz                            # Stek: 0=S

        ## U petlji se sabiraju elementi niza.  Svaki put kada se ucita
	## neki element niza, inkrementira se adresa tekuceg elementa.
next_element:   
        fadd    dword ptr [edx]         # Stek: S+x(i)=S
        add     edx, 4
        loop next_element

        ## Dijeli se suma elemenata niza sa brojem elemenata niza.
        fidiv   dword ptr [ebp+8]       # Stek: S/n

        ## Epilog funkcije.
        leave
        ret
