.intel_syntax noprefix
                
.text
.global parity

        ## Funkcija parity() ispituje da li data kvadratna logicka
	## matrica zadovoljava svojstvo parnosti.  Argumenti funkcije
	## su:
        ##   [ebp+8] - dimenzija matrice
        ##   [ebp+12] - adresa prvom elementa matrice (elementi matrice su
        ##              predstavljeni jednodimenzionim nizom, po redovima)
        ## Pretpostavlja se da je dimenzija matrice broj veci od 0.
        ## Funkcija vraca 1 ako matrica zadovoljava svojstvo parnosti, 0
        ## u suprotnom. Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
parity:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Postavlja se direction fleg i ucitava se dimenzija matrice u
	## registar ebx.
        cld
        mov     ebx, [ebp+8]

        ## Svojstvo parnosti ce biti prvo ispitano za redove matrice.  U
	## spoljasnoj petlji se prolazi kroz svaki red matrice i
        ## provjerava se da li tekuci red ima paran broj jedinica.
        ## Registar esi sadrzi adresu tekuceg elementa matrice.
        ## Registar ecx se koristi kao brojac u spoljasnjoj petlji i
	## inicijalizuje se na dimenziju matrice.
        mov     esi, [ebp+12]
        mov     ecx, ebx
        
next_row:
        ## Registar ecx se reinicijalizuje, posto se koristi kao brojac
        ## i u unutrasnjoj petlji.  Registar edx ce sadrzati broj
	## jedinica u tekucem redu do tekuceg elementa.
        push    ecx
        mov     ecx, ebx
        xor     edx, edx

next_row_element:
        ## Ucitava se tekuci element tekuceg reda i zaracunava se
        ## njegova vrijednost u tekucu parnost koja se cuva u registru
	## edx.  Potom se prelazi na naredni element tekuceg reda.
        lodsd
        add     edx, eax
        loop next_row_element

        ## Provjerava se da li je svojstvo parnosti zadovoljeno za
	## tekuci red matrice.  Ukoliko to nije slucaj, odmah se zavrsava
	## funkcija sa negativnim ishodom, a u suprotnom se nastavlja sa
	## testiranjem.
        pop     ecx
        test    edx, 1
        jnz false
        loop next_row

        ## Svojstvo parnosti ce sada biti ispitano za kolone matrice, na
        ## slican nacin kao za redove.  Registri esi i ecx se koriste na
	## isti nacin kao u prvom dijelu funkcije.
        mov     esi, [ebp+12]
        mov     ecx, ebx

next_column:
        ## I u unutrasnjoj petlji, registri ecx i edx se koriste na isti
	## nacin kao u prvom dijelu funkcije.  Registar edi cuva adresu
        ## prvog elementa tekuce kolone matrice tokom unutrasnje petlje.
	## Registar ebx ce tokom unutrasnje petlje umjesto dimenzije
	## matrice cuvati rastojanje u bajtovima izmedju dva susjedna
	## elementa iste kolone matrice.
        push    ecx
        mov     ecx, ebx
        xor     edx, edx
        mov     edi, esi
        shl     ebx, 2

next_column_element:
        ## Ucitava se tekuci element tekuce kolone i zaracunava se
        ## njegova vrijednost u tekucu parnost koja se cuva u registru
	## edx.  Potom se prelazi na naredni element tekuce kolone.
        mov     eax, [esi]
        add     edx, eax
        add     esi, ebx
        loop next_column_element

        ## Provjerava se da li je svojstvo parnosti zadovoljeno za
        ## tekucu kolonu matrice.  Ukoliko to nije slucaj, odmah se
	## zavrsava funkcija sa negativnim ishodom, a u suprotnom se
	## nastavlja sa testiranjem.
        shr     ebx, 2
        mov     esi, edi
        add     esi, 4
        pop     ecx
        test    edx, 1
        jnz false
        loop next_column

        ## Matrica zadovoljava svojstvo parnosti.
        mov     eax, 1
        jmp finish

false:
        ## Matrica ne zadovoljava svojstvo parnosti.
        xor     eax, eax

finish: 
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
