.intel_syntax noprefix

.data
zero:   .float  0               # Broj 0.

.text
.global diffuse

	## Funkcija diffuse() odredjuje iznos difuznog osvjetljenja u
	## datoj tacki na povrsi. Argumenti funkcije su:
	##   [ebp+8] - vektor normale na povrs u datoj tacki
        ##   [ebp+12] - vektor od date tacke prema poziciji svijetla
        ##   [ebp+16] - koeficijenti difuzne reflekcije materijala povrsi
        ##   [ebp+20] - komponente intenziteta svijetla
        ##   [ebp+24] - niz u koji treba upisati izracunate komponente
        ##              difuznog osvjetljenja
        ## Pretpostavlja se da su vektori normalizovani, kao i da su
	## koeficijenti difuzne reflekcije materijala povrsi kao i
	## komponente intenziteta svijetla brojevi iz opsega [0,1].
	## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
diffuse:
        ## Prolog funkcije.
	enter	16, 0
        push    ebx
        push    esi
        push    edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov     eax, 1
        cpuid
        test    edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara (i registara koprocesora).
        mov     edx, esp
        and     esp, 0xfffffff0
        sub     esp, 512
        fxsave  [esp]

        ## Postavlja se smjer za instrukcije za rad sa stringovima.
        cld     

        ## U krajnje desni element registra xmm7, kao i u poslednji
	## element pomocnog niza od 4 realna broja koji je alociran kao
	## lokalna promjenljiva, se upisuje 0.
        movss   xmm7, zero
        movss   [ebp-4], xmm7

        ## Preko pomocnog niza, u registar xmm0 se upisuje vektor
	## normale u datoj tacki povrsi N.
        mov     ecx, 3
        mov     esi, [ebp+8]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm0, [ebp-16]

        ## Na isti nacin, u registar xmm1 se upisuje vektor pravca
	## osvjetljenja L.
        mov     ecx, 3
        mov     esi, [ebp+12]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm1, [ebp-16]

        ## Mnoze se pojedinacne komponente datih vektora i rezultat se
	## smjesta u registar xmm0.  Treba primijetiti da je krajnji
	## lijevi element u ovom registru i dalje jednak 0.
        mulps   xmm0, xmm1

        ## Prebacuje se gornja polovina registra xmm1 u registar xmm0 i
	## onda se sabiraju elementi tih registara.  Na ovaj nacin se u
	## dva desna elementa registra xmm0 dobijaju dvije vrijednosti
	## ciji je zbir jednak skalarnom proizvodu vektora N i L.
        movhlps xmm1, xmm0
        addps   xmm0, xmm1

        ## Kopira se drugi element zdesna iz registra xmm0 u svaki
        ## element registra xmm1, a potom se sabiraju krajnji desni
        ## elementi registara xmm0 i xmm1, cime se dobija skalarni 
        ## proizvod vektora N i L.
        movaps  xmm1, xmm0
        shufps  xmm1, xmm1, 0x55
        addss   xmm0, xmm1

        ## Poredi se skalarni proizvod sa nulom i u svaki element
	## registra xmm0 se upisuje veci od ova dva broja.
        maxss   xmm0, xmm7
        shufps  xmm0, xmm0, 0x00

        ## Ako je veci broj jednak 0, onda i svaka komponenta difuznog
	## osvjetljenja mora biti jednaka 0, pa se odmah prelazi na upis
	## rezultata (koji je formiran u registru xmm0).
        comiss  xmm0, xmm7
        je result_write
        
        ## Preko pomocnog niza, u registar xmm1 se upisuju koeficijenti
	## difuzne refleksije materijala kd.
        mov     ecx, 3
        mov     esi, [ebp+16]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm1, [ebp-16]

        ## Na isti nacin, u registar xmm2 se upisuju komponente
	## intenziteta svijetla I.
        mov     ecx, 3
        mov     esi, [ebp+20]
        lea     edi, [ebp-16]
        rep movsd
        movups  xmm2, [ebp-16]

        ## Mnoze se koeficijenti difuzne refleksije sa odgovarajucim
	## komponentama intenziteta svijetla.
        mulps   xmm1, xmm2

        ## Svaka od ovako dobijenih komponenti mnozi se sa skalarnim
	## proizvodom vektora N i L, cime se dobija trazeni iznos
	## difuznog osvjetljenja.
        mulps   xmm0, xmm1

result_write:
        ## Preko pomocnog niza, upisuje se rezultat u niz odredjen
	## poslednjim argumentom funkcije.
        movups  [ebp-16], xmm0
        mov     ecx, 3
        lea     esi, [ebp-16]
        mov     edi, [ebp+24]
        rep movsd
        
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
	## registre.
        fxrstor [esp]
        mov     esp, edx

done:	
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
