.intel_syntax noprefix

.data
f_2:    .float  2       # Broj 2.
f_1_5:  .float  1.5     # Broj 1.5.
        
.text
.global mandelbrot

	## Funkcija mandelbrot() popunjava dato polje piksela slikom
	## Mandelbrot-ovog fraktala.  Argumenti funkcije su:
        ##   [ebp+8] - dimenzija slike
        ##   [ebp+24] - adresa prvog piksela u polju piksela
        ## Pretpostavlja se da je dimenzija slike broj veci od 0.
        ## Pikseli su predstavljeni karakterima, pocev od donjeg reda
        ## slike prema gornjem redu.  Funkcija upisuje ASCII kod
        ## karaktera '1' za crnu boju odn. ASCII kod karaktera '0' za
        ## bijelu boju.  Funkcija se u potpunosti podvrgava C
	## konvencijama pozivanja.
mandelbrot:
        ## Prolog funkcije.
	enter   0, 0
        push    ebx
        push    edi

        ## Registar edx ce cuvati dimenziju slike, a registar edi ce
        ## pokazivati na karakter koji odgovara tekucem pikselu slike.
        mov     edx, [ebp+8]
        mov     edi, [ebp+12]

        ## Na stek se stavlja kvadrat norme koja predstavlja granicnu
	## vrijednost tokom iteracija.
        fld     dword ptr [f_2]         # Stek: 2=limit
        fmul    st, st                  # Stek: limit^2

        ## Registar ebx ce u petlji po redovima sadrzati indeks tekuceg
        ## reda.  Broj redova piksela je veci od 0, tako da ova petlja
        ## mora imati bar jednu iteraciju i zato se ispitivanje uslova
	## izlaska iz petlje moze ostaviti za kraj petlje.
        xor     ebx, ebx
next_row:
        ## Izracunava se vrijednost Ci za dati red.
        push    ebx
        fild    [esp]                   # Stek: limit^2, y
        add     esp, 4
        fmul    dword ptr [f_2]         # Stek: limit^2, 2*y
        fidiv   dword ptr [ebp+8]       # Stek: limit^2, 2*y/h
        fld1                            # Stek: limit^2, 2*y/h, 1
        fsubp                           # Stek: limit^2, 2*y/h-1=Ci

        ## Registar ebx ce u petlji po kolonama sadrzati indeks tekuce
        ## kolone.  Broj kolona piksela je veci od 0, tako da ova petlja
        ## mora imati bar jednu iteraciju i zato se ispitivanje uslova
        ## izlaska iz petlje moze ostaviti za kraj petlje.
        xor     eax, eax
next_pixel:
        ## Izracunava se vrijednost Cr za datu kolonu.
        push    eax
        fild    [esp]                   # Stek: limit^2, Ci, x
        add     esp, 4
        fmul    dword ptr [f_2]         # Stek: limit^2, Ci, 2*x
        fidiv   dword ptr [ebp+8]       # Stek: limit^2, Ci, 2*x/w
        fsub    dword ptr [f_1_5]       # Stek: limit^2, Ci, 2*x/w-1.5=Cr

        ## Stavljaju se na stek inicijalne vrijednosti za Zr i Zi.
        fldz                            # Stek: limit^2, Ci, Cr, 0=Zr[0]=Zr[i]
        fldz                            # Stek: limit^2, Ci, Cr, Zr[i], 0=Zi[0]=Zi[i]

        ## Za svaki piksel, data iterativna procedura se ponavlja
	## najvise 50 puta.
        mov     ecx, 50
next_iteration:
        ## Stavljaju se na stek kvadrati tekucih vrijednosti Zr i Zi.
        fld     st(1)                   # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]
        fmul    st, st(2)               # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i]
        fld     st(1)                   # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i], Zi[i]
        fmul    st, st(2)               # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i], Zi[i]*Zi[i]

        ## Provjerava se da li je norma kompleksnog broja odredjenog
        ## vrijednostima Zr i Zi veca od zadate granicne vrijednosti za
	## normu i ako jeste odmah se prekida sa iteracijama.
        fld     st(1)                   # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i], , Zi[i]*Z[i], Zr[i]*Zr[i]
        fadd    st, st(1)               # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i], Zi[i]*Z[i], Zr[i]*Zr[i]+Zi[i]*Zi[i]
        fcomip  st, st(7)               # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i], Zi[i]*Z[i]
        ja white

        ## Izracunavaju se vrijednosti za Zr i Zi za narednu iteraciju.
        fsubp                           # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i]-Zi[i]*Z[i]
        fadd    st, st(3)               # Stek: limit^2, Ci, Cr, Zr[i], Zi[i], Zr[i]*Zr[i]-Zi[i]*Z[i]+Cr=Zr[i+1]
        fxch    st(2)                   # Stek: limit^2, Ci, Cr, Zr[i+1], Zi[i], Zr[i]
        fmulp                           # Stek: limit^2, Ci, Cr, Zr[i+1], Zr[i]*Zi[i]
        fmul    dword ptr [f_2]         # Stek: limit^2, Ci, Cr, Zr[i+1], 2*Zr[i]*Zi[i]
        fadd    st(3)                   # Stek: limit^2, Ci, Cr, Zr[i+1], 2*Zr[i]*Zi[i]+Ci=Zi[i+1]

        ## Prelazi se na narednu iteraciju.
        loop next_iteration

        ## U slucaju da je norma kompleksnog broja odredjenog
        ## vrijednostima Zr i Zi ostala manja od date granicne
        ## vrijednosti kroz sve iteracije, tekuci piksel se boji crnom
	## bojom.
        mov     cl, '1'
        jmp pixel_write

white:
        ## Skidaju se sa steka kvadrati vrijednosti Zr i Zi i tekuci
	## piksel se boji bijelom bojom.
        fcompp                          # Stek: limit^2, Ci, Cr, Zr[i], Zi[i]
        mov     cl, '0'

pixel_write:
        ## Upisuje se vrijednost tekuceg piksela i inkrementira se
        ## adresa tekuceg piksela tako da pokazuje na naredni piksel.
        mov     [edi], cl
        inc     edi

        ## Skidaju se sa steka vrijednosti Zr i Zi, kao i vrijednost Cr
	## i prelazi se na narednu kolonu.
        fcompp                          # Stek: limit^2, Ci, Cr
        fcomp   st                      # Stek: limit^2, Ci
        inc     eax
        cmp     eax, edx
        jl next_pixel

        ## Skida se sa steka vrijednost Ci i prelazi se na naredni red.
        fcomp                           # Stek: limit^2
        inc     ebx
        cmp     ebx, edx
        jl next_row

        ## Skida se sa steka kvadrat granicne vrijednosti norme.
        fcomp   st                      # Stek:
        
	## Epilog funkcije.
        pop     edi
        pop     ebx
	leave
        ret
