.intel_syntax noprefix

.data
l:      .float  697
        .float  770
        .float  852
        .float  941             # Frekvencije signala po redovima.
h:      .float  1209
        .float  1336
        .float  1477            # Frekvencije signala po kolonama.
two:    .int    2               # Broj 2.
        
.text
.global volume

	## Funkcija volume() izracunava jacinu signala tonskog biranja.
	## Argumenti funkcije su:
        ##   [ebp+8] - ASCII kod oznake na tipki koja je pritisnuta
        ##   [ebp+12] - trenutak u kome se izracunava jacina signala
        ## Pretpostavke su da se dati ASCII kod zaista odnosi na jedan
        ## od karaktera (cifre od '0' do '9', ili '*' ili ' # ') koji se
        ## mogu naci na tipkama uredjaja, kao i da je dati trenutak u
        ## kome se izracunava jacina signala veci ili jednak od 0.
        ## Funkcija vraca izracunatu vrijednost jacine signala. Funkcija se
	## u potpunosti podvrgava C konvencijama pozivanja.
volume:
        ## Prolog funkcije.
	enter   0, 0

        ## Izracunavaju se indeksi reda i kolone tipke koja je
        ## pritisnuta.  Ukoliko je pritisnuta cifra od '1' do '9',
	## indeksi reda i kolone se nalaze kao kolicnik i ostatak pri
	## dijeljenju indeksa cifre u ovom opsegu sa brojem 3.
        mov     eax, [ebp+8]
        cmp     eax, '0'
        jl non_digit
        cmp     eax, '9'
        jg non_digit
        cmp     eax, '0'
        je zero
        sub     eax, '1'
        cdq
        mov     ecx, 3
        idiv    ecx
        jmp calculation

non_digit:
        ## Ako je u pitanju tipka '#', indeks reda i kolone se direktno
	## odredjuju,...
        cmp     eax, '*'
        je star
        mov     eax, 3
        mov     edx, 2
        jmp calculation
        
star:
        ## ...isto kao za tipku '*',...
        mov     eax, 3
        mov     edx, 0
        jmp calculation

zero:
        ## ...odn. tipku '0'.
        mov     eax, 3
        mov     edx, 1

calculation:
        ## Izracunava se jacina signala, pri cemu se koriste vrijednosti
	## niske odn. visoke frekvencije koje odgovaraju datoj tipki.
        fild    two             # Stek: 2
        fldpi                   # Stek: 2, pi
        fmulp                   # Stek: 2*pi
        fmul    [ebp+12]        # Stek: 2*pi*t
        fld     [4*eax+l]       # Stek: 2*pi*t, l
        fmul    st, st(1)       # Stek: 2*pi*t, 2*pi*l*t
        fsin                    # Stek: 2*pi*t, sin(2*pi*l*t)
        fxch                    # Stek: sin(2*pi*l*t), 2*pi*t
        fld     [4*edx+h]       # Stek: sin(2*pi*l*t), 2*pi*t, h
        fmulp                   # Stek: sin(2*pi*l*t), 2*pi*h*t
        fsin                    # Stek: sin(2*pi*l*t), sin(2*pi*h*t)
        faddp                   # Stek: sin(2*pi*l*t)+sin(2*pi*h*t)
        
	## Epilog funkcije.
	leave
        ret
