.intel_syntax noprefix

.text
.global moving_averages

	## Funkcija moving_averages() izracunava vrijednosti elemenata
        ## vektora "pokretnih" aritmetickih sredina za dati vektor.
	## Argumenti funkcije su:
        ##   [ebp+8] - duzina datog vektora
        ##   [ebp+12] - adresa prvog elementa datog vektora
        ##   [ebp+16] - red "pokretnih" aritmetickih sredina
        ##   [ebp+20] - adresa prvog elementa u vektoru u kome treba
        ##              upisati izracunate vrijednosti "pokretnih"
	##              aritmetickih sredina
        ## Pretpostavka je da je duzina datog vektora broj veci od 0, te
        ## da je red "pokretnih" aritmetickih sredina pozivitan broj
        ## koji je manji ili jednak od duzine datog vektora.  Funkcija
	## se u potpunosti podvrgava C konvencijama pozivanja.
moving_averages:
        ## Prolog funkcije.
        enter           0, 0
        push            ebx
        push            esi
        push            edi

        ## Provjerava se da li procesor podrzava SSE instrukcije.
        mov             eax, 1
        cpuid
        test            edx, 0x2000000
        jz done

        ## Cuva se sadrzaj SSE registara i registara koprocesora.
        mov             edx, esp
        and             esp, 0xfffffff0
        sub             esp, 512
        fxsave          [esp]

        ## Registar ebx ce sadrzati adresu elementa datog vektora koji
        ## se izbacuje iz sume, registar eax ce sadrzati red "pokretnih"
        ## aritmetickih sredina, a registar edi ce sadrzati adresu
        ## tekuceg elementa vektora izracunatih "pokretnih" aritmetickih
	## sredina.
        mov             ebx, [ebp+12]
        mov             eax, [ebp+16]
        mov             edi, [ebp+20]

        ## Prvo ce biti izracunat prvi element vektora "pokretnih"
        ## aritmetickih sredina; tokom ovog izracunavnaja, egistar esi
        ## ce sadrzati adresu tekuceg elementa datog vektora, a registar
	## ecx ce sluziti kao brojac.
        mov             esi, ebx
        mov             ecx, eax

        ## Registar xmm0 ce akumulirati parcijalne sume pri racunanju
        ## prvog elementa vektora "pokretnih" aritmetickih sredina, a
	## registar xmm7 ce sadrzati red "pokretnih" aritmetickih sredina.
        xorps           xmm0, xmm0
        cvtsi2ss        xmm7, eax

next_four:
        ## Provjerava se da li je preostalo 4 ili vise elementa ulaznog
	## vektora za racunanje vrijednosti prvog elementa vektora
	## "pokretnih" aritmetickih sredina.
        cmp             ecx, 4
        jl next_one

        ## Ucitavaju se naredna 4 elementa ulaznog vektora i
        ## zaracunavaju u parcijalne sume, a zatim se azuriraju registar
        ## koji pokazuje na adresu tekuceg elementa ulaznog vektora, kao
	## i brojac.
        movups          xmm1, [esi]
        addps           xmm0, xmm1
        add             esi, 16        
        sub             ecx, 4
        jmp next_four

next_one:
        ## Provjerava se da li je preostao jos neki element ulaznog
	## vektora za racunanje vrijednosti prvog elementa vektora
	## "pokretnih" aritmetickih sredina.
        cmp             ecx, 0
        je calculate_sum

        ## Zaracunava se tekuci element ulaznog vektora u parcijalnu
        ## sumu, a zatim se azuriraju registar koji pokazuje na adresu
	## tekuceg elementa ulaznog vektora, kao i brojac.
        addss           xmm0, [esi]
        add             esi, 4
        dec             ecx        
        jmp next_one        

calculate_sum:
        ## Vrsi se sumiranje parcijalnih suma, cime se dobija ukupna
        ## suma koja figurise u izrazu za prvi element vektora
	## "pokretnih" aritmetickih sredina.
        movhlps         xmm1, xmm0
        addps           xmm0, xmm1
        movaps          xmm1, xmm0
        shufps          xmm1, xmm1, 0x55
        addss           xmm0, xmm1

        ## Izracunava se duzina vektora "pokretnih" aritmetickih sredina
	## u registru ecx, koji ce opet biti koriscen kao brojac.
        mov             ecx, [ebp+8]
        sub             ecx, eax
        inc             ecx

next_average:
        ## Dijeljenjem izracunate sume sa redom "pokretnih" aritmetickih
        ## sredina, izracunava se vrijednost tekuceg elementa vektora
        ## "pokretnih" aritmetickih sredina, a potom se ista upisuje u
	## za to predvidjenu lokaciju u memoriji.
        movss           xmm1, xmm0
        divss           xmm1, xmm7
        movss           [edi], xmm1

        ## Azuriraju se registar koji pokazuje na tekuci element vektora
        ## "pokretnih" aritmetickih sredina, kao i brojac, i provjerava
	## se da li su izracunati svi elementi ovog vektora.
        add             edi, 4
        dec             ecx
        jecxz restore

        ## Izbacuje se prvi clan iz sume koja figurise u izrazu za
        ## tekuci element vektora "pokretnih" aritmetickih sredina, i
        ## dodaje se element datog vektora koji slijedi poslednji clan
        ## te sume, cime se dobija vrijednost sume za naredni element
        ## vektora "pokretnih" aritmetickih sredina.  Istovremeno se
        ## azuriraju registri koji pokazuju na element datog vektora
        ## koji u narednoj iteraciji treba izbaciti iz sume, odn. koga
	## treba dodati u sumu.
        subss           xmm0, [ebx]
        add             ebx, 4
        addss           xmm0, [esi]
        add             esi, 4
        jmp next_average        
        
restore:
        ## Vraca se prethodni sadrzaj u registre koprocesora i SSE
        ## registre.
        fxrstor         [esp]
        mov             esp, edx

done:
	## Epilog funkcije.
        pop             edi
        pop             esi
        pop             ebx
	leave
        ret
