.intel_syntax noprefix

.data
	two:    .int    2               # Broj 2
	
.text
.global trapeze

	## Funkcija trapeze() racuna integral funkcije sin(x) trapeznom
	## formulom. Argumenti funkcije su:
	##   [ebp+8], [ebp+12] - granice intervala integracije
	##   [ebp+16] - broj podintervala
	## Pretpostavlja se da je broj podintervala veci ili jednak
	## 2. Funkcija vraca vrijednost integrala. Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
trapeze:
        ## Prolog funkcije.
        enter	0, 0

	## Registar ecx ce se koristiti kao brojac za unutrasnje cvorove
	## (kojih ima n-1).
	mov	ecx, [ebp+16]
	dec	ecx

	## Izracunava se korak h.
	fld	dword ptr [ebp+12]	# Stek: b
	fld	dword ptr [ebp+8]	# Stek:	b, a
	fsubp				# Stek:	b-a
	fild	dword ptr [ebp+16]	# Stek:	b-a, n
	fdivp				# Stek:	(b-a)/n=h

	## Prvo ce biti izracunata suma vrijednosti funkcije u
	## unutrasnjim cvorovima intervala. Unutrasnji cvorovi ce biti
	## oznacavani u komentarima sa x[k] (pri cemu k ide od 1 do
	## n-1). Parcijalna suma u koju su uracunate vrijednosti
	## funkcije u cvorovima sa indeksima od 1 do k bice oznacena sa
	## S[k]. Parcijalna suma se inicijalizuje na 0 i stavlja na vrh
	## steka, a onda se na vrh steka stavlja i donja granica
	## intervala.
	fldz				# Stek:	h, 0=S[0]
	fld	dword ptr [ebp+8]	# Stek:	h, S[0], a=x[0]

next_node:
	## Izracunava se naredni cvor (tako sto se na tekuci doda duzina
	## koraka h). Zatim se ovaj cvor udvaja na vrhu steka,
	## izracunava se vrijednost funkcije u cvoru i dodaje se ta
	## vrijednost parcijalnoj sumi.
	fadd	st, st(2)	# Stek:	h, S[k-1], x[k-1]+h=x[k]
	fld	st		# Stek:	h, S[k-1], x[k], x[k]
	fsin			# Stek:	h, S[k-1], x[k], sin(x[k])
	faddp	st(2), st	# Stek:	h, S[k-1]+sin(x[k])=S[k], x[k]
	loop next_node

	## Odbacuje se vrijednost koja se nalazi na vrhu steka.
	ffree	st		# Stek: h, S[n-1], invalidna vrijednost
	fincstp			# Stek: h, S[n-1]

	## Mnozi se parcijalna suma sa 2.
	fimul	dword ptr two	# Stek:	h, 2*S[n-1]

	## Dodaju se na parcijalnu sumu vrijednosti funkcije u granicama
	## intervala, cime se suma kompletira.
	fld	dword ptr [ebp+8]	# Stek:	h, 2*S[n-1], a
	fsin				# Stek: h, 2*S[n-1], sin(a)
	faddp				# Stek: h, sin(a)+2*S[n-1],
	fld	dword ptr [ebp+12]	# Stek: h, sin(a)+2*S[n-1], b
	fsin				# Stek: h, sin(a)+2*S[n-1], sin(b)
	faddp				# Stek: h, sin(a)+2*S[n-1]+sin(b)

	## Mnozi se suma sa h/2, cime se kompletira izracunavanje.
	fmulp			# Stek: h*(sin(a)+2*S[n-1]+sin(b))
	fidiv	dword ptr two	# Stek: h/2*(sin(a)+2*S[n-1]+sin(b))
	
        ## Epilog funkcije.
        leave
        ret
