.intel_syntax noprefix
			
.text
.global simplify

        ## Funkcija simplify() vrsi skracivanje razlomka datog preko
	## brojioca i imenioca.  Argumenti funkcije su:
        ##   [ebp+8] - adresa na kojoj se nalazi brojilac
        ##   [ebp+12] - adresa na kojoj se nalazi imenilac
        ## Funkcija ne vraca nikakvu vrijednost.  Smatra se da su
        ## brojilac i imenilac brojevi razliciti od 0.  Funkcija se u
        ## potpunosti podvrgava C konvencijama pozivanja.
simplify:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Procedura prvo racuna najveci zajednicki djelilac apsolutnih
	## vrijednosti brojioca i imenioca, da bi potom podijelila ove
	## dvije vrijednosti sa istim.  U registrima esi i edi cuvaju se
	## originalne vrijednosti brojioca i imenioca, a u registrima ebx i
	## ecx izracunavaju se njihove apsolutne vrijednosti potrebne za
	## racunanje najveceg zajednickog djelioca.

        ## Brojilac se ucitava u registar esi i odredjuje se njegova
	## apsolutna vrijednost u registru ebx.
	mov	eax, [ebp+8]
        mov     esi, [eax]
        mov     ebx, esi
        cmp     ebx, 0
        jge load_denominator
        neg     ebx

load_denominator:
        ## Imenilac se ucitava u registar edi i odredjuje se njegova
	## apsolutna vrijednost u registru ecx.  Ukoliko je imenilac
	## negativan, mijenja se znak i brojioca i imenioca.
        mov	eax, [ebp+12]
        mov     edi, [eax]
        mov     ecx, edi
        cmp     ecx, 0
        jge order_numbers
        neg     ecx
        neg     esi
        neg     edi
        
order_numbers:
        ## U registar ebx se prebacuje veci od dva data broja, u
	## registru ecx ostaje manji od njih.
        cmp     ebx, ecx
        jge next_iteration
        xchg    ebx, ecx

next_iteration:
	## Dijeli se veci broj sa manjim.
	xor	edx, edx
	mov	eax, ebx
	div	ecx

	## Ukoliko je ostatak dijeljenja jednak 0, manji broj je najveci
	## zajednicki djelilac i iterativni postupak se prekida,...
	cmp	edx, 0
	je gcd_found

	## ...a u suprotnom manji broj ide na mjesto veceg, a ostatak
	## dijeljenja na mjesto manjeg i nastavlja se sa iteracijama.
	mov	ebx, ecx
	mov	ecx, edx
	jmp next_iteration

gcd_found:
        ## Najveci zajednicki djelilac se u ovom trenutku nalazi u
	## registru ecx. Smjesta se brojilac u registar eax i dijeli sa
        ## najvecim zajednicim djeliocem, a potom se svedeni brojilac
	## upisuje natrag u memoriju.
	mov	eax, esi
        cdq
        idiv    ecx
        mov     ebx, [ebp+8]
        mov     [ebx], eax

	## Smjesta se imenilac u registar eax i dijeli sa najvecim
        ## zajednickim djeliocem, a potom se svedeni imenilac upisuje
	## natrag u memoriju.
        mov	eax, edi
        cdq
        idiv    ecx
        mov     ebx, [ebp+12]
        mov     [ebx], eax
	
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
