.intel_syntax noprefix

.data
ten:    .int    10              # Broj 10.

.bss
.lcomm  value, 4                # 

.text
.global stof

	## Funkcija stof() parsira realan broj iz datog
	## stringa. Argumenti funkcije su:
	##   [ebp+8] - string koji se parsira
	##   [ebp+12] - lokacija na koju treba upisati parsirani broj
        ## Funkcija vraca 1 ako parsiranje prodje u redu, odnosno 0 u
	## suprotnom. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
stof:
        ## Prolog funkcije.
        enter	0, 0
        push    esi

        ## Parsiranje je implementirano pomocu konacnog automata sa 4
	## stanja:  cifra ispred decimalne tacke, decimalna tacka ispred
	## koje nema cifara, cifra iza decimalne tacke, decimalna tacka
	## ispred koje je bilo cifara.  Stanja automata su u kodu
	## predstavljena odgovarajucim labelama.

        ## Registar esi pokazuje na tekuci karakter u stringu.
        cld
        mov     esi, [ebp+8]

        ## Inicijalizuje se stek.  Na steku ce se nalaziti broj koji
	## predstavlja parsirani dio stringa, pri cemu ce taj broj biti
        ## predstavljen u vidu razlomka sa cjelobrojnim brojiocem (u
	## oznaci num) i imeniocem (u oznaci den), gdje je imenilac
	## stepen broja 10.
        fld1                            # Stek: 1=den
        fldz                            # Stek: den, 0=num

        ## Poziva se funkcija next_token() da odredi naredni token.
        lodsb
        push    eax
        call next_token
        add     esp, 4

        ## Prelazi se u odgovarajuce stanje automata.
        cmp     eax, 1
        je digit_before_dot
        cmp     eax, 2
        je dot_before_digits
        jmp error

digit_before_dot:
        ## Zaracunava se tekuca cifra (u oznaci d).
        fimul   dword ptr [ten]         # Stek: den, 10*num
        fiadd   dword ptr [value]       # Stek: den, num*10+d=num

        ## Poziva se funkcija next_token() da odredi naredni token.
        lodsb
        push    eax
        call next_token
        add esp, 4

        ## Prelazi se u odgovarajuce stanje automata.
        cmp     eax, 1
        je digit_before_dot
        cmp     eax, 2
        je dot_after_digits
        jmp error

dot_before_digits:
        ## Poziva se funkcija next_token() da odredi naredni token.
        lodsb
        push    eax
        call next_token
        add esp, 4

        ## Prelazi se u odgovarajuce stanje automata.
        cmp     eax, 1
        je digit_after_dot
        jmp error

digit_after_dot:
        ## Zaracunava se tekuca cifra (u oznaci d).
        fimul   dword ptr [ten]         # Stek: den, 10*num
        fiadd   dword ptr [value]       # Stek: den, 10*num+d=num
        fxch                            # Stek: num, den
        fimul   dword ptr [ten]         # Stek: num, 10*den=den
        fxch                            # Stek: den, num

        ## Poziva se funkcija next_token() da odredi naredni token.
        lodsb
        push    eax
        call next_token
        add esp, 4

        ## Prelazi se u odgovarajuce stanje automata.
        cmp     eax, 0
        je ok
        cmp     eax, 1
        je digit_after_dot
        jmp error
        
dot_after_digits:
        ## Poziva se funkcija next_token() da odredi naredni token.
        lodsb
        push    eax
        call next_token
        add esp, 4

        ## Prelazi se u odgovarajuce stanje automata.
        cmp     eax, 0
        je ok
        cmp     eax, 1
        je digit_after_dot
        jmp error

ok:
        ## Izracunava se trazeni broj i smjesta na datu lokaciju.
        fdivrp                       # Stek: num/den
        mov     eax, [ebp+12]
        fstp    dword ptr [eax]

        ## Rezultat se postavlja na 1.
        mov     eax, 1
        jmp finish
        
error:
        ## Rezultat se postavlja na 0.
        xor     eax, eax

finish:
        ## Epilog funkcije.
        pop     esi
        leave
        ret

      	## Funkcija next_token() prepoznaje naredni token na osnovu
	## datog karaktera. Argumenti funkcije su:
	##   [ebp+8] - karakter koji se parsira
        ## Funkcija vraca tip tokena koji moze biti: 0 - za kraj broja,
	## 1 - za cifru, 2 - za decimalnu tacku u -1 inace.  Ukoliko je
	## u pitanju cifra, funkcija stavlja broj koji odgovara cifri u
	## globalnu promjenljivu value.  Funkcija se u potpunosti 
        ## podvrgava C konvencijama pozivanja.
next_token:
        ## Prolog funkcije.
        enter   0, 0

        ## Argument funkcije se prebacuje u eax registar.
        mov     eax, [ebp+8]

        ## Provjerava se da li je u pitanju oznaka za kraj stringa.
        cmp     al, 0
        jne test_dot
        xor     eax, eax
        jmp token_scanned
        
test_dot:
        ## Provjerava se da li je u pitanju decimalna tacka.
        cmp     al, '.'
        jne test_digit
        mov     eax, 2
        jmp token_scanned
        
test_digit:
        ## Provjerava se da li je u pitanju cifra.
        sub     al, '0'
        cmp     al, 9
        ja invalid_token
        xor     edx, edx
        mov     dl, al
        mov     [value], edx
        mov     eax, 1
        jmp token_scanned
        
invalid_token:
        ## Karakter nije validan token.
        mov     eax, -1

token_scanned:
        ## Epilog funkcije.
        leave
        ret
