.intel_syntax noprefix
                
.text
.global collatz

        ## Funkcija collatz() odredjuje broj iz datog skupa brojeva sa
	## pocetka skupa cijelih pozitivnih brojeva koji ima najduzu
	## Collatz-ovu sekvencu, kao i duzinu te sekvence. Argumenti
	## funkcije su:
        ##   [ebp+8] - velicina skupa brojeva koji se ispituju
        ##   [ebp+12] - lokacija na koju treba upisati broj iz gornjeg
        ##         	skupa sa najduzom Collatz-ovom sekvencom
        ##   [ebp+16] - lokacija na koju treba upisati duzinu sekvence
        ##              za gornji broj
        ## Pretpostavlja se da je prvi argument cijeli pozitivan broj.
	## Funkcija se u potpunosti podvrgava C konvencijama pozivanja.
collatz:
        ## Prolog funkcije.
        enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Broj sa najduzom sekvencom iz datog skupa ce biti cuvan u
	## registru eax, a duzina njegove sekvence u registru ebx.
        mov     eax, 1
        mov     ebx, 1

        ## Registar ecx ce sluziti kao brojac u petlji i sadrzace broj
	## cija se sekvenca izracunava u tekucoj iteraciji.  Registar
	## edx ce sadrzati velicinu skupa brojeva koji se ispituju.
        mov     ecx, 2
        mov     edx, [ebp+8]

outer_loop:
        ## Provjerava se da li su ispitani svi brojevi iz datog skupa
	## brojeva.
        cmp     ecx, edx
        ja finish

        ## Registar esi ce sadrzati tekucu vrijednost u sekvenci za broj
	## cija se sekvenca trenutno izracunava, a registar edi ce
	## cuvati tekucu duzinu te sekvence.
        mov     esi, ecx
        mov     edi, 1

inner_loop:
        ## Provjerava se da li je sekvenca zavrsena.
        cmp     esi, 1
        je sequence_done

        ## Ispituje se sa li je tekuca vrijednost u sekvenci paran ili
	## neparan broj.
        test    esi, 1
        jz even

        ## Izracunava se naredna vrijednost u sekvenci ako je tekuca
	## vrijednost neparan broj.  Treba uociti kako se izbjegava skupa
	## operacija mnozenja.
        push    eax
        mov     eax, esi
        shl     esi, 1
        add     esi, eax
        add     esi, 1
        pop     eax
        jmp sequence_next

even:
        ## Izracunava se naredna vrijednost u sekvenci ako je tekuca
	## vrijednost paran broj.
        shr     esi, 1

sequence_next:
        ## Prelazi se na naredni korak u sekvenci.
        inc edi
        jmp inner_loop

sequence_done:
        ## Ispituje se da li je duzina sekvence tekuceg broja veca od
	## dosad izracunate najduze sekvence; ako je to slucaj, pamte se
	## tekuci broj i duzina njegove sekvence.
        cmp     edi, ebx
        jna number_next
        mov     eax, ecx
        mov     ebx, edi

number_next:
        ## Prelazi se na naredni broj iz datog skupa brojeva.
        inc     ecx
        jmp outer_loop

finish:
        ## Upisuju se broj sa najduzom sekvencom i duzina te sekvence u
	## zadate memorijske lokacije.
        mov     edi, [ebp+12]
        mov     [edi], eax
        mov     edi, [ebp+16]
        mov     [edi], ebx
        
        ## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
