.intel_syntax noprefix
        
.text
.global haar

	## Funkcija haar() racuna Haar-ovu DWT datog vektora realnih
	## brojeva.  Argumenti funkcije su:
	##   [ebp+8] - duzina vektora, izrazena kao stepen dvojke
	##   [ebp+12] - adresa prvog elementa vektora
        ## Pretpostavlja se da je prvi argument u intervalu [1,16].
	## Funkcija je destruktivna, odnosno upisuje rezultat na mjesto
        ## elemenata polaznog vektora.  Funkcija se u potpunosti
	## podvrgava C konvencijama pozivanja.
haar:
        ## Prolog funkcije.
        enter	0, 0
        push    ebx
        push    esi
        push    edi

        ## Postavlja se direction fleg.
        cld

        ## Izracunava se duzina vektora i pamti u registru ecx, koji ce
	## biti koriscen kao brojac u spoljasnoj petlji.
        mov     eax, 1
        mov     ecx, [ebp+8]
        shl     eax, cl
        mov     ecx, eax

        ## Alocira se na steku memorija iste velicine kao i vektor, za
	## smjestaj rezultata u svakom koraku transformacije.  Velicina
	## alocirane memorija biva zapamcena u registru ebx.
        mov     ebx, ecx
        shl     ebx, 2
        sub     esp, ebx

        ## Registar esi ce sadrzati adresu prvog elementa polaznog
	## vektora, a registar edi adresu prvog elementa rezultujuceg
	## vektora.
        mov     esi, [ebp+12]
        mov     edi, esp

next_iteration:
        ## Provjerava se da li se doslo do kraja spoljasnje petlje.
        cmp     ecx, 1
        jbe finish

        ## Registar ecx ce biti koriscen kao brojac i u unutrasnjoj
	## petlji.  Brojac ce se kretati od 0 do polovine broja elemenata
	## vektora koji ucestvuju u transformaciji u tekucoj iteraciji
	## spoljasnje petlje (ovaj broj ce se cuvati u registru edx).
        shr     ecx, 1
        push    ecx
        mov     edx, ecx
        xor     ecx, ecx

next_pair:
        ## Provjerava se da li se doslo do kraja unutrasnje petlje.
        cmp     ecx, edx
        jae copy_results

        ## Za dati par elemenata x[2*k] i x[2*k+1] racunaju se nove
        ## vrijednosti elemenata x[k] i x[k+m/2], gdje je m broj
        ## elemenata koji ucestvuju u izracunavanju u tekucoj iteraciji
	## spoljasnje petlje (odnosno broj koji se cuva u registru edx).
        fld     [esi+8*ecx]     # Stek: x[2*k]
        fld     st              # Stek: x[2*k], x[2*k]
        fadd    [esi+8*ecx+4]   # Stek: x[2*k], x[2*k]+x[2*k+1]
        fld1                    # Stek: x[2*k], x[2*k]+x[2*k+1], 1
        fchs                    # Stek: x[2*k], x[2*k]+x[2*k+1], -1
        fxch                    # Stek: x[2*k], -1, x[2*k]+x[2*k+1]
        fscale                  # Stek: x[2*k], -1, x[2*k]+x[2*k+1]*2^(-1)=(x[2*k]+x[2*k+1])/2
        fxch                    # Stek: x[2*k], (x[2*k]+x[2*k+1])/2, -1
        fcomp                   # Stek: x[2*k], (x[2*k]+x[2*k+1])/2
        fst     [edi+4*ecx]     # x[k] <- (x[2*k]+x[2*k+1])/2, Stek: x[2*k], (x[2*k]+x[2*k+1])/2
        fsubp                   # Stek: x[2*k]-(x[2*k]+x[2*k+1])/2=(x[2*k]-x[2*k+1])/2
        add     ecx, edx
        fstp    [edi+4*ecx]     # x[k+m/2] <- (x[2*k]-x[2*k+1])/2, Stek: 
        sub     ecx, edx

        ## Prelazi se na narednu iteraciju unutrasnje petlje.
        inc     ecx
        jmp next_pair

copy_results:
        ## Kopiraju se izmijenje vrijednosti polaznog vektora natrag u
	## polazni vektor.
        shl     edx, 1
        mov     ecx, edx
        push    esi
        push    edi
        xchg    esi, edi
        rep movsd
        pop     esi
        pop     edi

        ## Prelazi se na narednu iteraciju spoljasnje petlje.
        pop     ecx
        jmp     next_iteration

finish:        
        ## Epilog funkcije.
        add     esp, ebx
        pop     edi
        pop     esi
        pop     ebx
        leave
        ret
