.intel_syntax noprefix

.data
C:      .float  0.38196601
        
.text
.global golden_section_search

	## Funkcija golden_section_search() odredjuje minimum date
	## funkcije na datom intervalu.  Argumenti funkcije su:
        ##   [ebp+8] - lijeva granica intervala a
        ##   [ebp+12] - tacka na intervalu b
        ##   [ebp+16] - desna granica intervala c
        ##   [ebp+20] - trazena preciznost
        ##   [ebp+24] - adresa na koju treba upisati tacku minimuma
        ##   [ebp+24] - adresa na koju treba upisati vrijednost
	##              funkcije u tacki minimuma
        ## Pretpostavlja se da je a<b<c, te da je f(b)<f(a) i f(b)<f(c),
        ## kao i da funkcija nije sinularna na datom intervalu; takodje
        ## se pretpostavlja da je preciznost veca od 0. Funkcija se u
	## potpunosti podvrgava C konvencijama pozivanja.
golden_section_search:
        ## Prolog funkcije.
	enter   0, 0
        
        ## Na stek se stavlja preciznost, kao i tacke a, b i c, a 
        ## potom se izracunava vrijednost date funkcije u tacki b.  Stek
        ## ce nakon svake iteracije imati ovakav oblik; obzirom da se
        ## ove tacke mijenjaju tokom iteracija, njihove tekuce
	## vrijednosti ce biti oznacene indeksima.
        fld     dword ptr [ebp+20]      # Stek: epsilon
        fld     dword ptr [ebp+8]       # Stek: epsilon, a=a[0]=a[i]
        fld     dword ptr [ebp+12]      # Stek: epsilon, a[i], b=b[0]=b[i]
        fld     dword ptr [ebp+16]      # Stek: epsilon, a[i], b[i], c=c[0]=c[i]
        fld     st(1)                   # Stek: epsilon, a[i], b[i], c[i], b[i]
        fsin                            # Stek: epsilon, a[i], b[i], c[i], sin(b[i])
        fmul    st, st(2)               # Stek: epsilon, a[i], b[i], c[i], b[i]*sin(b[i])=f(b[i])

again:
        ## Izracunava se duzina intervala [a,c] i ako je ista postala
        ## manja od trazene preciznosti, prekida se iterativni postupak.
        fld     st(1)                   # Stek: epsilon, a[i], b[i], c[i], f(b[i]), c[i]
        fsub    st, st(4)               # Stek: epsilon, a[i], b[i], c[i], f(b[i]), c[i]-a[i]
        fcomip  st, st(5)               # Stek: epsilon, a[i], b[i], c[i], f(b[i])
        jbe done

        ## Izracunavaju se i porede duzine intervala [a,b] i [b,c].
        fld     st(2)                   # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]
        fsub    st(4)                   # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]-a[i]
        fld     st(2)                   # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]-a[i], c[i]
        fsub    st(4)                   # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]-a[i], c[i]-b[i]
        fcomi   st, st(1)
        jbe left

        ## Ako je interval [b,c] duzi, nova tacka x se izracunava na
        ## ovom intervalu, a potom se izracunava vrijednost date
        ## funkcije u istoj.  Potom se tackama b i x dodjeljuju oznake
        ## x0 i x1 tako da je x0<=x1.
        fxch                            # Stek: epsilon, a[i], b[i], c[i], f(b[i]), c[i]-b[i], b[i]-a[i]
        fcomp   st                      # Stek: epsilon, a[i], b[i], c[i], f(b[i]), c[i]-b[i]
        fmul    dword ptr [C]           # Stek: epsilon, a[i], b[i], c[i], f(b[i]), C*(c[i]-b[i])
        fadd    st, st(3)               # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]+C*(c[i]-b[i])=x[i]
        fld     st                      # Stek: epsilon, a[i], b[i], c[i], f(b[i]), x[i], x[i]
        fsin                            # Stek: epsilon, a[i], b[i], c[i], f(b[i]), x[i], sin(x[i])
        fmul    st, st(1)               # Stek: epsilon, a[i], b[i], c[i], f(b[i]), x[i], x[i]*sin(x[i])=f(x[i])
                                        # Stek: epsilon, a[i], b[i]=x0[i], c[i], f(b[i])=f(x0[i]), x[i]=x1[i], f(x[i])=f(x1[i])
        jmp minimum
        
left:
        ## Ako je interval [a,b] duzi, nova tacka x se izracunava na
        ## ovom intervalu, a potom se izracunava vrijednost date
        ## funkcije u istoj.  Potom se tackama b i x dodjeljuju oznake
        ## x0 i x1 tako da je x0<=x1.
        fcomp   st                      # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]-a[i]
        fmul    dword ptr [C]           # Stek: epsilon, a[i], b[i], c[i], f(b[i]), C*(b[i]-a[i])
        fsubr   st, st(3)               # Stek: epsilon, a[i], b[i], c[i], f(b[i]), b[i]-C*(b[i]-a[i])=x[i]
        fxch    st(3)                   # Stek: epsilon, a[i], x[i], c[i], f(b[i]), b[i]
        fld     st(3)                   # Stek: epsilon, a[i], x[i], c[i], f(b[i]), b[i], x[i]
        fsin                            # Stek: epsilon, a[i], x[i], c[i], f(b[i]), b[i], sin(x[i])
        fmul    st, st(4)               # Stek: epsilon, a[i], x[i], c[i], f(b[i]), b[i], x[i]*sin(x[i])=f(x[i])
        fxch    st(2)                   # Stek: epsilon, a[i], x[i], c[i], f(x[i]), b[i], f(b[i])
                                        # Stek: epsilon, a[i], x[i]=x0[i], c[i], f(x[i])=f(x0[i]), b[i]=x1[i], f(b[i])=f(x1[i])
        
minimum:
        ## Porede se vrijednosti date funkcije u tackama x1 i x0.
        fcomi   st, st(2)
        jbe right

        ## Ako je vrijednost date funkcije manja u tacki x0, novi tekuci
        ## interval postaje interval [a,x1], sa tackom x0 kao novom
	## tackom na intervalu.
        fcomp   st                      # Stek: epsilon, a[i], x0[i], c[i], f(x0[i]), x1[i]
        fstp    st(2)                   # Stek: epsilon, a[i]=a[i+1], x0[i]=b[i+1], x1[i]=c[i+1], f(x0[i])=f(b[i+1])
        jmp again
        
right:
        ## Ako je vrijednost date funkcije manja u tacki x1, novi tekuci
        ## interval postaje interval [x0,c], sa tackom x1 kao novom
	## tackom na intervalu.
        fstp    st(2)                   # Stek: epsilon, a[i], x0[i], c[i], f(x1[i]), x1[i]
        fxch    st(3)                   # Stek: epsilon, a[i], x1[i], c[i], f(x1[i]), x0[i]
        fxch    st(4)                   # Stek: epsilon, x0[i], x1[i], c[i], f(x1[i]), a[i]
        fcomp   st                      # Stek: epsilon, x0[i]=a[i+1], x1[i]=b[i+1], c[i]=c[i+1], f(x1[i])=f(b[i+1])
        jmp again

done:
        ## Upisuju se izracunate vrijednosti u za to predvidjene
	## lokacije u memoriji.
        mov     edx, [ebp+28]
        fstp    dword ptr [edx]         # Stek: epsilon, a[i], b[i], c[i]
        fcomp   st                      # Stek: epsilon, a[i], b[i]
        mov     edx, [ebp+24]
        fstp    dword ptr [edx]         # Stek: epsilon, a[i]
        fcompp                          # Stek:
        
	## Epilog funkcije.
	leave
        ret
