.intel_syntax noprefix
        
.text
.global product

	## Funkcija product() izracunava proizvod datih polinoma.
	## Argumenti funkcije su:
        ##   [ebp+8] - stepen prvog polinoma
        ##   [ebp+12] - adresa prvog elementa polja koeficijenata prvog
	##              polinoma
        ##   [ebp+16] - stepen drugog polinoma
        ##   [ebp+20] - adresa prvog elementa polja koeficijenata drugog
        ##              polinoma
        ##   [ebp+24] - adresa prvog elementa polja koeficijenata
        ##              polinoma proizvoda
        ## Pretpostavka je da su stepeni polinoma veci ili jednaki
        ## 0. Funkcija se u potpunosti podvrgava C konvencijama
	## pozivanja.
product:
        ## Prolog funkcije.
	enter   0, 0
        push    ebx
        push    esi
        push    edi

        ## Postavljaju se svi elementi polinoma proizvoda na 0.
        ## Registar edi sadrzi adresu prvog elementa u polju
        ## koeficijenata polinoma proizvoda.  Registar edx sadrzi indeks
	## tekuceg elementa u tom polju.
        mov     edi, [ebp+24]
        mov     edx, [ebp+8]
        add     edx, [ebp+16]
        fldz                    # Stek: 0
next_coeff:
        fst     [edi+4*edx]     # Stek: 0
        dec     edx
        cmp     edx, 0
        jge next_coeff
        ffree   st              # Stek: invalidna vrijednost
        fincstp                 # Stek:

        ## Obavlja se mnozenje datih polinoma.  Registar ebx sadrzi
        ## adresu prvog elementa u polju koeficijenata prvog polinoma.
        ## Registar eax sadrzi indeks tekuceg elementa u tom polju.
        ## Registar esi sadrzi adresu prvog elementa u polju
        ## koeficijenata drugog polinoma.  Registar ecx sadrzi indeks
	## tekuceg elementa u tom polju.
        mov     ebx, [ebp+12]
        mov     eax, [ebp+8]
        mov     esi, [ebp+20]
        mov     ecx, [ebp+16]
next_coeff_0:
        ## Ucitava se na stek tekuci (u oznaci i-ti) element prvog
	## polinoma.
        fld     [ebx+4*eax]     # Stek: p0[i]

        ## Registar edx se inicijalizuje na indeks tekuceg elementa
	## polinoma proizvoda.
        mov     edx, eax
        add     edx, ecx

        ## Mnozi se tekuci element prvog polinoma sa svakim (u oznaci
        ## j-tim) elementom drugog polinoma; dobijeni proizvod se dodaje
        ## vrijednosti odgovarajuceg elementa polinoma proizvoda.
	## Stepen drugog polinoma se tokom iteracija cuva na steku.
        push    ecx
next_coeff_1:
        ## Ucitava ste tekuci element drugog polinoma i mnozi sa tekucim
	## elementom prvog polinoma, a zatim se azurira tekuci element
	## polinoma proizvoda.
        fld     [esi+4*ecx]     # Stek: p0[i], p1[j]
        fmul    st, st(1)       # Stek: p0[i], p0[i]*p1[j]
        fadd    [edi+4*edx]     # Stek: p0[i], p[i+j]+p0[i]*p1[j]
        fstp    [edi+4*edx]     # Stek: p0[i]

        ## Prelazi se na naredni element drugog polinoma
        dec     edx
        dec     ecx
        cmp     ecx, 0
        jge next_coeff_1
        pop     ecx

        ## Skida se sa steka tekuci element prvog polinoma.
        ffree   st              # Stek: invalidna vrijednost
        fincstp                 # Stek:

        ## Prelazi se na naredni element prvog polinoma.
        dec     eax
        cmp     eax, 0
        jge next_coeff_0
        
	## Epilog funkcije.
        pop     edi
        pop     esi
        pop     ebx
	leave
        ret
